@@ -43,7 +43,6 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <poll.h>
-#include <sched.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -99,7 +98,6 @@ strong_alias(env_unset_environment,	slurm_env_unset_environment);
 typedef struct {
 	char *cmdstr;
 	int *fildes;
-	int mode;
 	bool perform_mount;
 	int rlimit;
 	char **tmp_env;
@@ -463,12 +461,22 @@ int setup_env(env_t *env, bool preserve_env)
 		} else
 			str_bind_type = xstrdup("");
 
-		if (setenvf(&env->env, "SLURM_CPU_BIND", "%s", str_bind)) {
+		/*
+		 * Don't set SLURM_CPU_BIND or SLURM_CPU_BIND_LIST in the
+		 * environment if they are too long. These are informational
+		 * for the user and don't merit an error in the log if they
+		 * can't be set, so avoid calling setenvf().
+		 */
+		if (strlen(str_bind) >= MAX_ENV_STRLEN)
+			debug("Not setting SLURM_CPU_BIND: value too long");
+		else if (setenvf(&env->env, "SLURM_CPU_BIND", "%s", str_bind)) {
 			error("Unable to set SLURM_CPU_BIND");
 			rc = SLURM_ERROR;
 		}
-		if (setenvf(&env->env, "SLURM_CPU_BIND_LIST", "%s",
-			    str_bind_list)) {
+		if (strlen(str_bind_list) >= MAX_ENV_STRLEN)
+			debug("Not setting SLURM_CPU_BIND_LIST: value too long");
+		else if (setenvf(&env->env, "SLURM_CPU_BIND_LIST", "%s",
+				 str_bind_list)) {
 			error("Unable to set SLURM_CPU_BIND_LIST");
 			rc = SLURM_ERROR;
 		}
@@ -890,7 +898,7 @@ extern char *uint16_array_to_str(int array_len, const uint16_t *array)
 {
 	int i;
 	int previous = 0;
-	char *sep = ",";  /* seperator */
+	char *sep = ",";  /* separator */
 	char *str = xstrdup("");
 
 	if (array == NULL)
@@ -934,7 +942,7 @@ extern char *uint32_compressed_to_str(uint32_t array_len,
 				      const uint32_t *array_reps)
 {
 	int i;
-	char *sep = ","; /* seperator */
+	char *sep = ","; /* separator */
 	char *str = xstrdup("");
 
 	if (!array || !array_reps)
@@ -989,7 +997,7 @@ extern int env_array_for_job(char ***dest,
 	char *dist = NULL;
 	char *key, *value;
 	slurm_step_layout_t *step_layout = NULL;
-	int i, rc = SLURM_SUCCESS;
+	int i, new_cpt, rc = SLURM_SUCCESS;
 	slurm_step_layout_req_t step_layout_req;
 	uint16_t cpus_per_task_array[1];
 	uint32_t cpus_task_reps[1];
@@ -1192,11 +1200,23 @@ extern int env_array_for_job(char ***dest,
 					    het_job_offset,
 					    "%d", desc->num_tasks);
 	}
-	if (desc->bitflags & JOB_CPUS_SET) {
+
+	new_cpt = slurm_opt_get_tres_per_task_cpu_cnt(alloc->tres_per_task);
+	if (new_cpt) {
+		env_array_overwrite_het_fmt(dest, "SLURM_CPUS_PER_TASK",
+					    het_job_offset, "%d", new_cpt);
+	} else if (desc->bitflags & JOB_CPUS_SET) {
 		env_array_overwrite_het_fmt(dest, "SLURM_CPUS_PER_TASK",
 					    het_job_offset, "%d",
 					     desc->cpus_per_task);
 	}
+
+	if (alloc->tres_per_task) {
+		env_array_overwrite_het_fmt(dest, "SLURM_TRES_PER_TASK",
+					    het_job_offset, "%s",
+					    alloc->tres_per_task);
+	}
+
 	if (desc->ntasks_per_node && (desc->ntasks_per_node != NO_VAL16)) {
 		env_array_overwrite_het_fmt(dest, "SLURM_NTASKS_PER_NODE",
 					    het_job_offset, "%d",
@@ -1241,7 +1261,6 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	slurm_step_layout_req_t step_layout_req;
 	uint16_t cpus_per_task_array[1];
 	uint32_t cpus_task_reps[1];
-	char *tres_per_task = NULL;
 
 	if (!batch)
 		return SLURM_ERROR;
@@ -1324,15 +1343,9 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	if (getenvp(*dest, "SLURM_CPUS_PER_TASK"))
 		env_array_overwrite_fmt(dest, "SLURM_CPUS_PER_TASK", "%u",
 					cpus_per_task);
-	if ((tres_per_task = getenvp(*dest, "SLURM_TRES_PER_TASK")) &&
-	    xstrstr(tres_per_task, "cpu=")) {
-		char *new_tres_per_task = xstrdup(tres_per_task);
-		slurm_option_update_tres_per_task(cpus_per_task, "cpu",
-						  &new_tres_per_task);
+	if (batch->tres_per_task)
 		env_array_overwrite_fmt(dest, "SLURM_TRES_PER_TASK", "%s",
-					new_tres_per_task);
-		xfree(new_tres_per_task);
-	}
+					batch->tres_per_task);
 
 	if (step_layout_req.num_tasks) {
 		env_array_overwrite_fmt(dest, "SLURM_NTASKS", "%u",
@@ -1519,7 +1532,7 @@ env_array_for_step(char ***dest,
 }
 
 /*
- * Enviroment variables set elsewhere
+ * Environment variables set elsewhere
  * ----------------------------------
  *
  * Set by slurmstepd:
@@ -1876,23 +1889,6 @@ void env_array_merge_slurm_spank(char ***dest_array, const char **src_array)
 	xfree(value);
 }
 
-/*
- * Strip out trailing carriage returns and newlines
- */
-static void _strip_cr_nl(char *line)
-{
-	int len = strlen(line);
-	char *ptr;
-
-	for (ptr = line+len-1; ptr >= line; ptr--) {
-		if (*ptr=='\r' || *ptr=='\n') {
-			*ptr = '\0';
-		} else {
-			return;
-		}
-	}
-}
-
 /* Return the net count of curly brackets in a string
  * '{' adds one and '}' subtracts one (zero means it is balanced).
  * Special case: return -1 if no open brackets are found */
@@ -1915,11 +1911,6 @@ static int _bracket_cnt(char *value)
  * via the --export-file option in sbatch. The NAME=value entries must
  * be NULL separated to support special characters in the environment
  * definitions.
- *
- * (Note: This is being added to a minor release. For the
- * next major release, it might be a consideration to merge
- * this functionality with that of load_env_cache and update
- * env_cache_builder to use the NULL character.)
  */
 char **env_array_from_file(const char *fname)
 {
@@ -2043,65 +2034,6 @@ rwfail:
 	return rc;
 }
 
-/*
- * Load user environment from a cache file located in
- * <state_save_location>/env_username
- */
-static char **_load_env_cache(const char *username)
-{
-	char fname[PATH_MAX];
-	char *line, name[256], *value;
-	char **env = NULL;
-	FILE *fp;
-	int i;
-
-	i = snprintf(fname, sizeof(fname), "%s/env_cache/%s",
-		     slurm_conf.state_save_location, username);
-	if (i < 0) {
-		error("Environment cache filename overflow");
-		return NULL;
-	}
-	if (!(fp = fopen(fname, "r"))) {
-		error("Could not open user environment cache at %s: %m",
-			fname);
-		return NULL;
-	}
-
-	verbose("Getting cached environment variables at %s", fname);
-	env = env_array_create();
-	line  = xmalloc(ENV_BUFSIZE);
-	value = xmalloc(ENV_BUFSIZE);
-	while (1) {
-		if (!fgets(line, ENV_BUFSIZE, fp))
-			break;
-		_strip_cr_nl(line);
-		if (_env_array_entry_splitter(line, name, sizeof(name),
-					      value, ENV_BUFSIZE) &&
-		    (!_discard_env(name, value))) {
-			if (value[0] == '(') {
-				/* This is a bash function.
-				 * It may span multiple lines */
-				while (_bracket_cnt(value) > 0) {
-					if (!fgets(line, ENV_BUFSIZE, fp))
-						break;
-					_strip_cr_nl(line);
-					if ((strlen(value) + strlen(line)) >
-					    (ENV_BUFSIZE - 2))
-						break;
-					strcat(value, "\n");
-					strcat(value, line);
-				}
-			}
-			env_array_overwrite(&env, name, value);
-		}
-	}
-	xfree(line);
-	xfree(value);
-
-	fclose(fp);
-	return env;
-}
-
 static int _child_fn(void *arg)
 {
 	char **tmp_env = NULL;
@@ -2140,17 +2072,11 @@ static int _child_fn(void *arg)
 	while (fd < child_args->rlimit)
 		close(fd++);
 
-	if (child_args->mode == 1)
-		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
-	else if (child_args->mode == 2)
-		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
-	else {	/* Default system configuration */
 #ifdef LOAD_ENV_NO_LOGIN
-		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
+	execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
 #else
-		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
+	execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
 #endif
-	}
 	if (devnull >= 0)	/* Avoid Coverity resource leak notification */
 		(void) close(devnull);
 
@@ -2246,32 +2172,28 @@ static bool _ns_disabled()
 
 /*
  * Return an array of strings representing the specified user's default
- * environment variables following a two-prongged approach.
- * 1. Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
+ * environment variables:
+ *    Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
  *    Depending upon the user's login scripts, this may take a very
  *    long time to complete or possibly never return
- * 2. Load the user environment from a cache file. This is used
- *    in the event that option 1 times out.  This only happens if no_cache isn't
- *    set.  If it is set then NULL will be returned if the normal load fails.
  *
- * timeout value is in seconds or zero for default (2 secs)
- * mode is 1 for short ("su <user>"), 2 for long ("su - <user>")
+ * timeout value is in seconds or zero for default (120 secs)
  * On error, returns NULL.
  *
  * NOTE: The calling process must have an effective uid of root for
  * this function to succeed.
  */
-char **env_array_user_default(const char *username, int timeout, int mode,
-			      bool no_cache)
+char **env_array_user_default(const char *username)
 {
 	char *line = NULL, *last = NULL, name[PATH_MAX], *value, *buffer;
 	char **env = NULL;
 	char *starttoken = "XXXXSLURMSTARTPARSINGHEREXXXX";
 	char *stoptoken  = "XXXXSLURMSTOPPARSINGHEREXXXXX";
-	char cmdstr[256], *env_loc = NULL;
+	char *cmdstr = NULL, *env_loc = NULL;
 	char *stepd_path = NULL;
 	int fildes[2], found, fval, len, rc, timeleft;
 	int buf_read, buf_rem, config_timeout;
+	int timeout = DEFAULT_GET_ENV_TIMEOUT;
 	pid_t child;
 	child_args_t child_args = {0};
 	struct timeval begin, now;
@@ -2284,9 +2206,6 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		return NULL;
 	}
 
-	if (!slurm_conf.get_env_timeout)	/* just read directly from cache */
-		return _load_env_cache(username);
-
 	if (stat(SUCMD, &buf))
 		fatal("Could not locate command: "SUCMD);
 	if (stat("/bin/echo", &buf))
@@ -2301,10 +2220,11 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		env_loc = "/usr/bin/env";
 	else
 		fatal("Could not locate command: env");
-	snprintf(cmdstr, sizeof(cmdstr),
-		 "/bin/echo; /bin/echo; /bin/echo; "
-		 "/bin/echo %s; %s; /bin/echo %s",
-		 starttoken, env_loc, stoptoken);
+
+	/* Construct the final command */
+	cmdstr = xstrdup_printf("/bin/echo; /bin/echo; /bin/echo; "
+				"/bin/echo %s; %s; /bin/echo %s",
+				starttoken, env_loc, stoptoken);
 	xfree(stepd_path);
 
 	if (pipe(fildes) < 0) {
@@ -2312,7 +2232,6 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		return NULL;
 	}
 
-	child_args.mode = mode;
 	child_args.fildes = fildes;
 	child_args.username = username;
 	child_args.cmdstr = cmdstr;
@@ -2357,6 +2276,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		}
 	}
 #endif
+	xfree(cmdstr);
 	close(fildes[1]);
 	if ((fval = fcntl(fildes[0], F_GETFL, 0)) < 0)
 		error("fcntl(F_GETFL) failed: %m");
@@ -2368,8 +2288,6 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	ufds.events = POLLIN;
 
 	/* Read all of the output from /bin/su into buffer */
-	if (timeout == 0)
-		timeout = slurm_conf.get_env_timeout;	/* != 0 test above */
 	found = 0;
 	buf_read = 0;
 	buffer = xmalloc(ENV_BUFSIZE);
@@ -2442,7 +2360,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to load current user environment variables");
 		xfree(buffer);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	/* First look for the start token in the output */
@@ -2459,7 +2377,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to get current user environment variables");
 		xfree(buffer);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	/* Process environment variables until we find the stop token */
@@ -2499,7 +2417,7 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	if (!found) {
 		error("Failed to get all user environment variables");
 		env_array_free(env);
-		return no_cache ? _load_env_cache(username) : NULL;
+		return NULL;
 	}
 
 	return env;
