@@ -93,6 +93,7 @@ typedef struct env_options {
 	time_t job_end_time;    /* job's end time */
 	char *job_licenses;	/* job's licenses */
 	time_t job_start_time;  /* job's start time */
+	uint16_t oom_kill_step;   /* --oom-kill-step */
 } env_t;
 
 
@@ -311,6 +312,12 @@ int env_array_overwrite_het_fmt(char ***array_ptr, const char *name,
  */
 void env_array_set_environment(char **env_array);
 
+/*
+ * load environment from specified file name.
+ */
+char **env_array_from_file(const char *filename);
+
+
 /*
  * Write environment to specified file name.
  * IN newline - if true, write 1 env variable terminated with \n
@@ -321,23 +328,17 @@ int env_array_to_file(const char *filename, const char **env_array,
 
 /*
  * Return an array of strings representing the specified user's default
- * environment variables following a two-prongged approach.
- * 1. Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
+ * environment variables:
+ *    Execute (more or less): "/bin/su - <username> -c /usr/bin/env"
  *    Depending upon the user's login scripts, this may take a very
  *    long time to complete or possibly never return
- * 2. Load the user environment from a cache file. This is used
- *    in the event that option 1 times out.  This only happens if no_cache isn't
- *    set.  If it is set then NULL will be returned if the normal load fails.
  *
- * timeout value is in seconds or zero for default (8 secs)
- * mode is 1 for short ("su <user>"), 2 for long ("su - <user>")
  * On error, returns NULL.
  *
  * NOTE: The calling process must have an effective uid of root for
  * this function to succeed.
  */
-char **env_array_user_default(const char *username, int timeout, int mode,
-			      bool no_cache);
+char **env_array_user_default(const char *username);
 
 /*
  * Return a string representation of an array of uint16_t elements.
@@ -397,4 +398,11 @@ extern char *find_quote_token(char *tmp, char *sep, char **last);
  */
 extern void env_merge_filter(slurm_opt_t *opt, job_desc_msg_t *desc);
 
+/*
+ * Set the internal SLURM_PRIO_PROCESS environment variable to support
+ * the propagation of the users nice value and the "PropagatePrioProcess"
+ * config keyword.
+ */
+extern void set_prio_process_env(void);
+
 #endif
