@@ -86,7 +86,7 @@ typedef struct _plugrack_entry {
  * entries is the list of plugrack_entry_t.
  */
 struct _plugrack {
-	List entries;
+	list_t *entries;
 	const char *major_type;
 };
 
@@ -129,7 +129,7 @@ plugrack_t *plugrack_create(const char *major_type)
 
 int plugrack_destroy(plugrack_t *rack)
 {
-	ListIterator it;
+	list_itr_t *it;
 	plugrack_entry_t *e;
 
 	if (!rack)
@@ -330,7 +330,7 @@ static bool _match_major(const char *path_name, const char *major_type)
 
 plugin_handle_t plugrack_use_by_type(plugrack_t *rack, const char *full_type)
 {
-	ListIterator it;
+	list_itr_t *it;
 	plugrack_entry_t *e;
 
 	if ((!rack) || (!full_type))
@@ -405,7 +405,7 @@ typedef struct {
 
 extern int plugrack_print_mpi_plugins(plugrack_t *rack)
 {
-	ListIterator itr;
+	list_itr_t *itr;
 	plugrack_entry_t *e = NULL;
 	char *sep, tmp[64], *pmix_vers = NULL, *comma = "";
 	int i;
@@ -517,11 +517,18 @@ extern int load_plugins(plugins_t **plugins_ptr, const char *major_type,
 
 	xassert(plugins_ptr);
 	if (!*plugins_ptr) {
+		const char *plugin_dir;
+
 		plugins = xmalloc(sizeof(*plugins));
 		plugins->magic = PLUGINS_MAGIC;
 		plugins->rack = plugrack_create(major_type);
 
-		if ((rc = plugrack_read_dir(plugins->rack, slurm_conf.plugindir))) {
+		if (slurm_conf.plugindir)
+			plugin_dir = slurm_conf.plugindir;
+		else
+			plugin_dir = default_plugin_path;
+
+		if ((rc = plugrack_read_dir(plugins->rack, plugin_dir))) {
 			error("%s: plugrack_read_dir(%s) failed: %s",
 			      __func__, slurm_conf.plugindir, slurm_strerror(rc));
 			goto cleanup;
