@@ -519,6 +519,8 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 	case 'J': /* Jobid.stepid */
 	case 'j': /* Job ID */
 		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->jobid);
+		if ((*ptr == 'J') && (job->first_step_id != SLURM_BATCH_SCRIPT))
+			xstrfmtcatat(*expanded, pos, ".%d", job->first_step_id);
 		break;
 	case 'a': /* Array task ID */
 		xstrfmtcatat(*expanded, pos, "%0*u", padding,
@@ -532,7 +534,11 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 		xstrfmtcatat(*expanded, pos, "%s", job->first_step_node);
 		break;
 	case 's': /* Stepid of the running job */
-		xstrfmtcatat(*expanded, pos, "%s", job->first_step_name);
+		if (job->first_step_id == SLURM_BATCH_SCRIPT)
+			xstrcatat(*expanded, pos, "batch");
+		else
+			xstrfmtcatat(*expanded, pos, "%0*u", padding,
+				     job->first_step_id);
 		break;
 	case 'n': /* Node id relative to current job */
 	case 't': /* Task id (rank) relative to current job */
@@ -551,11 +557,9 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 
 /*
  * Special expansion function for stdin/stdout/stderr filename patterns.
- * Fields that can potientially map to a range of values will use the first in
+ * Fields that can potentially map to a range of values will use the first in
  * that range (e.g %t is replaced by 0).
  *
- * The parser do not support steps and is only for batch jobs.
- *
  * \      If we found this symbol, don't replace anything.
  * %%     The character "%".
  * %A     Job array's master job allocation number.
@@ -589,8 +593,13 @@ extern char *expand_stdio_fields(char *stdio_path, job_std_pattern_t *job)
 	if (!stdio_path || !*stdio_path || !job)
 		return NULL;
 
-	if (stdio_path[0] != '/')
-		xstrcatat(expanded, &pos, job->work_dir);
+	if (job->work_dir && (stdio_path[0] != '/')) {
+		size_t len = strlen(job->work_dir);
+		if (job->work_dir[len - 1] == '/')
+			xstrcatat(expanded, &pos, job->work_dir);
+		else
+			xstrfmtcatat(expanded, &pos, "%s/", job->work_dir);
+	}
 
 	/*
 	 * Special case, if we find a \ it means the file has not been
