@@ -53,20 +53,25 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/uio.h>
+#include <sys/un.h>
 #include <unistd.h>
 
 #include "slurm/slurm_errno.h"
+#include "src/common/fd.h"
+#include "src/common/log.h"
+#include "src/common/net.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_protocol_api.h"
-#include "src/common/slurm_protocol_interface.h"
-#include "src/common/slurm_protocol_defs.h"
-#include "src/common/log.h"
-#include "src/common/fd.h"
+#include "src/common/slurm_protocol_socket.h"
+#include "src/common/slurm_time.h"
 #include "src/common/strlcpy.h"
+#include "src/common/util-net.h"
+#include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
-#include "src/common/xmalloc.h"
-#include "src/common/util-net.h"
+
+#include "src/interfaces/conn.h"
 
 #define PORT_RETRIES    3
 #define MIN_USER_PORT   (IPPORT_RESERVED + 1)
@@ -82,20 +87,6 @@ static int _slurm_connect(int __fd, struct sockaddr const * __addr,
  * MIDDLE LAYER MSG FUNCTIONS
  ****************************************************************/
 
-/*
- * Return time in msec since "start time"
- */
-static int _tot_wait (struct timeval *start_time)
-{
-	struct timeval end_time;
-	int msec_delay;
-
-	gettimeofday(&end_time, NULL);
-	msec_delay =   (end_time.tv_sec  - start_time->tv_sec ) * 1000;
-	msec_delay += ((end_time.tv_usec - start_time->tv_usec + 500) / 1000);
-	return msec_delay;
-}
-
 /*
  * Pick a random port number to use. Use this if the system
  * selected port can't connect. This may indicate that the
@@ -124,14 +115,14 @@ static void _sock_bind_wild(int sockfd)
 	return;
 }
 
-extern ssize_t slurm_msg_recvfrom_timeout(int fd, char **pbuf, size_t *lenp,
-					  uint32_t flags, int tmout)
+extern ssize_t slurm_msg_recvfrom_timeout(void *tls_conn, char **pbuf,
+					  size_t *lenp, int timeout)
 {
 	ssize_t  len;
 	uint32_t msglen;
 
-	len = slurm_recv_timeout( fd, (char *)&msglen,
-				  sizeof(msglen), 0, tmout );
+	len = slurm_recv_timeout(tls_conn, (char *) &msglen, sizeof(msglen),
+				 timeout);
 
 	if (len < ((ssize_t) sizeof(msglen)))
 		return SLURM_ERROR;
@@ -147,7 +138,7 @@ extern ssize_t slurm_msg_recvfrom_timeout(int fd, char **pbuf, size_t *lenp,
 	if (!(*pbuf = try_xmalloc(msglen)))
 		slurm_seterrno_ret(ENOMEM);
 
-	if (slurm_recv_timeout(fd, *pbuf, msglen, 0, tmout) != msglen) {
+	if (slurm_recv_timeout(tls_conn, *pbuf, msglen, timeout) != msglen) {
 		xfree(*pbuf);
 		*pbuf = NULL;
 		return SLURM_ERROR;
@@ -158,46 +149,19 @@ extern ssize_t slurm_msg_recvfrom_timeout(int fd, char **pbuf, size_t *lenp,
 	return (ssize_t) msglen;
 }
 
-extern ssize_t slurm_msg_sendto(int fd, char *buffer, size_t size)
+static int _writev_timeout(int fd, void *tls_conn, struct iovec *iov,
+			   int iovcnt, int timeout)
 {
-	int   len;
-	uint32_t usize;
-	SigFunc *ohandler;
-	int timeout = slurm_conf.msg_timeout * 1000;
-
-	/*
-	 *  Ignore SIGPIPE so that send can return a error code if the
-	 *    other side closes the socket
-	 */
-	ohandler = xsignal(SIGPIPE, SIG_IGN);
-
-	usize = htonl(size);
-
-	if ((len = slurm_send_timeout(
-				fd, (char *)&usize, sizeof(usize), 0,
-				timeout)) < 0)
-		goto done;
-
-	if ((len = slurm_send_timeout(fd, buffer, size, 0, timeout)) < 0)
-		goto done;
-
-
-     done:
-	xsignal(SIGPIPE, ohandler);
-	return len;
-}
-
-static int _send_timeout(int fd, char *buf, size_t size,
-			 uint32_t flags, int *timeout)
-{
-	int rc;
-	int sent = 0;
+	int tot_bytes_sent = 0;
+	size_t size = 0;
 	int fd_flags;
 	struct pollfd ufds;
 	struct timeval tstart;
-	int timeleft = *timeout;
 	char temp[2];
 
+	if (tls_conn)
+		fd = conn_g_get_fd(tls_conn);
+
 	ufds.fd     = fd;
 	ufds.events = POLLOUT;
 
@@ -206,13 +170,23 @@ static int _send_timeout(int fd, char *buf, size_t size,
 
 	gettimeofday(&tstart, NULL);
 
-	while (sent < size) {
-		timeleft = *timeout - _tot_wait(&tstart);
+	for (int i = 0; i < iovcnt; i++)
+		size += iov[i].iov_len;
+
+	while (true) {
+		ssize_t bytes_sent = 0;
+		int rc;
+		int timeleft = timeout - timeval_tot_wait(&tstart);
+
+		if (conn_g_peek(tls_conn))
+			goto ready;
+
 		if (timeleft <= 0) {
-			debug("%s at %d of %zu, timeout", __func__, sent, size);
-			slurm_seterrno(SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT);
-			sent = SLURM_ERROR;
-			goto done;
+			debug("%s at %d of %zu, timeout",
+			      __func__, tot_bytes_sent, size);
+			errno = SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT;
+			tot_bytes_sent = SLURM_ERROR;
+			break;
 		}
 
 		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
@@ -220,10 +194,11 @@ static int _send_timeout(int fd, char *buf, size_t size,
  				continue;
 			else {
 				debug("%s at %d of %zu, poll error: %s",
-				      __func__, sent, size, strerror(errno));
-				slurm_seterrno(SLURM_COMMUNICATIONS_SEND_ERROR);
-				sent = SLURM_ERROR;
-				goto done;
+				      __func__, tot_bytes_sent, size,
+				      strerror(errno));
+				errno = SLURM_COMMUNICATIONS_SEND_ERROR;
+				tot_bytes_sent = SLURM_ERROR;
+				break;
 			}
 		}
 
@@ -235,7 +210,7 @@ static int _send_timeout(int fd, char *buf, size_t size,
 		 * nonblocking read means just that.
 		 */
 		if (ufds.revents & POLLERR) {
-			int e, rc;
+			int e;
 
 			if ((rc = fd_get_socket_error(fd, &e)))
 				debug("%s: Socket POLLERR, fd_get_socket_error failed: %s",
@@ -244,84 +219,171 @@ static int _send_timeout(int fd, char *buf, size_t size,
 				debug("%s: Socket POLLERR: %s",
 				      __func__, slurm_strerror(e));
 
-			slurm_seterrno(e);
-			sent = SLURM_ERROR;
-			goto done;
+			errno = e;
+			tot_bytes_sent = SLURM_ERROR;
+			break;
 		}
 		if ((ufds.revents & POLLHUP) || (ufds.revents & POLLNVAL) ||
-		    (recv(fd, &temp, 1, flags) == 0)) {
-			int so_err, rc;
+		    (recv(fd, &temp, 1, 0) == 0)) {
+			int so_err;
 			if ((rc = fd_get_socket_error(fd, &so_err)))
 				debug2("%s: Socket no longer there, fd_get_socket_error failed: %s",
 				       __func__, slurm_strerror(rc));
 			else
 				debug2("%s: Socket no longer there: %s",
 				       __func__, slurm_strerror(so_err));
-			slurm_seterrno(so_err);
-			sent = SLURM_ERROR;
-			goto done;
+			errno = so_err;
+			tot_bytes_sent = SLURM_ERROR;
+			break;
 		}
 		if ((ufds.revents & POLLOUT) != POLLOUT) {
 			error("%s: Poll failure, revents:%d",
 			      __func__, ufds.revents);
 		}
 
-		rc = send(fd, &buf[sent], (size - sent), flags);
-		if (rc < 0) {
+ready:
+		if (tls_conn) {
+			bytes_sent = conn_g_sendv(tls_conn, iov, iovcnt);
+		} else {
+			bytes_sent = writev(fd, iov, iovcnt);
+		}
+
+		if (bytes_sent < 0) {
  			if (errno == EINTR)
 				continue;
-			debug("%s at %d of %zu, send error: %s",
-			      __func__, sent, size, strerror(errno));
- 			if (errno == EAGAIN) {	/* poll() lied to us */
+
+			log_flag(NET, "%s: [fd:%d] writev() sent %zd/%zu bytes failed: %m",
+				 __func__, fd, bytes_sent, size);
+
+			if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+				/* poll() lied to us */
 				usleep(10000);
 				continue;
 			}
- 			slurm_seterrno(SLURM_COMMUNICATIONS_SEND_ERROR);
-			sent = SLURM_ERROR;
-			goto done;
+			errno = SLURM_COMMUNICATIONS_SEND_ERROR;
+			tot_bytes_sent = SLURM_ERROR;
+			break;
 		}
-		if (rc == 0) {
+		if (bytes_sent == 0) {
 			/*
 			 * If driver false reports POLLIN but then does not
 			 * provide any output: try poll() again.
 			 */
-			log_flag(NET, "send() sent zero bytes out of %d/%zu",
-				 sent, size);
+			log_flag(NET, "%s: [fd:%d] writev() sent zero bytes out of %d/%zu",
+				 __func__, fd, tot_bytes_sent, size);
 			continue;
 		}
 
-		sent += rc;
+		tot_bytes_sent += bytes_sent;
+
+		if (tot_bytes_sent >= size) {
+			log_flag(NET, "%s: [fd:%d] writev() completed sending %d/%zu bytes",
+				 __func__, fd, tot_bytes_sent, size);
+			break;
+		}
+
+		/* partial write, need to adjust iovec before next call */
+		for (int i = 0; i < iovcnt; i++) {
+			if (bytes_sent < iov[i].iov_len) {
+				iov[i].iov_base += bytes_sent;
+				iov[i].iov_len -= bytes_sent;
+				break;
+			}
+
+			bytes_sent -= iov[i].iov_len;
+			iov[i].iov_base = NULL;
+			iov[i].iov_len = 0;
+		}
 	}
 
-    done:
 	/* Reset fd flags to prior state, preserve errno */
 	if (fd_flags != -1) {
-		int slurm_err = slurm_get_errno();
+		int slurm_err = errno;
 		if (fcntl(fd, F_SETFL, fd_flags) < 0)
 			error("%s: fcntl(F_SETFL) error: %m", __func__);
-		slurm_seterrno(slurm_err);
+		errno = slurm_err;
 	}
 
-	*timeout = *timeout - _tot_wait(&tstart);
-	return sent;
-
+	return tot_bytes_sent;
 }
 
 /*
  * Send slurm message with timeout
  * RET message size (as specified in argument) or SLURM_ERROR on error
  */
-extern int slurm_send_timeout(int fd, char *buf, size_t size,
-			      uint32_t flags, int timeout)
+extern int slurm_send_timeout(void *tls_conn, char *buf, size_t size,
+			      int timeout)
 {
-	return _send_timeout(fd, buf, size, flags, &timeout);
+	struct iovec iov = { .iov_base = buf, .iov_len = size };
+	return _writev_timeout(-1, tls_conn, &iov, 1, timeout);
 }
 
-extern size_t slurm_bufs_sendto(int fd, msg_bufs_t *buffers)
+extern ssize_t slurm_msg_sendto(void *tls_conn, char *buffer, size_t size)
 {
+	struct iovec iov[2];
+	uint32_t usize;
+	int len;
+	SigFunc *ohandler;
+	int timeout = slurm_conf.msg_timeout * 1000;
+
+	/*
+	 *  Ignore SIGPIPE so that send can return a error code if the
+	 *    other side closes the socket
+	 */
+	ohandler = xsignal(SIGPIPE, SIG_IGN);
+
+	iov[0].iov_base = &usize;
+	iov[0].iov_len = sizeof(usize);
+	iov[1].iov_base = buffer;
+	iov[1].iov_len = size;
+
+	usize = htonl(iov[1].iov_len);
+
+	len = _writev_timeout(-1, tls_conn, iov, 2, timeout);
+
+	xsignal(SIGPIPE, ohandler);
+
+	/* Returned size should not include the 4-byte length header. */
+	if (len < 0)
+		return SLURM_ERROR;
+	return size;
+}
+
+extern ssize_t slurm_msg_sendto_socket(int fd, char *buffer, size_t size)
+{
+	struct iovec iov[2];
+	uint32_t usize;
+	int len;
+	SigFunc *ohandler;
+	int timeout = slurm_conf.msg_timeout * 1000;
+
+	/*
+	 *  Ignore SIGPIPE so that send can return a error code if the
+	 *    other side closes the socket
+	 */
+	ohandler = xsignal(SIGPIPE, SIG_IGN);
+
+	iov[0].iov_base = &usize;
+	iov[0].iov_len = sizeof(usize);
+	iov[1].iov_base = buffer;
+	iov[1].iov_len = size;
+
+	usize = htonl(iov[1].iov_len);
+
+	len = _writev_timeout(fd, NULL, iov, 2, timeout);
+
+	xsignal(SIGPIPE, ohandler);
+
+	/* Returned size should not include the 4-byte length header. */
+	if (len < 0)
+		return SLURM_ERROR;
+	return size;
+}
+
+extern ssize_t slurm_bufs_sendto(void *tls_conn, msg_bufs_t *buffers)
+{
+	struct iovec iov[4];
 	int len;
-	int part_len;
-	size_t size = 0;
 	uint32_t usize;
 	SigFunc *ohandler;
 	int timeout = slurm_conf.msg_timeout * 1000;
@@ -335,47 +397,29 @@ extern size_t slurm_bufs_sendto(int fd, msg_bufs_t *buffers)
 	ohandler = xsignal(SIGPIPE, SIG_IGN);
 
 	/* auth portion is optional. header and body are mandatory. */
-	size += get_buf_offset(buffers->header);
-	if (buffers->auth)
-		size += get_buf_offset(buffers->auth);
-	size += get_buf_offset(buffers->body);
+	iov[0].iov_base = &usize;
+	iov[0].iov_len = sizeof(usize);
+	iov[1].iov_base = get_buf_data(buffers->header);
+	iov[1].iov_len = get_buf_offset(buffers->header);
+	iov[2].iov_base = buffers->auth ? get_buf_data(buffers->auth) : NULL;
+	iov[2].iov_len = buffers->auth ? get_buf_offset(buffers->auth) : 0;
+	iov[3].iov_base = get_buf_data(buffers->body);
+	iov[3].iov_len = get_buf_offset(buffers->body);
 
-	usize = htonl(size);
+	usize = htonl(iov[1].iov_len + iov[2].iov_len + iov[3].iov_len);
 
-	if ((len = _send_timeout(fd, (char *) &usize, sizeof(usize), 0,
-				 &timeout)) < 0)
-		goto done;
+	len = _writev_timeout(-1, tls_conn, iov, 4, timeout);
 
-	if ((part_len = _send_timeout(fd, get_buf_data(buffers->header),
-				      get_buf_offset(buffers->header), 0,
-				      &timeout)) < 0)
-			goto done;
-	len += part_len;
-
-	if (buffers->auth) {
-		if ((part_len = _send_timeout(fd, get_buf_data(buffers->auth),
-					      get_buf_offset(buffers->auth), 0,
-					      &timeout)) < 0)
-				goto done;
-		len += part_len;
-	}
-
-	if ((part_len = _send_timeout(fd, get_buf_data(buffers->body),
-				      get_buf_offset(buffers->body), 0,
-				      &timeout)) < 0)
-			goto done;
-	len += part_len;
-
-done:
 	xsignal(SIGPIPE, ohandler);
 	return len;
 }
 
 /* Get slurm message with timeout
  * RET message size (as specified in argument) or SLURM_ERROR on error */
-extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
-			      uint32_t flags, int timeout )
+extern int slurm_recv_timeout(void *tls_conn, char *buffer, size_t size,
+			      int timeout)
 {
+	int fd = -1;
 	int rc;
 	int recvlen = 0;
 	int fd_flags;
@@ -383,6 +427,8 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 	struct timeval tstart;
 	int timeleft = timeout;
 
+	fd = conn_g_get_fd(tls_conn);
+
 	ufds.fd     = fd;
 	ufds.events = POLLIN;
 
@@ -392,23 +438,25 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 	gettimeofday(&tstart, NULL);
 
 	while (recvlen < size) {
-		timeleft = timeout - _tot_wait(&tstart);
+		timeleft = timeout - timeval_tot_wait(&tstart);
 		if (timeleft <= 0) {
 			debug("%s at %d of %zu, timeout", __func__, recvlen,
 			      size);
-			slurm_seterrno(SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT);
+			errno = SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT;
 			recvlen = SLURM_ERROR;
 			goto done;
 		}
 
+		if (conn_g_peek(tls_conn))
+			goto ready;
+
 		if ((rc = poll(&ufds, 1, timeleft)) <= 0) {
 			if ((errno == EINTR) || (errno == EAGAIN) || (rc == 0))
 				continue;
 			else {
 				debug("%s at %d of %zu, poll error: %m",
 				      __func__, recvlen, size);
- 				slurm_seterrno(
-					SLURM_COMMUNICATIONS_RECEIVE_ERROR);
+				errno = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
  				recvlen = SLURM_ERROR;
   				goto done;
 			}
@@ -424,7 +472,7 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 				debug("%s: Socket POLLERR: %s",
 				      __func__, slurm_strerror(e));
 
-			slurm_seterrno(e);
+			errno = e;
 			recvlen = SLURM_ERROR;
 			goto done;
 		}
@@ -435,11 +483,11 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 			if ((rc = fd_get_socket_error(fd, &so_err))) {
 				debug2("%s: Socket no longer there: fd_get_socket_error failed: %s",
 				       __func__, slurm_strerror(rc));
-				slurm_seterrno(rc);
+				errno = rc;
 			} else {
 				debug2("%s: Socket no longer there: %s",
 				       __func__, slurm_strerror(so_err));
-				slurm_seterrno(so_err);
+				errno = so_err;
 			}
 			recvlen = SLURM_ERROR;
 			goto done;
@@ -450,7 +498,9 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 			continue;
 		}
 
-		rc = recv(fd, &buffer[recvlen], (size - recvlen), flags);
+ready:
+		rc = conn_g_recv(tls_conn, &buffer[recvlen], (size - recvlen));
+
 		if (rc < 0)  {
 			if ((errno == EINTR) || (errno == EAGAIN)) {
 				log_flag(NET, "%s: recv(fd:%d) got %m. retrying.",
@@ -459,16 +509,20 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
 			} else {
 				debug("%s at %d of %zu, recv error: %m",
 				      __func__, recvlen, size);
-				slurm_seterrno(
-					SLURM_COMMUNICATIONS_RECEIVE_ERROR);
+				errno = SLURM_COMMUNICATIONS_RECEIVE_ERROR;
 				recvlen = SLURM_ERROR;
 				goto done;
 			}
 		}
 		if (rc == 0) {
-			debug("%s at %d of %zu, recv zero bytes",
-			      __func__, recvlen, size);
-			slurm_seterrno(SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT);
+			log_flag(NET, "%s: recv(fd:%d)=0 at %d/%zu bytes",
+				 __func__, fd, recvlen, size);
+
+			if (recvlen > 0)
+				errno = ESLURM_PROTOCOL_INCOMPLETE_PACKET;
+			else
+				errno = SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT;
+
 			recvlen = SLURM_ERROR;
 			goto done;
 		}
@@ -479,10 +533,10 @@ extern int slurm_recv_timeout(int fd, char *buffer, size_t size,
     done:
 	/* Reset fd flags to prior state, preserve errno */
 	if (fd_flags != -1) {
-		int slurm_err = slurm_get_errno();
+		int slurm_err = errno;
 		if (fcntl(fd, F_SETFL, fd_flags) < 0)
 			error("%s: fcntl(F_SETFL) error: %m", __func__);
-		slurm_seterrno(slurm_err);
+		errno = slurm_err;
 	}
 
 	return recvlen;
@@ -536,51 +590,48 @@ error:
  * set *ADDR (which is *ADDR_LEN bytes long) to the address of the connecting
  * peer and *ADDR_LEN to the address's actual length, and return the
  * new socket's descriptor, or -1 for errors.  */
-extern int slurm_accept_msg_conn(int fd, slurm_addr_t *addr)
+extern int slurm_accept_conn(int fd, slurm_addr_t *addr)
 {
 	socklen_t len = sizeof(*addr);
-	return accept4(fd, (struct sockaddr *) addr, &len, SOCK_CLOEXEC);
+	int sock = accept4(fd, (struct sockaddr *) addr, &len, SOCK_CLOEXEC);
+	net_set_nodelay(sock, true, NULL);
+	return sock;
+}
+
+extern void *slurm_accept_msg_conn(int fd, slurm_addr_t *addr)
+{
+	socklen_t len = sizeof(*addr);
+	int sock = -1;
+	void *tls_conn = NULL;
+	conn_args_t tls_args = {
+		.mode = TLS_CONN_SERVER,
+	};
+
+	sock = accept4(fd, (struct sockaddr *) addr, &len, SOCK_CLOEXEC);
+
+	if (sock < 0) {
+		error("%s: Unable to accept() connection to address %pA: %m",
+		      __func__, addr);
+		return NULL;
+	}
+
+	tls_args.input_fd = tls_args.output_fd = sock;
+	net_set_nodelay(sock, true, NULL);
+
+	if (!(tls_conn = conn_g_create(&tls_args))) {
+		error("%s: Unable to create server TLS connection to address %pA: %m",
+		      __func__, addr);
+		(void) close(sock);
+	}
+
+	return tls_conn;
 }
 
 extern int slurm_open_stream(slurm_addr_t *addr, bool retry)
 {
-	int retry_cnt;
-	int fd;
-
-#ifdef HAVE_NATIVE_CRAY
-	static int check_quiesce = -1;
-	if (check_quiesce == -1) {
-		if (xstrcasestr(slurm_conf.comm_params, "CheckGhalQuiesce"))
-			check_quiesce = 1;
-		else
-			check_quiesce = 0;
-	}
-
-	if (check_quiesce) {
-		char buffer[20];
-		char *quiesce_status = "/sys/class/gni/ghal0/quiesce_status";
-		int max_retry = 300;
-		int quiesce_fd = open(quiesce_status, O_RDONLY);
-
-		retry_cnt = 0;
-		while (quiesce_fd >= 0 && retry_cnt < max_retry) {
-			if (read(quiesce_fd, buffer, sizeof(buffer)) > 0) {
-				if (buffer[0] == '0')
-					break;
-			}
-			usleep(500000);
-			if (retry_cnt % 10 == 0)
-				debug3("WARNING: ghal0 quiesce status: %c, retry count %d",
-				       buffer[0], retry_cnt);
-			retry_cnt++;
-			close(quiesce_fd);
-			quiesce_fd = open(quiesce_status,
-					  (O_RDONLY | O_CLOEXEC));
-		}
-		if (quiesce_fd >= 0)
-			close(quiesce_fd);
-	}
-#endif
+	int retry_cnt = 0, ehostunreach_cnt = 0;
+	int fd, rc = SLURM_SUCCESS;
+	uint32_t sleep_ns = 500 * NSEC_IN_MSEC;
 
 	if ((slurm_addr_is_unspec(addr)) || (slurm_get_port(addr) == 0)) {
 		error("Error connecting, bad data: family = %u, port = %u",
@@ -588,17 +639,16 @@ extern int slurm_open_stream(slurm_addr_t *addr, bool retry)
 		return SLURM_ERROR;
 	}
 
-	for (retry_cnt=0; ; retry_cnt++) {
-		int rc;
-
+	while (true) {
 		fd = socket(addr->ss_family, SOCK_STREAM | SOCK_CLOEXEC,
 			    IPPROTO_TCP);
 		if (fd < 0) {
 			error("Error creating slurm stream socket: %m");
-			slurm_seterrno(errno);
 			return SLURM_ERROR;
 		}
 
+		net_set_nodelay(fd, true, NULL);
+
 		if (retry_cnt) {
 			if (retry_cnt == 1) {
 				debug3("Error connecting, "
@@ -609,31 +659,85 @@ extern int slurm_open_stream(slurm_addr_t *addr, bool retry)
 
 		rc = _slurm_connect(fd, (struct sockaddr const *)addr,
 				    sizeof(*addr));
-		/* always set errno as upstream callers expect it */
-		slurm_seterrno(rc);
-
 		if (!rc) {
 			/* success */
 			break;
 		}
+		if (slurm_conf.host_unreach_retry_count &&
+		    (rc == EHOSTUNREACH)) {
+			if (ehostunreach_cnt >=
+			    slurm_conf.host_unreach_retry_count) {
+				debug2("Error connecting to %pA (%s), host_unreach_retry_count (%d) reached",
+				       addr, strerror(rc),
+				       slurm_conf.host_unreach_retry_count);
+				goto error;
+			}
 
-		if (((rc != ECONNREFUSED) && (rc != ETIMEDOUT)) ||
-		    (!retry) || (retry_cnt >= PORT_RETRIES)) {
-			slurm_seterrno(rc);
+			ehostunreach_cnt++;
+			debug2("Error connecting to %pA (%s), trying again",
+			       addr, strerror(rc));
+			/*
+			 * Assume issue was temporary network outage. Wait for
+			 * a bit and hope it comes back.
+			 */
+			slurm_nanosleep(0, sleep_ns);
+		} else if (((rc != ECONNREFUSED) && (rc != ETIMEDOUT)) ||
+			   (!retry) || (retry_cnt >= PORT_RETRIES)) {
 			goto error;
+		} else {
+			retry_cnt++;
 		}
 
 		(void) close(fd);
 	}
 
+	errno = rc;
 	return fd;
 
 error:
-	debug2("Error connecting slurm stream socket at %pA: %m", addr);
+	debug2("Error connecting slurm stream socket at %pA: %s", addr,
+	       strerror(rc));
 	(void) close(fd);
+	/* Always set errno as upstream callers expect it. */
+	errno = rc;
 	return SLURM_ERROR;
 }
 
+extern int slurm_open_unix_stream(char *addr_name, int sock_flags, int *fd)
+{
+	struct sockaddr_un sa;
+	int rc;
+
+	xassert(addr_name);
+
+	if (strlen(addr_name) >= sizeof(sa.sun_path)) {
+		rc = ESLURMD_INVALID_SOCKET_NAME_LEN;
+		error("%s: [%s]: %s", __func__, addr_name, slurm_strerror(rc));
+		return rc;
+	}
+
+	if ((*fd = socket(AF_UNIX, SOCK_STREAM | sock_flags, 0)) < 0) {
+		rc = errno;
+		error("%s: [%s]: socket() failed: %m", __func__, addr_name);
+		return rc;
+	}
+
+	memset(&sa, 0, sizeof(sa));
+	sa.sun_family = AF_UNIX;
+	strcpy(sa.sun_path, addr_name);
+
+	while ((rc = connect(*fd, (struct sockaddr *) &sa, SUN_LEN(&sa))) &&
+	       (errno == EINTR))
+		; /* empty loop */
+
+	if (rc < 0) {
+		rc = errno;
+		debug2("%s: [%s]: connect() failed: %m", __func__, addr_name);
+		fd_close(fd);
+	}
+	return rc;
+}
+
 /* Put the local address of FD into *ADDR and its length in *LEN.  */
 extern int slurm_get_stream_addr(int fd, slurm_addr_t *addr )
 {
@@ -749,7 +853,7 @@ extern void slurm_set_addr(slurm_addr_t *addr, uint16_t port, char *host)
 	 * Make sure we return the v6 wildcard address first (when applicable)
 	 * since we want v6 to be the default.
 	 */
-	if (host || !(slurm_conf.conf_flags & CTL_CONF_IPV6_ENABLED)) {
+	if (host || !(slurm_conf.conf_flags & CONF_FLAG_IPV6_ENABLED)) {
 		ai_ptr = ai_start;
 	} else {
 		for (ai_ptr = ai_start; ai_ptr; ai_ptr = ai_ptr->ai_next) {
