@@ -48,11 +48,9 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/un.h>
 #include <unistd.h>
 
 #include "src/common/fd.h"
@@ -60,9 +58,6 @@
 #include "src/common/macros.h"
 #include "src/common/pack.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/auth.h"
-#include "src/interfaces/cred.h"
-#include "src/interfaces/jobacct_gather.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/stepd_api.h"
 #include "src/common/strlcpy.h"
@@ -70,6 +65,11 @@
 #include "src/common/xregex.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/auth.h"
+#include "src/interfaces/conn.h"
+#include "src/interfaces/cred.h"
+#include "src/interfaces/jobacct_gather.h"
+
 strong_alias(stepd_available, slurm_stepd_available);
 strong_alias(stepd_connect, slurm_stepd_connect);
 strong_alias(stepd_get_uid, slurm_stepd_get_uid);
@@ -146,8 +146,7 @@ _step_connect(const char *directory, const char *nodename,
 	      slurm_step_id_t *step_id)
 {
 	int fd;
-	int len;
-	struct sockaddr_un addr;
+	int rc;
 	char *name = NULL, *pos = NULL;
 	uint32_t stepid = step_id->step_id;
 
@@ -156,33 +155,10 @@ _step_connect(const char *directory, const char *nodename,
 	if (step_id->step_het_comp != NO_VAL)
 		xstrfmtcatat(name, &pos, ".%u", step_id->step_het_comp);
 
-	/*
-	 * If socket name would be truncated, emit error and exit
-	 */
-	if (strlen(name) >= sizeof(addr.sun_path)) {
-		error("%s: Unix socket path '%s' is too long. (%ld > %ld)",
-		      __func__, name, (long int)(strlen(name) + 1),
-		      (long int)sizeof(addr.sun_path));
-		xfree(name);
-		return -1;
-	}
-
-	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
-		error("%s: socket() failed for %s: %m",
-		      __func__, name);
-		xfree(name);
-		return -1;
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.sun_family = AF_UNIX;
-	strlcpy(addr.sun_path, name, sizeof(addr.sun_path));
-	len = strlen(addr.sun_path) + 1 + sizeof(addr.sun_family);
-
-	if (connect(fd, (struct sockaddr *) &addr, len) < 0) {
+	if ((rc = slurm_open_unix_stream(name, 0, &fd))) {
 		/* Can indicate race condition at step termination */
-		debug("%s: connect() failed for %s: %m",
-		      __func__, name);
+		debug("%s: failed for %s: %s",
+		      __func__, name, slurm_strerror(rc));
 		if (errno == ECONNREFUSED && running_in_slurmd()) {
 			_handle_stray_socket(name);
 
@@ -229,7 +205,7 @@ _guess_nodename(void)
 }
 
 /*
- * Connect to a slurmstepd proccess by way of its unix domain socket.
+ * Connect to a slurmstepd process by way of its unix domain socket.
  *
  * Both "directory" and "nodename" may be null, in which case stepd_connect
  * will attempt to determine them on its own.  If you are using multiple
@@ -278,7 +254,7 @@ extern int stepd_connect(const char *directory, const char *nodename,
 	return fd;
 
 rwfail:
-	close(fd);
+	fd_close(&fd);
 fail1:
 	xfree(local_nodename);
 	return fd;
@@ -378,12 +354,12 @@ extern int stepd_get_namespace_fd(int fd, uint16_t protocol_version)
 	/*
 	 * Receive the file descriptor of the namespace to be joined if valid fd
 	 * is coming. Note that the number of ns_fd will not be the same
-	 * returned from receive_fd_over_pipe().  The number we got from the
+	 * returned from receive_fd_over_socket().  The number we got from the
 	 * safe_read was the fd on the sender which will be different on our
 	 * end.
 	 */
 	if (ns_fd > 0)
-		ns_fd = receive_fd_over_pipe(fd);
+		ns_fd = receive_fd_over_socket(fd);
 
 	return ns_fd;
 
@@ -397,19 +373,39 @@ rwfail:
  * On success returns SLURM_SUCCESS and fills in resp->local_pids,
  * resp->gtids, resp->ntasks, and resp->executable.
  */
-int stepd_attach(int fd, uint16_t protocol_version, slurm_addr_t *ioaddr,
-		 slurm_addr_t *respaddr, void *job_cred_sig, uint32_t sig_len,
-		 uid_t uid, reattach_tasks_response_msg_t *resp)
+extern int stepd_attach(int fd, uint16_t protocol_version, slurm_addr_t *ioaddr,
+			slurm_addr_t *respaddr, char *cert, char *io_key,
+			uid_t uid, reattach_tasks_response_msg_t *resp)
 {
 	int req = REQUEST_ATTACH;
+	uint32_t io_key_len = strlen(io_key) + 1;
+	uint32_t cert_len;
 	int rc = SLURM_SUCCESS;
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_25_05_PROTOCOL_VERSION) {
+		safe_write(fd, &req, sizeof(int)); /* needs to be first */
+
+		if (cert) {
+			cert_len = strlen(cert) + 1;
+			safe_write(fd, &cert_len, sizeof(uint32_t));
+			safe_write(fd, cert, cert_len);
+		} else {
+			cert_len = 0;
+			safe_write(fd, &cert_len, sizeof(uint32_t));
+		}
+
+		safe_write(fd, ioaddr, sizeof(slurm_addr_t));
+		safe_write(fd, respaddr, sizeof(slurm_addr_t));
+		safe_write(fd, &io_key_len, sizeof(uint32_t));
+		safe_write(fd, io_key, io_key_len);
+		safe_write(fd, &uid, sizeof(uid_t));
+		safe_write(fd, &protocol_version, sizeof(uint16_t));
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_write(fd, &req, sizeof(int));
 		safe_write(fd, ioaddr, sizeof(slurm_addr_t));
 		safe_write(fd, respaddr, sizeof(slurm_addr_t));
-		safe_write(fd, &sig_len, sizeof(uint32_t));
-		safe_write(fd, job_cred_sig, sig_len);
+		safe_write(fd, &io_key_len, sizeof(uint32_t));
+		safe_write(fd, io_key, io_key_len);
 		safe_write(fd, &uid, sizeof(uid_t));
 		safe_write(fd, &protocol_version, sizeof(uint16_t));
 	} else
@@ -527,22 +523,23 @@ _sockname_regex(regex_t *re, const char *filename, slurm_step_id_t *step_id)
  * slurmd on one node (unusual outside of development environments), you
  * will get one of the local NodeNames more-or-less at random.
  *
- * Returns a List of pointers to step_loc_t structures.
+ * Returns a list of pointers to step_loc_t structures.
  */
-extern List
-stepd_available(const char *directory, const char *nodename)
+extern list_t *stepd_available(const char *directory, const char *nodename)
 {
-	List l;
+	list_t *l = NULL;
 	DIR *dp;
 	struct dirent *ent;
 	regex_t re;
 	struct stat stat_buf;
+	char *local_nodename = NULL;
 
 	if (nodename == NULL) {
-		if (!(nodename = _guess_nodename())) {
+		if (!(local_nodename = _guess_nodename())) {
 			error("%s: Couldn't find nodename", __func__);
 			return NULL;
 		}
+		nodename = local_nodename;
 	}
 	if (directory == NULL) {
 		slurm_conf_t *cf = slurm_conf_lock();
@@ -587,6 +584,7 @@ stepd_available(const char *directory, const char *nodename)
 
 	closedir(dp);
 done:
+	xfree(local_nodename);
 	regfree(&re);
 	return l;
 }
@@ -1003,7 +1001,7 @@ stepd_suspend(int fd, uint16_t protocol_version,
 	int rc = 0;
 	int errnum = 0;
 
-	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		if (phase == 0) {
 			safe_write(fd, &req, sizeof(int));
 		} else {
@@ -1012,16 +1010,6 @@ stepd_suspend(int fd, uint16_t protocol_version,
 			safe_read(fd, &errnum, sizeof(int));
 			errno = errnum;
 		}
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		if (phase == 0) {
-			safe_write(fd, &req, sizeof(int));
-			safe_write(fd, NO_VAL16, sizeof(uint16_t));
-		} else {
-			/* Receive the return code and errno */
-			safe_read(fd, &rc, sizeof(int));
-			safe_read(fd, &errnum, sizeof(int));
-			errno = errnum;
-		}
 	}
 
 	return rc;
@@ -1045,7 +1033,7 @@ stepd_resume(int fd, uint16_t protocol_version,
 	int rc = 0;
 	int errnum = 0;
 
-	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		if (phase == 0) {
 			safe_write(fd, &req, sizeof(int));
 		} else {
@@ -1054,16 +1042,6 @@ stepd_resume(int fd, uint16_t protocol_version,
 			safe_read(fd, &errnum, sizeof(int));
 			errno = errnum;
 		}
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		if (phase == 0) {
-			safe_write(fd, &req, sizeof(int));
-			safe_write(fd, NO_VAL16, sizeof(uint16_t));
-		} else {
-			/* Receive the return code and errno */
-			safe_read(fd, &rc, sizeof(int));
-			safe_read(fd, &errnum, sizeof(int));
-			errno = errnum;
-		}
 	}
 
 	return rc;
@@ -1079,7 +1057,7 @@ extern int stepd_reconfig(int fd, uint16_t protocol_version, buf_t *reconf)
 
 	safe_write(fd, &req, sizeof(int));
 
-	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		int len = 0;
 		if (reconf) {
 			len = get_buf_offset(reconf);
@@ -1144,7 +1122,34 @@ stepd_completion(int fd, uint16_t protocol_version, step_complete_msg_t *sent)
 	debug("Entering stepd_completion for %ps, range_first = %d, range_last = %d",
 	      &sent->step_id, sent->range_first, sent->range_last);
 
-	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_24_05_PROTOCOL_VERSION) {
+		safe_write(fd, &req, sizeof(int));
+		safe_write(fd, &sent->range_first, sizeof(int));
+		safe_write(fd, &sent->range_last, sizeof(int));
+		safe_write(fd, &sent->step_rc, sizeof(int));
+		safe_write(fd, &sent->step_id.step_id, sizeof(uint32_t));
+		safe_write(fd, &sent->send_to_stepmgr, sizeof(bool));
+
+		/*
+		 * We must not use setinfo over a pipe with slurmstepd here
+		 * Indeed, slurmd does a large use of getinfo over a pipe
+		 * with slurmstepd and doing the reverse can result in
+		 * a deadlock scenario with slurmstepd :
+		 * slurmd(lockforread,write)/slurmstepd(write,lockforread)
+		 * Do pack/unpack instead to be sure of independances of
+		 * slurmd and slurmstepd
+		 */
+		jobacctinfo_pack(sent->jobacct, protocol_version,
+				 PROTOCOL_TYPE_SLURM, buffer);
+		len = get_buf_offset(buffer);
+		safe_write(fd, &len, sizeof(int));
+		safe_write(fd, get_buf_data(buffer), len);
+		FREE_NULL_BUFFER(buffer);
+
+		/* Receive the return code and errno */
+		safe_read(fd, &rc, sizeof(int));
+		safe_read(fd, &errnum, sizeof(int));
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_write(fd, &req, sizeof(int));
 		safe_write(fd, &sent->range_first, sizeof(int));
 		safe_write(fd, &sent->range_last, sizeof(int));
@@ -1375,3 +1380,23 @@ extern uint32_t stepd_get_nodeid(int fd, uint16_t protocol_version)
 rwfail:
 	return NO_VAL;
 }
+
+extern int stepd_relay_msg(int fd, slurm_msg_t *msg, uint16_t protocol_version)
+{
+	int req = msg->msg_type;
+	uint32_t buf_size;
+
+	safe_write(fd, &req, sizeof(int));
+
+	buf_size = get_buf_offset(msg->buffer) - msg->body_offset;
+
+	safe_write(fd, &msg->protocol_version, sizeof(uint16_t));
+	send_fd_over_socket(fd, conn_g_get_fd(msg->tls_conn));
+	safe_write(fd, &buf_size, sizeof(uint32_t));
+	safe_write(fd, &msg->buffer->head[msg->body_offset], buf_size);
+
+	return 0;
+
+rwfail:
+	return -1;
+}
