@@ -48,6 +48,8 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 
+#include "src/conmgr/conmgr.h"
+
 /*
  * Define slurm-specific aliases for use by plugins, see slurm_xlator.h
  * for details.
@@ -64,6 +66,9 @@ xsignal(int signo, SigFunc *f)
 {
 	struct sigaction sa, old_sa;
 
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	sa.sa_handler = f;
 	sigemptyset(&sa.sa_mask);
 	sigaddset(&sa.sa_mask, signo);
@@ -86,6 +91,10 @@ xsignal(int signo, SigFunc *f)
 extern SigFunc *xsignal_default(int sig)
 {
 	struct sigaction act;
+
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	if (sigaction(sig, NULL, &act)) {
 		error("sigaction(%d): %m", sig);
 		return NULL;
@@ -106,6 +115,9 @@ _sigmask(int how, sigset_t *set, sigset_t *oset)
 {
 	int err;
 
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	if ((err = pthread_sigmask(how, set, oset)))
 		return error ("pthread_sigmask: %s", slurm_strerror(err));
 
@@ -120,6 +132,9 @@ _sigmask(int how, sigset_t *set, sigset_t *oset)
 int
 xsignal_sigset_create(int sigarray[], sigset_t *setp)
 {
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	int i = 0, sig;
 
 	if (sigemptyset(setp) < 0)
@@ -136,6 +151,9 @@ xsignal_sigset_create(int sigarray[], sigset_t *setp)
 int
 xsignal_save_mask(sigset_t *set)
 {
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	sigemptyset(set);
 	return _sigmask(SIG_SETMASK, NULL, set);
 }
@@ -143,6 +161,9 @@ xsignal_save_mask(sigset_t *set)
 int
 xsignal_set_mask(sigset_t *set)
 {
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	return _sigmask(SIG_SETMASK, set, NULL);
 }
 
@@ -151,6 +172,9 @@ xsignal_block(int sigarray[])
 {
 	sigset_t set;
 
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	xassert(sigarray != NULL);
 
 	if (xsignal_sigset_create(sigarray, &set) < 0)
@@ -164,6 +188,9 @@ xsignal_unblock(int sigarray[])
 {
 	sigset_t set;
 
+	if (conmgr_enabled())
+		return SLURM_SUCCESS;
+
 	xassert(sigarray != NULL);
 
 	if (xsignal_sigset_create(sigarray, &set) < 0)
