@@ -45,7 +45,7 @@
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_time.h"
 
-#define CONMGR_THREAD_COUNT_DEFAULT 10
+#define CONMGR_THREAD_COUNT_DEFAULT 8
 #define CONMGR_THREAD_COUNT_MIN 2
 #define CONMGR_THREAD_COUNT_MAX 1024
 
@@ -109,6 +109,29 @@ typedef struct {
 	 */
 	int (*on_data)(conmgr_fd_t *con, void *arg);
 
+	/*
+	 * Call back when there is initial data ready in "in" buffer but before
+	 * on_data() or on_msg() to allow fingerprinting and re-typing of
+	 * the incoming connection.
+	 *
+	 * This may be called again in the same connection if prior call
+	 * returned EWOULDBLOCK.
+	 *
+	 * If NULL, then fingerprinting will be skipped for this connection.
+	 *
+	 * IN con - connection handler
+	 * IN buffer - pointer to buffer of already read() data
+	 * IN bytes - number of bytes in buffer
+	 * IN arg - ptr returned by on_connection() callback.
+	 * RET
+	 *	SLURM_SUCCESS: fingerprinting complete - stop callback
+	 *	EWOULDBLOCK: fingerprint requires more data, call again on new
+	 *		data.
+	 *	Any other error will cause connection to close in error.
+	 */
+	int (*on_fingerprint)(conmgr_fd_t *con, const void *buffer,
+			      const size_t bytes, void *arg);
+
 	/*
 	 * Call back when there is new RPC msg ready
 	 * This may be called several times in the same connection.
@@ -326,6 +349,22 @@ typedef enum {
 	 * takes longer than timeout when connection is otherwise idle.
 	 */
 	CON_FLAG_WATCH_CONNECT_TIMEOUT = SLURM_BIT(17),
+	/*
+	 * Incoming and outgoing is wrapped by TLS (or connection will fail)
+	 * in TLS_CONN_SERVER mode.
+	 * Mutually exclusive with CON_FLAG_TLS_CLIENT.
+	 */
+	CON_FLAG_TLS_SERVER = SLURM_BIT(18),
+	/*
+	 * Incoming and outgoing is wrapped by TLS (or connection will fail)
+	 * in TLS_CONN_CLIENT mode.
+	 * Mutually exclusive with CON_FLAG_TLS_SERVER.
+	 */
+	CON_FLAG_TLS_CLIENT = SLURM_BIT(19),
+	/*
+	 * Receive and forward incoming messages to their intended destinations.
+	 */
+	CON_FLAG_RPC_RECV_FORWARD = SLURM_BIT(23),
 } conmgr_con_flags_t;
 
 /*
@@ -357,6 +396,7 @@ extern int conmgr_get_fd_auth_creds(conmgr_fd_t *con, uid_t *cred_uid,
  * IN flags bit-or'ed flags to apply to connection
  * IN addr socket address (if known or NULL) (will always xfree())
  * IN addrlen sizeof addr or 0 if addr is NULL
+ * IN tls_conn - TLS connection state or NULL
  * IN arg ptr handed to on_connection callback
  * RET SLURM_SUCCESS or error
  */
@@ -364,7 +404,7 @@ extern int conmgr_process_fd(conmgr_con_type_t type, int input_fd,
 			     int output_fd, const conmgr_events_t *events,
 			     conmgr_con_flags_t flags,
 			     const slurm_addr_t *addr, socklen_t addrlen,
-			     void *arg);
+			     void *tls_conn, void *arg);
 
 /*
  * instruct connection manager to listen to fd (async)
@@ -438,6 +478,7 @@ extern int conmgr_fd_change_mode(conmgr_fd_t *con, conmgr_con_type_t type);
 /*
  * Create listening socket
  * IN type - connection type for new sockets
+ * IN flags - flags for connection
  * IN listen_on - cstrings to listen on:
  *	formats:
  *		host:port
@@ -447,13 +488,15 @@ extern int conmgr_fd_change_mode(conmgr_fd_t *con, conmgr_con_type_t type);
  * RET SLURM_SUCCESS or error
  */
 extern int conmgr_create_listen_socket(conmgr_con_type_t type,
-					const char *listen_on,
-					const conmgr_events_t *events,
-					void *arg);
+				       conmgr_con_flags_t flags,
+				       const char *listen_on,
+				       const conmgr_events_t *events,
+				       void *arg);
 
 /*
  * Create listening sockets from list of host:port pairs
  * IN type - connection type for new sockets
+ * IN flags - flags for connection
  * IN hostports - list_t* of cstrings to listen on:
  *	formats:
  *		host:port
@@ -463,6 +506,7 @@ extern int conmgr_create_listen_socket(conmgr_con_type_t type,
  * RET SLURM_SUCCESS or error
  */
 extern int conmgr_create_listen_sockets(conmgr_con_type_t type,
+					conmgr_con_flags_t flags,
 					list_t *hostports,
 					const conmgr_events_t *events,
 					void *arg);
@@ -470,6 +514,7 @@ extern int conmgr_create_listen_sockets(conmgr_con_type_t type,
 /*
  * Instruct conmgr to create new socket and connect to addr
  * IN type - connection for new socket
+ * IN flags - flags for connection
  * IN addr - destination address to connect() socket
  * IN addrlen - sizeof(*addr)
  * IN events - ptr to function callback on events
@@ -477,6 +522,7 @@ extern int conmgr_create_listen_sockets(conmgr_con_type_t type,
  * RET SLURM_SUCCESS or error
  */
 extern int conmgr_create_connect_socket(conmgr_con_type_t type,
+					conmgr_con_flags_t flags,
 					slurm_addr_t *addr, socklen_t addrlen,
 					const conmgr_events_t *events,
 					void *arg);
@@ -755,10 +801,11 @@ extern bool conmgr_enabled(void);
  * IN conmgr_args - Args relaying conmgr callback state
  * IN input_fd - input file descriptor or -1 - Ownership is transferred.
  * IN output_fd - output file descriptor or -1 - Ownership is transferred.
+ * IN tls_connn - TLS connection state or NULL
  */
 typedef void (*conmgr_extract_fd_func_t)(conmgr_callback_args_t conmgr_args,
 					 int input_fd, int output_fd,
-					 void *arg);
+					 void *tls_conn, void *arg);
 
 /*
  * Queue up extraction of file descriptors from a connection.
@@ -786,6 +833,7 @@ extern int conmgr_queue_extract_con_fd(conmgr_fd_t *con,
 #define CONMGR_PARAM_READ_TIMEOUT "CONMGR_READ_TIMEOUT="
 #define CONMGR_PARAM_WRITE_TIMEOUT "CONMGR_WRITE_TIMEOUT="
 #define CONMGR_PARAM_CONNECT_TIMEOUT "CONMGR_CONNECT_TIMEOUT="
+#define CONMGR_PARAM_QUIESCE_TIMEOUT "CONMGR_QUIESCE_TIMEOUT="
 
 /*
  * Set configuration parameters to be applied when conmgr_init() is called.
@@ -852,4 +900,15 @@ extern conmgr_fd_t *conmgr_fd_get_ref(conmgr_fd_ref_t *ref);
 #define conmgr_ref_get_name(ref) \
 	conmgr_fd_get_name(conmgr_fd_get_ref(ref))
 
+/*
+ * Checks if incoming data matches a TLS handshake and will change connection to
+ * CON_FLAG_TLS_SERVER on match
+ * Note: function is designed to be a on_fingerprint() event callback
+ */
+extern int on_fingerprint_tls(conmgr_fd_t *con, const void *buffer,
+			      const size_t bytes, void *arg);
+
+/* Return true if connection is TLS wrapped */
+extern bool conmgr_fd_is_tls(conmgr_fd_ref_t *ref);
+
 #endif /* _CONMGR_H */
