@@ -63,7 +63,14 @@
 #include "src/conmgr/mgr.h"
 #include "src/conmgr/polling.h"
 #include "src/conmgr/signals.h"
+#include "src/conmgr/tls.h"
 
+/* Default watch() sleep to only 5 minutes */
+#define WATCH_DEFAULT_SLEEP    \
+	(timespec_t)           \
+	{                      \
+		.tv_sec = 300, \
+	}
 #define CTIME_STR_LEN 72
 
 typedef struct {
@@ -75,22 +82,74 @@ typedef struct {
 
 static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg);
 
-static bool _handle_time_limit(const handle_connection_args_t *args,
+static void _set_time(handle_connection_args_t *args)
+{
+	if (!args->time.tv_sec)
+		args->time = timespec_now();
+}
+
+static bool _handle_time_limit(handle_connection_args_t *args,
 			       const struct timespec timestamp,
-			       const struct timespec limit)
+			       const struct timespec limit, const char *what,
+			       const char *name, const char *caller)
 {
 	const struct timespec deadline = timespec_add(timestamp, limit);
+	bool after, change_max_sleep = false;
 
-	if (timespec_is_after(args->time, deadline))
+	_set_time(args);
+
+	if (!(after = timespec_is_after(args->time, deadline)))
+		change_max_sleep =
+			(!mgr.watch_max_sleep.tv_sec ||
+			 timespec_is_after(mgr.watch_max_sleep, deadline));
+
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
+		char str[CTIME_STR_LEN];
+
+		timespec_ctime(limit, false, str, sizeof(str));
+
+		log_flag(CONMGR, "%s->%s: %s%s%s%s %s timeout %s %s",
+			 caller, __func__, (name ? "[" : ""), name,
+			 (name ? "] " : ""),
+			 (change_max_sleep ? "updating watch() sleep" :
+			  "evaluating"),
+			 what, (after ? "triggered" : "ETA"), str);
+	}
+
+	if (after)
 		return true;
 
-	if (!mgr.watch_max_sleep.tv_sec ||
-	    timespec_is_after(mgr.watch_max_sleep, deadline))
+	if (change_max_sleep) {
 		mgr.watch_max_sleep = deadline;
 
+		/* Always wake up watch() as if deadline changed */
+		EVENT_SIGNAL(&mgr.watch_sleep);
+	}
+
 	return false;
 }
 
+static void _reset_watch_max_sleep(void)
+{
+	const timespec_t now = timespec_now();
+
+	/* skip if the timeout hasn't elapsed yet */
+	if (timespec_is_after(mgr.watch_max_sleep, now))
+		return;
+
+	/* timeout triggered and needs reset */
+	mgr.watch_max_sleep = timespec_add(now, WATCH_DEFAULT_SLEEP);
+
+	if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
+		char str[CTIME_STR_LEN];
+
+		timespec_ctime(mgr.watch_max_sleep, true, str, sizeof(str));
+
+		log_flag(CONMGR, "%s: reset watch() sleep to %s",
+			 __func__, str);
+	}
+}
+
 static void _on_finish_wrapper(conmgr_callback_args_t conmgr_args, void *arg)
 {
 	conmgr_fd_t *con = conmgr_args.con;
@@ -236,7 +295,7 @@ static void _close_output_fd(conmgr_callback_args_t conmgr_args, void *arg)
 	 * > does not support synchronization.
 	 */
 	if (!con_flag(con, FLAG_IS_SOCKET) && !con_flag(con, FLAG_IS_FIFO) &&
-	    !con_flag(con, FLAG_IS_CHR)) {
+	    !con_flag(con, FLAG_IS_CHR) && (output_fd >= 0)) {
 		do {
 			if (fsync(output_fd)) {
 				rc = errno;
@@ -260,12 +319,20 @@ static void _close_output_fd(conmgr_callback_args_t conmgr_args, void *arg)
 
 static void _on_close_output_fd(conmgr_fd_t *con)
 {
+	if (con->output_fd < 0) {
+		log_flag(CONMGR, "%s: [%s] skipping as output_fd already closed",
+			 __func__, con->name);
+		return;
+	}
+
 	con_set_polling(con, PCTL_TYPE_NONE, __func__);
 
-	list_flush(con->out);
+	if (con->out)
+		list_flush(con->out);
 
-	add_work_con_fifo(true, con, _close_output_fd,
-			  ((void *) (uint64_t) con->output_fd));
+	if (con->output_fd >= 0)
+		add_work_con_fifo(true, con, _close_output_fd,
+				  ((void *) (uint64_t) con->output_fd));
 
 	con->output_fd = -1;
 }
@@ -329,6 +396,8 @@ static void _on_connect_timeout(handle_connection_args_t *args,
 	if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
 		char time_str[CTIME_STR_LEN], total_str[CTIME_STR_LEN];
 
+		_set_time(args);
+
 		timespec_ctime(timespec_diff_ns(con->last_read,
 						args->time).diff, false,
 			       time_str, sizeof(time_str));
@@ -396,6 +465,8 @@ static void _on_write_timeout(handle_connection_args_t *args, conmgr_fd_t *con)
 	if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
 		char time_str[CTIME_STR_LEN], total_str[CTIME_STR_LEN];
 
+		_set_time(args);
+
 		timespec_ctime(timespec_diff_ns(con->last_write,
 						args->time).diff, false,
 			       time_str, sizeof(time_str));
@@ -456,6 +527,8 @@ static void _on_read_timeout(handle_connection_args_t *args, conmgr_fd_t *con)
 	if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
 		char time_str[CTIME_STR_LEN], total_str[CTIME_STR_LEN];
 
+		_set_time(args);
+
 		timespec_ctime(timespec_diff_ns(con->last_read, args->time).diff,
 			       false, time_str, sizeof(time_str));
 		timespec_ctime(mgr.conf_read_timeout, false, total_str,
@@ -468,6 +541,116 @@ static void _on_read_timeout(handle_connection_args_t *args, conmgr_fd_t *con)
 	add_work_con_fifo(true, con, _wrap_on_read_timeout, NULL);
 }
 
+/* Caller must hold mgr->mutex lock */
+static bool _is_accept_deferred(void)
+{
+	return (list_count(mgr.connections) > mgr.max_connections);
+}
+
+/* caller must hold mgr->mutex lock */
+extern void queue_on_connection(conmgr_fd_t *con)
+{
+	/* disable polling until on_connect() is done */
+	con_set_polling(con, PCTL_TYPE_CONNECTED, __func__);
+
+	add_work_con_fifo(true, con, wrap_on_connection, con);
+
+	log_flag(CONMGR, "%s: [%s] Fully connected. Queuing on_connect() callback.",
+		 __func__, con->name);
+}
+
+static int _handle_connection_wait_write(conmgr_fd_t *con,
+					 handle_connection_args_t *args,
+					 list_t *out)
+{
+	/*
+	 * Only monitor for when connection is ready for writes
+	 * as there is no point reading until the write is
+	 * complete since it will be ignored.
+	 */
+	con_set_polling(con, PCTL_TYPE_WRITE_ONLY, __func__);
+
+	if (con_flag(con, FLAG_WATCH_WRITE_TIMEOUT) &&
+	    _handle_time_limit(args, con->last_write, mgr.conf_write_timeout,
+			       "write", con->name, __func__)) {
+		_on_write_timeout(args, con);
+		return 0;
+	}
+
+	/* must wait until poll allows write of this socket */
+	log_flag(CONMGR, "%s: [%s] waiting for %u writes",
+		 __func__, con->name, list_count(out));
+
+	return 0;
+}
+
+static int _handle_connection_write(conmgr_fd_t *con,
+				    handle_connection_args_t *args)
+{
+	if (!con_flag(con, FLAG_CAN_WRITE) &&
+	    (con->polling_output_fd != PCTL_TYPE_UNSUPPORTED))
+		return _handle_connection_wait_write(con, args, con->out);
+
+	log_flag(CONMGR, "%s: [%s] %u pending writes",
+		 __func__, con->name, list_count(con->out));
+	add_work_con_fifo(true, con, handle_write, con);
+	return 0;
+}
+
+extern void _handle_fingerprint(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	conmgr_fd_t *con = conmgr_args.con;
+	int match = EINVAL;
+	bool bail = false;
+
+	xassert(con->magic == MAGIC_CON_MGR_FD);
+
+	slurm_mutex_lock(&mgr.mutex);
+
+	xassert(con_flag(con, FLAG_IS_CONNECTED));
+	xassert(!con_flag(con, FLAG_IS_TLS_CONNECTED));
+
+	if (con_flag(con, FLAG_READ_EOF) || con_flag(con, FLAG_CAN_READ))
+		bail = true;
+
+	slurm_mutex_unlock(&mgr.mutex);
+
+	if (bail) {
+		log_flag(CONMGR, "%s: [%s] skipping fingerprint match",
+			 __func__, con->name);
+		return;
+	}
+
+	match = con->events->on_fingerprint(con, get_buf_data(con->in),
+					    get_buf_offset(con->in), con->arg);
+
+	if (match == SLURM_SUCCESS) {
+		log_flag(CONMGR, "%s: [%s] fingerprint match completed",
+					 __func__, con->name);
+
+		slurm_mutex_lock(&mgr.mutex);
+		con_unset_flag(con, FLAG_WAIT_ON_FINGERPRINT);
+		con_unset_flag(con, FLAG_ON_DATA_TRIED);
+
+		if (con->events->on_connection &&
+		    !con_flag(con, FLAG_TLS_SERVER))
+			queue_on_connection(con);
+		slurm_mutex_unlock(&mgr.mutex);
+	} else if (match == EWOULDBLOCK) {
+		log_flag(CONMGR, "%s: [%s] waiting for more bytes for fingerprint",
+				 __func__, con->name);
+
+		slurm_mutex_lock(&mgr.mutex);
+		con_set_flag(con, FLAG_ON_DATA_TRIED);
+		slurm_mutex_unlock(&mgr.mutex);
+	} else {
+		log_flag(CONMGR, "%s: [%s] fingerprint failed: %s",
+				 __func__, con->name, slurm_strerror(match));
+
+		close_con(false, con);
+	}
+}
+
 /*
  * handle connection states and apply actions required.
  * mgr mutex must be locked.
@@ -477,9 +660,22 @@ static void _on_read_timeout(handle_connection_args_t *args, conmgr_fd_t *con)
 static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 {
 	int count;
+	const bool is_tls = (con_flag(con, FLAG_TLS_SERVER) ||
+			     con_flag(con, FLAG_TLS_CLIENT));
+	handle_connection_args_t local_args = {
+		.magic = MAGIC_HANDLE_CONNECTION,
+	};
+
+	if (!args) {
+		/*
+		 * Always have args pointer populated to avoid breaking timeout
+		 * logic
+		 */
+		args = &local_args;
+	}
 
 	xassert(con->magic == MAGIC_CON_MGR_FD);
-	xassert(!args || (args->magic == MAGIC_HANDLE_CONNECTION));
+	xassert(args->magic == MAGIC_HANDLE_CONNECTION);
 
 	/* connection may have a running thread, do nothing */
 	if (con_flag(con, FLAG_WORK_ACTIVE)) {
@@ -488,7 +684,11 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		return 0;
 	}
 
-	if (con_flag(con, FLAG_IS_CONNECTED)) {
+	if (((con->input_fd < 0) && (con->output_fd < 0)) ||
+	    con_flag(con, FLAG_TLS_WAIT_ON_CLOSE)) {
+		xassert(con_flag(con, FLAG_READ_EOF));
+		/* connection already closed */
+	} else if (con_flag(con, FLAG_IS_CONNECTED)) {
 		/* continue on to follow other checks */
 	} else if (!con_flag(con, FLAG_IS_SOCKET) ||
 		   con_flag(con, FLAG_CAN_READ) ||
@@ -501,10 +701,8 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		con_set_flag(con, FLAG_IS_CONNECTED);
 
 		if (con_flag(con, FLAG_WATCH_READ_TIMEOUT)) {
-			if (args)
-				con->last_read = args->time;
-			else
-				con->last_read = timespec_now();
+			_set_time(args);
+			con->last_read = args->time;
 		}
 
 		if (con_flag(con, FLAG_IS_SOCKET) && (con->output_fd != -1)) {
@@ -527,13 +725,9 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 			} else {
 				/* follow normal checks */
 			}
-		} else if (con->events->on_connection) {
-			/* disable polling until on_connect() is done */
-			con_set_polling(con, PCTL_TYPE_CONNECTED, __func__);
-			add_work_con_fifo(true, con, wrap_on_connection, con);
-
-			log_flag(CONMGR, "%s: [%s] Fully connected. Queuing on_connect() callback.",
-				 __func__, con->name);
+		} else if (con->events->on_connection && !is_tls &&
+			   !con_flag(con, FLAG_WAIT_ON_FINGERPRINT)) {
+			queue_on_connection(con);
 			return 0;
 		} else {
 			/*
@@ -564,9 +758,10 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		 */
 		con_set_polling(con, PCTL_TYPE_READ_WRITE, __func__);
 
-		if (con_flag(con, FLAG_WATCH_CONNECT_TIMEOUT) && args &&
+		if (con_flag(con, FLAG_WATCH_CONNECT_TIMEOUT) &&
 		    _handle_time_limit(args, con->last_read,
-				       mgr.conf_connect_timeout)) {
+				       mgr.conf_connect_timeout, "connect",
+				       con->name, __func__)) {
 			_on_connect_timeout(args, con);
 			return 0;
 		}
@@ -608,46 +803,70 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		return 0;
 	}
 
-	if (con->extract) {
-		/* extraction of file descriptors requested */
+	if (con_flag(con, FLAG_TLS_WAIT_ON_CLOSE)) {
+		log_flag(CONMGR, "%s: [%s] waiting on delayed close of TLS connection",
+			 __func__, con->name);
+		return 0;
+	}
+
+	if (con->extract && !con_flag(con, FLAG_WAIT_ON_FINGERPRINT) &&
+	    (!is_tls || (con_flag(con, FLAG_IS_TLS_CONNECTED) &&
+			 !con_flag(con, FLAG_WAIT_ON_FINISH))) &&
+	    !con_flag(con, FLAG_QUIESCE) && list_is_empty(con->out) &&
+	    (!con->tls_out || list_is_empty(con->tls_out)) &&
+	    (!con->tls_in || !get_buf_offset(con->tls_in)) &&
+	    !con_flag(con, FLAG_READ_EOF) &&
+	    !con_flag(con, FLAG_WAIT_ON_FINISH) && !get_buf_offset(con->in)) {
+		/*
+		 * extraction of file descriptors requested
+		 * but only after starting TLS if needed
+		 */
 		extract_con_fd(con);
 		return 0;
 	}
 
 	/* handle out going data */
 	if (!con_flag(con, FLAG_IS_LISTEN) && (con->output_fd >= 0) &&
-	    !list_is_empty(con->out)) {
+	    con->tls_out && !list_is_empty(con->tls_out)) {
 		if (con_flag(con, FLAG_CAN_WRITE) ||
 		    (con->polling_output_fd == PCTL_TYPE_UNSUPPORTED)) {
-			log_flag(CONMGR, "%s: [%s] %u pending writes",
-				 __func__, con->name, list_count(con->out));
-			add_work_con_fifo(true, con, handle_write, con);
+			log_flag(CONMGR, "%s: [%s] %u pending TLS writes",
+				 __func__, con->name, list_count(con->tls_out));
+			add_work_con_fifo(true, con, tls_handle_write, con);
+			return 0;
 		} else {
-			/*
-			 * Only monitor for when connection is ready for writes
-			 * as there is no point reading until the write is
-			 * complete since it will be ignored.
-			 */
-			con_set_polling(con, PCTL_TYPE_WRITE_ONLY, __func__);
+			return _handle_connection_wait_write(con, args,
+							     con->tls_out);
+		}
+	}
 
-			if (con_flag(con, FLAG_WATCH_WRITE_TIMEOUT) && args &&
-			    _handle_time_limit(args, con->last_write,
-					       mgr.conf_write_timeout)) {
-				_on_write_timeout(args, con);
+	if (!con_flag(con, FLAG_IS_LISTEN) && (con->output_fd >= 0) &&
+	    !list_is_empty(con->out)) {
+		if (con->tls) {
+			if (con_flag(con, FLAG_IS_TLS_CONNECTED)) {
+				log_flag(CONMGR, "%s: [%s] %u pending writes to encrypt",
+					 __func__, con->name,
+					 list_count(con->out));
+				add_work_con_fifo(true, con, tls_handle_encrypt,
+						  con);
 				return 0;
+			} else {
+				log_flag(CONMGR, "%s: [%s] deferring %u pending writes to encrypt until TLS connected",
+					 __func__, con->name,
+					 list_count(con->out));
 			}
-
-			/* must wait until poll allows write of this socket */
-			log_flag(CONMGR, "%s: [%s] waiting for %u writes",
-				 __func__, con->name, list_count(con->out));
+		} else {
+			return _handle_connection_write(con, args);
 		}
-		return 0;
 	}
 
 	if (!con_flag(con, FLAG_IS_LISTEN) &&
 	    (count = list_count(con->write_complete_work))) {
 		bool queue_work = false;
 
+		xassert(!con_flag(con, FLAG_WAIT_ON_FINGERPRINT));
+		xassert(!is_tls || con_flag(con, FLAG_IS_TLS_CONNECTED));
+
 		if (con->output_fd < 0) {
 			/* output_fd is already closed so no more write()s */
 			queue_work = true;
@@ -699,11 +918,11 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		con_set_polling(con, PCTL_TYPE_CONNECTED, __func__);
 		con_unset_flag(con, FLAG_CAN_READ);
 
-		if (list_count(mgr.connections) >= mgr.max_connections) {
+		if (_is_accept_deferred()) {
 			warning("%s: [%s] Deferring incoming connection due to %d/%d connections",
-				 __func__, con->name,
-				 list_count(mgr.connections),
-				 mgr.max_connections);
+				__func__, con->name,
+				list_count(mgr.connections),
+				mgr.max_connections);
 		} else {
 			log_flag(CONMGR, "%s: [%s] listener has incoming connection",
 				 __func__, con->name);
@@ -716,19 +935,65 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 	if (!con_flag(con, FLAG_IS_LISTEN) && !con_flag(con, FLAG_READ_EOF) &&
 	    (con_flag(con, FLAG_CAN_READ) ||
 	     (con->polling_input_fd == PCTL_TYPE_UNSUPPORTED))) {
-		log_flag(CONMGR, "%s: [%s] queuing read", __func__, con->name);
-		/* reset if data has already been tried if about to read data */
+		/*
+		 * reset if data has already been tried if about to read
+		 * data
+		 */
 		con_unset_flag(con, FLAG_ON_DATA_TRIED);
-		add_work_con_fifo(true, con, handle_read, con);
+
+		if (con->tls_in) {
+			log_flag(CONMGR, "%s: [%s] queuing TLS read",
+				 __func__, con->name);
+			add_work_con_fifo(true, con, tls_handle_read, con);
+		} else {
+			log_flag(CONMGR, "%s: [%s] queuing read", __func__, con->name);
+			add_work_con_fifo(true, con, handle_read, con);
+		}
+		return 0;
+	}
+
+	if (is_tls && !con_flag(con, FLAG_IS_LISTEN) &&
+	    !con_flag(con, FLAG_ON_DATA_TRIED) &&
+	    !con_flag(con, FLAG_TLS_WAIT_ON_CLOSE)
+	    && con_flag(con, FLAG_IS_TLS_CONNECTED) && con->tls_in &&
+	    get_buf_offset(con->tls_in)) {
+		log_flag(CONMGR, "%s: [%s] queuing TLS decrypt of %u bytes",
+			 __func__, con->name, get_buf_offset(con->tls_in));
+		add_work_con_fifo(true, con, tls_handle_decrypt, con);
+		return 0;
+	}
+
+	if (!con_flag(con, FLAG_IS_LISTEN) && is_tls &&
+	    !con_flag(con, FLAG_IS_TLS_CONNECTED) &&
+	    !con_flag(con, FLAG_ON_DATA_TRIED) &&
+	    !con_flag(con, FLAG_READ_EOF)) {
+		xassert(!con_flag(con, FLAG_WAIT_ON_FINGERPRINT));
+
+		/*
+		 * TLS handshake must happen attempting to process any of the
+		 * incoming data but unwrapped data must flow both directions
+		 * until negotiations are complete
+		 */
+		log_flag(CONMGR, "%s: [%s] queuing up TLS handshake",
+			 __func__, con->name);
+		add_work_con_fifo(true, con, tls_create, NULL);
 		return 0;
 	}
 
 	/* handle already read data */
 	if (!con_flag(con, FLAG_IS_LISTEN) && get_buf_offset(con->in) &&
 	    !con_flag(con, FLAG_ON_DATA_TRIED)) {
-		log_flag(CONMGR, "%s: [%s] need to process %u bytes",
-			 __func__, con->name, get_buf_offset(con->in));
-		add_work_con_fifo(true, con, wrap_on_data, con);
+		xassert(!is_tls || con_flag(con, FLAG_IS_TLS_CONNECTED));
+
+		if (con_flag(con, FLAG_WAIT_ON_FINGERPRINT)) {
+			log_flag(CONMGR, "%s: [%s] checking for fingerprint in %u bytes",
+				 __func__, con->name, get_buf_offset(con->in));
+			add_work_con_fifo(true, con, _handle_fingerprint, con);
+		} else {
+			log_flag(CONMGR, "%s: [%s] need to process %u bytes",
+				 __func__, con->name, get_buf_offset(con->in));
+			add_work_con_fifo(true, con, wrap_on_data, con);
+		}
 		return 0;
 	}
 
@@ -737,11 +1002,11 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 
 		/* must wait until poll allows read from this socket */
 		if (con_flag(con, FLAG_IS_LISTEN)) {
-			if (list_count(mgr.connections) >= mgr.max_connections) {
+			if (_is_accept_deferred()) {
 				warning("%s: [%s] Deferring polling for new connections due to %d/%d connections",
-					 __func__, con->name,
-					 list_count(mgr.connections),
-					 mgr.max_connections);
+					__func__, con->name,
+					list_count(mgr.connections),
+					mgr.max_connections);
 
 				con_set_polling(con, PCTL_TYPE_CONNECTED,
 						__func__);
@@ -755,19 +1020,24 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 			con_set_polling(con, PCTL_TYPE_READ_ONLY, __func__);
 
 			if (con_flag(con, CON_FLAG_WATCH_READ_TIMEOUT) &&
-			    args && list_is_empty(con->write_complete_work) &&
+			    list_is_empty(con->write_complete_work) &&
 			    _handle_time_limit(args, con->last_read,
-					       mgr.conf_read_timeout)) {
+					       mgr.conf_read_timeout, "read",
+					       con->name, __func__)) {
 				_on_read_timeout(args, con);
 				return 0;
 			}
 
 			if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) {
 				char *flags = con_flags_string(con->flags);
-				log_flag(CONMGR, "%s: [%s] waiting for events: pending_read=%u pending_writes=%u work=%d write_complete_work=%d flags=%s",
+				log_flag(CONMGR, "%s: [%s] waiting for events: pending_read=%u pending_writes=%u pending_tls_read=%d pending_tls_writes=%d work=%d write_complete_work=%d flags=%s",
 					 __func__, con->name,
 					 get_buf_offset(con->in),
 					 list_count(con->out),
+					 (con->tls_in ?
+					  get_buf_offset(con->tls_in) : -1),
+					 (con->tls_out ?
+					  list_count(con->tls_out) : -1),
 					 list_count(con->work),
 					 list_count(con->write_complete_work),
 					 flags);
@@ -836,6 +1106,13 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 		return 0;
 	}
 
+	if (!con_flag(con, FLAG_IS_LISTEN) && is_tls && con->tls) {
+		log_flag(CONMGR, "%s: [%s] waiting to close TLS connection",
+			 __func__, con->name);
+		add_work_con_fifo(true, con, tls_close, NULL);
+		return 0;
+	}
+
 	/*
 	 * This connection has no more pending work or possible IO:
 	 * Remove the connection and close everything.
@@ -854,6 +1131,19 @@ static int _handle_connection(conmgr_fd_t *con, handle_connection_args_t *args)
 	return 1;
 }
 
+extern void handle_connection(bool locked, conmgr_fd_t *con)
+{
+	xassert(con->magic == MAGIC_CON_MGR_FD);
+
+	if (!locked)
+		slurm_mutex_lock(&mgr.mutex);
+
+	(void) _handle_connection(con, NULL);
+
+	if (!locked)
+		slurm_mutex_unlock(&mgr.mutex);
+}
+
 static int _list_transfer_handle_connection(void *x, void *arg)
 {
 	conmgr_fd_t *con = x;
@@ -865,12 +1155,9 @@ static int _list_transfer_handle_connection(void *x, void *arg)
 	return _handle_connection(con, args);
 }
 
-/*
- * listen socket is ready to accept
- */
-static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
+/* RET true to run again */
+static bool _attempt_accept(conmgr_fd_t *con)
 {
-	conmgr_fd_t *con = conmgr_args.con;
 	slurm_addr_t addr = {0};
 	socklen_t addrlen = sizeof(addr);
 	int input_fd = -1, fd = -1, rc = EINVAL;
@@ -884,12 +1171,18 @@ static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
 		slurm_mutex_unlock(&mgr.mutex);
 		log_flag(CONMGR, "%s: [%s] skipping accept on closed connection",
 			 __func__, con->name);
-		return;
+		return false;
 	} else if (con_flag(con, FLAG_QUIESCE)) {
 		slurm_mutex_unlock(&mgr.mutex);
 		log_flag(CONMGR, "%s: [%s] skipping accept on quiesced connection",
 			 __func__, con->name);
-		return;
+		return false;
+	} else if (_is_accept_deferred()) {
+		warning("%s: [%s] Deferring to attempt to accept new incoming connection due to %d/%d connections",
+			 __func__, con->name, list_count(mgr.connections),
+			 mgr.max_connections);
+		slurm_mutex_unlock(&mgr.mutex);
+		return false;
 	} else
 		log_flag(CONMGR, "%s: [%s] attempting to accept new connection",
 			 __func__, con->name);
@@ -905,11 +1198,12 @@ static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
 		if (errno == EINTR) {
 			log_flag(CONMGR, "%s: [%s] interrupt on accept(). Retrying.",
 				 __func__, con->name);
-			return;
+			return true;
 		}
 		if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
-			log_flag(CONMGR, "%s: [%s] retry: %m", __func__, con->name);
-			return;
+			log_flag(CONMGR, "%s: [%s] retry: %m",
+				 __func__, con->name);
+			return false;
 		}
 
 		error("%s: [%s] Error on accept socket: %m",
@@ -919,12 +1213,12 @@ static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
 		    (errno == ENOBUFS) || (errno == ENOMEM)) {
 			error("%s: [%s] retry on error: %m",
 			      __func__, con->name);
-			return;
+			return false;
 		}
 
 		/* socket is likely dead: fail out */
 		close_con(false, con);
-		return;
+		return true;
 	}
 
 	if (addrlen <= 0)
@@ -964,24 +1258,37 @@ static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
 			unix_path = usock->sun_path;
 	}
 
-	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED) {
-		log_flag(CONMGR, "%s: [%s] closing new connection to %pA during shutdown",
-				 __func__, con->name, &addr);
-		fd_close(&fd);
-		return;
-	}
-
 	/* hand over FD for normal processing */
 	if ((rc = add_connection(type, con, fd, fd, con->events,
 				 (conmgr_con_flags_t) flags, &addr, addrlen,
-				 false, unix_path, con->new_arg))) {
+				 false, unix_path, NULL, con->new_arg))) {
 		log_flag(CONMGR, "%s: [fd:%d] unable to a register new connection: %s",
 			 __func__, fd, slurm_strerror(rc));
-		return;
+		return true;
 	}
 
 	log_flag(CONMGR, "%s: [%s->fd:%d] registered newly accepted connection",
 		 __func__, con->name, fd);
+	return true;
+}
+
+/*
+ * listen socket is ready to accept
+ */
+static void _listen_accept(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	conmgr_fd_t *con = conmgr_args.con;
+
+	xassert(con->magic == MAGIC_CON_MGR_FD);
+
+	if (conmgr_args.status == CONMGR_WORK_STATUS_CANCELLED) {
+		log_flag(CONMGR, "%s: [%s] skipping accept during shutdown",
+			 __func__, con->name);
+		return;
+	}
+
+	while (_attempt_accept(con))
+		;
 }
 
 /*
@@ -998,11 +1305,7 @@ static void _inspect_connections(conmgr_callback_args_t conmgr_args, void *arg)
 	slurm_mutex_lock(&mgr.mutex);
 	xassert(mgr.inspecting);
 
-	/*
-	 * Always clear max watch sleep as it will be set before releasing lock
-	 */
-	mgr.watch_max_sleep = (struct timespec) {0};
-	args.time = timespec_now();
+	_set_time(&args);
 
 	/*
 	 * Always check mgr.connections list first to avoid
@@ -1018,16 +1321,6 @@ static void _inspect_connections(conmgr_callback_args_t conmgr_args, void *arg)
 				_list_transfer_handle_connection, &args))
 		send_signal = true;
 
-	if ((slurm_conf.debug_flags & DEBUG_FLAG_CONMGR) &&
-	    mgr.watch_max_sleep.tv_sec) {
-		char str[CTIME_STR_LEN];
-
-		timespec_ctime(mgr.watch_max_sleep, true, str, sizeof(str));
-
-		log_flag(CONMGR, "%s: set max watch sleep wait: %s",
-			 __func__, str);
-	}
-
 	mgr.inspecting = false;
 
 	if (send_signal)
@@ -1184,12 +1477,15 @@ static void _connection_fd_delete(conmgr_callback_args_t conmgr_args, void *arg)
 	conmgr_fd_t *con = arg;
 
 	xassert(con->magic == MAGIC_CON_MGR_FD);
+	xassert(!con->tls);
 
 	log_flag(CONMGR, "%s: [%s] free connection input_fd=%d output_fd=%d",
 		 __func__, con->name, con->input_fd, con->output_fd);
 
 	FREE_NULL_BUFFER(con->in);
+	FREE_NULL_BUFFER(con->tls_in);
 	FREE_NULL_LIST(con->out);
+	FREE_NULL_LIST(con->tls_out);
 	FREE_NULL_LIST(con->work);
 	FREE_NULL_LIST(con->write_complete_work);
 	xfree(con->name);
@@ -1296,17 +1592,57 @@ static int _get_quiesced_waiter_count(void)
 	return waiters;
 }
 
+/* NOTE: must hold mgr.mutex */
+static int _close_con_for_each(void *x, void *arg)
+{
+	conmgr_fd_t *con = x;
+	close_con(true, con);
+	return 1;
+}
+
+static void _on_quiesce_timeout(void)
+{
+	log_flag(CONMGR, "%s: Quiesce timed out. Closing all connections.",
+		 __func__);
+
+	/* Close all connections but not listeners */
+	list_for_each(mgr.connections, _close_con_for_each, NULL);
+}
+
+static void _quiesce_max_sleep(void)
+{
+	handle_connection_args_t args = {
+		.magic = MAGIC_HANDLE_CONNECTION,
+	};
+
+	_set_time(&args);
+
+	if (_handle_time_limit(&args, mgr.quiesce.start,
+			       mgr.quiesce.conf_timeout, "quiesce", NULL,
+			       __func__))
+		_on_quiesce_timeout();
+}
+
 static bool _watch_loop(void)
 {
 	if (mgr.shutdown_requested) {
 		signal_mgr_stop();
-		cancel_delayed_work();
+		cancel_delayed_work(false);
 		close_all_connections();
 	}
 
 	if (mgr.quiesce.requested) {
 		int waiters;
 
+		/*
+		 * Limit amount of time watch() will sleep to ensure that the
+		 * quiesce timeout is enforced
+		 */
+		_quiesce_max_sleep();
+
+		/* Cancel any delayed connection work to avoid waiting on it */
+		cancel_delayed_work(true);
+
 		if (signal_mgr_has_incoming()) {
 			/*
 			 * Must wait for any outstanding incoming signals to be
@@ -1390,6 +1726,8 @@ extern void *watch(void *arg)
 	add_work_fifo(true, signal_mgr_start, NULL);
 
 	while (_watch_loop()) {
+		char timeout_str[CTIME_STR_LEN];
+
 		if (mgr.poll_active && _is_poll_interrupt()) {
 			/*
 			 * poll() hasn't returned yet but we want to
@@ -1399,7 +1737,11 @@ extern void *watch(void *arg)
 			pollctl_interrupt(__func__);
 		}
 
-		log_flag(CONMGR, "%s: waiting for new events: workers:%d/%d work:%d delayed_work:%d connections:%d listeners:%d complete:%d polling:%c inspecting:%c shutdown_requested:%c quiesce_requested:%c waiting_on_work:%c",
+		if (slurm_conf.debug_flags & DEBUG_FLAG_CONMGR)
+			timespec_ctime(mgr.watch_max_sleep, true, timeout_str,
+				       sizeof(timeout_str));
+
+		log_flag(CONMGR, "%s: waiting for new events: workers:%d/%d work:%d delayed_work:%d connections:%d listeners:%d complete:%d polling:%c inspecting:%c shutdown_requested:%c quiesce_requested:%c waiting_on_work:%c timeout:%s",
 				 __func__, mgr.workers.active,
 				 mgr.workers.total, list_count(mgr.work),
 				 list_count(mgr.delayed_work),
@@ -1410,11 +1752,13 @@ extern void *watch(void *arg)
 				 BOOL_CHARIFY(mgr.inspecting),
 				 BOOL_CHARIFY(mgr.shutdown_requested),
 				 BOOL_CHARIFY(mgr.quiesce.requested),
-				 BOOL_CHARIFY(mgr.waiting_on_work));
+				 BOOL_CHARIFY(mgr.waiting_on_work),
+				 timeout_str);
 
 		EVENT_WAIT_TIMED(&mgr.watch_sleep, mgr.watch_max_sleep,
 				 &mgr.mutex);
 		mgr.waiting_on_work = false;
+		_reset_watch_max_sleep();
 	}
 
 	log_flag(CONMGR, "%s: returning shutdown_requested=%c connections=%u listen_conns=%u",
