@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  cgroup.c - driver for cgroup plugin
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC
- *  Written by Felip Moll <felip.moll@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -68,7 +67,7 @@ typedef struct {
         int	(*constrain_apply)	(cgroup_ctl_type_t sub,
                                          cgroup_level_t level,
                                          uint32_t task_id);
-	int	(*step_start_oom_mgr)	(void);
+	int	(*step_start_oom_mgr)	(stepd_step_rec_t *step);
 	cgroup_oom_t *(*step_stop_oom_mgr) (stepd_step_rec_t *step);
 	int	(*task_addto)		(cgroup_ctl_type_t sub,
 					 stepd_step_rec_t *step, pid_t pid,
@@ -76,6 +75,11 @@ typedef struct {
 	cgroup_acct_t *(*task_get_acct_data) (uint32_t taskid);
 	long int (*get_acct_units)	(void);
 	bool (*has_feature) (cgroup_ctl_feature_t f);
+	char *(*get_scope_path)(void);
+	int (*setup_scope)(char *scope_path);
+	int (*signal)(int signal);
+	char *(*get_task_empty_event_path)(uint32_t taskid, bool *on_modify);
+	int (*is_task_empty)(uint32_t taskid);
 } slurm_ops_t;
 
 /*
@@ -103,12 +107,18 @@ static const char *syms[] = {
 	"cgroup_p_task_get_acct_data",
 	"cgroup_p_get_acct_units",
 	"cgroup_p_has_feature",
+	"cgroup_p_get_scope_path",
+	"cgroup_p_setup_scope",
+	"cgroup_p_signal",
+	"cgroup_p_get_task_empty_event_path",
+	"cgroup_p_is_task_empty",
 };
 
 /* Local variables */
 static slurm_ops_t ops;
 static plugin_context_t *g_context = NULL;
 static pthread_mutex_t g_context_lock =	PTHREAD_MUTEX_INITIALIZER;
+static plugin_init_t plugin_inited = PLUGIN_NOT_INITED;
 
 cgroup_conf_t slurm_cgroup_conf;
 
@@ -117,6 +127,8 @@ static buf_t *cg_conf_buf = NULL;
 static bool cg_conf_inited = false;
 static bool cg_conf_exist = true;
 
+static char scope_path[PATH_MAX] = "";
+
 /* local functions */
 static void _cgroup_conf_fini();
 static void _clear_slurm_cgroup_conf();
@@ -129,9 +141,8 @@ static int _defunct_option(void **dest, slurm_parser_enum_t type,
 			  const char *key, const char *value,
 			  const char *line, char **leftover)
 {
-	if (running_in_daemon())
-		error("The option \"%s\" is defunct, please remove it from cgroup.conf.",
-		      key);
+	error_in_daemon("The option \"%s\" is defunct, please remove it from cgroup.conf.",
+			key);
 	return 0;
 }
 
@@ -151,6 +162,7 @@ static void _clear_slurm_cgroup_conf(void)
 	xfree(slurm_cgroup_conf.cgroup_mountpoint);
 	xfree(slurm_cgroup_conf.cgroup_plugin);
 	xfree(slurm_cgroup_conf.cgroup_prepend);
+	xfree(slurm_cgroup_conf.enable_extra_controllers);
 
 	memset(&slurm_cgroup_conf, 0, sizeof(slurm_cgroup_conf));
 }
@@ -173,6 +185,7 @@ static void _init_slurm_cgroup_conf(void)
 	slurm_cgroup_conf.constrain_ram_space = false;
 	slurm_cgroup_conf.constrain_swap_space = false;
 	slurm_cgroup_conf.enable_controllers = false;
+	slurm_cgroup_conf.enable_extra_controllers = NULL;
 	slurm_cgroup_conf.ignore_systemd = false;
 	slurm_cgroup_conf.ignore_systemd_on_failure = false;
 	slurm_cgroup_conf.max_ram_percent = 100;
@@ -180,6 +193,7 @@ static void _init_slurm_cgroup_conf(void)
 	slurm_cgroup_conf.memory_swappiness = NO_VAL64;
 	slurm_cgroup_conf.min_ram_space = XCGROUP_DEFAULT_MIN_RAM;
 	slurm_cgroup_conf.signal_children_processes = false;
+	slurm_cgroup_conf.systemd_timeout = 1000;
 }
 
 static void _pack_cgroup_conf(buf_t *buffer)
@@ -218,12 +232,13 @@ static void _pack_cgroup_conf(buf_t *buffer)
 	packbool(slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
 
 	packbool(slurm_cgroup_conf.enable_controllers, buffer);
+	packstr(slurm_cgroup_conf.enable_extra_controllers, buffer);
 	packbool(slurm_cgroup_conf.signal_children_processes, buffer);
+	pack64(slurm_cgroup_conf.systemd_timeout, buffer);
 }
 
 static int _unpack_cgroup_conf(buf_t *buffer)
 {
-	uint32_t uint32_tmp = 0;
 	bool tmpbool = false;
 	/*
 	 * No protocol version needed, at the time of writing we are only
@@ -237,11 +252,9 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 
 	_clear_slurm_cgroup_conf();
 
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_mountpoint,
-			       &uint32_tmp, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_mountpoint, buffer);
 
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_prepend,
-			       &uint32_tmp, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_prepend, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_cores, buffer);
 
@@ -257,14 +270,15 @@ static int _unpack_cgroup_conf(buf_t *buffer)
 	safe_unpack64(&slurm_cgroup_conf.memory_swappiness, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.constrain_devices, buffer);
-	safe_unpackstr_xmalloc(&slurm_cgroup_conf.cgroup_plugin,
-			       &uint32_tmp, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.cgroup_plugin, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd, buffer);
 	safe_unpackbool(&slurm_cgroup_conf.ignore_systemd_on_failure, buffer);
 
 	safe_unpackbool(&slurm_cgroup_conf.enable_controllers, buffer);
+	safe_unpackstr(&slurm_cgroup_conf.enable_extra_controllers, buffer);
 	safe_unpackbool(&slurm_cgroup_conf.signal_children_processes, buffer);
+	safe_unpack64(&slurm_cgroup_conf.systemd_timeout, buffer);
 
 	return SLURM_SUCCESS;
 
@@ -301,7 +315,9 @@ static void _read_slurm_cgroup_conf(void)
 		{"IgnoreSystemd", S_P_BOOLEAN},
 		{"IgnoreSystemdOnFailure", S_P_BOOLEAN},
 		{"EnableControllers", S_P_BOOLEAN},
+		{"EnableExtraControllers", S_P_STRING},
 		{"SignalChildrenProcesses", S_P_BOOLEAN},
+		{"SystemdTimeout", S_P_UINT64},
 		{NULL} };
 	s_p_hashtbl_t *tbl = NULL;
 	char *conf_path = NULL, *tmp_str;
@@ -311,8 +327,8 @@ static void _read_slurm_cgroup_conf(void)
 	/* Get the cgroup.conf path and validate the file */
 	conf_path = get_extra_conf_path("cgroup.conf");
 	if ((conf_path == NULL) || (stat(conf_path, &buf) == -1)) {
-		log_flag(CGROUP, "%s: No cgroup.conf file (%s)", __func__,
-			 conf_path);
+		info("%s: No cgroup.conf file (%s), using defaults",
+		     __func__, conf_path);
 		cg_conf_exist = false;
 	} else {
 		debug("Reading cgroup.conf file %s", conf_path);
@@ -402,10 +418,18 @@ static void _read_slurm_cgroup_conf(void)
 
 		(void) s_p_get_boolean(&slurm_cgroup_conf.enable_controllers,
 				       "EnableControllers", tbl);
+		if (s_p_get_string(&tmp_str, "EnableExtraControllers", tbl)) {
+			xfree(slurm_cgroup_conf.enable_extra_controllers);
+			slurm_cgroup_conf.enable_extra_controllers = tmp_str;
+			tmp_str = NULL;
+		}
 		(void) s_p_get_boolean(
 			&slurm_cgroup_conf.signal_children_processes,
 			"SignalChildrenProcesses", tbl);
 
+		(void) s_p_get_uint64(&slurm_cgroup_conf.systemd_timeout,
+				      "SystemdTimeout", tbl);
+
 		s_p_hashtbl_destroy(tbl);
 	}
 
@@ -487,13 +511,16 @@ extern int cgroup_conf_init(void)
 	if (!cg_conf_inited) {
 		_init_slurm_cgroup_conf();
 		_read_slurm_cgroup_conf();
-		/*
-		 * Initialize and pack cgroup.conf info into a buffer that can
-		 * be used by slurmd to send to stepd every time, instead of
-		 * re-packing every time we want to send to slurmstepd
-		 */
-		cg_conf_buf = init_buf(0);
-		_pack_cgroup_conf(cg_conf_buf);
+		if (running_in_slurmd()) {
+			/*
+			 * Initialize and pack cgroup.conf info into a buffer
+			 * that can be used by slurmd to send to stepd every
+			 * time, instead of re-packing every time we want to
+			 * send to slurmstepd
+			 */
+			cg_conf_buf = init_buf(0);
+			_pack_cgroup_conf(cg_conf_buf);
+		}
 		cg_conf_inited = true;
 	} else
 		rc = SLURM_ERROR;
@@ -508,12 +535,6 @@ extern void cgroup_conf_destroy(void)
 	_cgroup_conf_fini();
 }
 
-extern void cgroup_conf_reinit(void)
-{
-	cgroup_conf_destroy();
-	cgroup_conf_init();
-}
-
 extern void cgroup_free_limits(cgroup_limits_t *limits)
 {
 	if (!limits)
@@ -546,114 +567,110 @@ extern void cgroup_init_limits(cgroup_limits_t *limits)
  *      cgroup.conf  file and return a key pair <name,value> ordered list.
  * RET List with cgroup.conf <name,value> pairs if no error, NULL otherwise.
  */
-extern List cgroup_get_conf_list(void)
+extern list_t *cgroup_get_conf_list(void)
 {
-	config_key_pair_t *key_pair;
-	List cgroup_conf_l;
+	list_t *cgroup_conf_l;
 	cgroup_conf_t *cg_conf = &slurm_cgroup_conf;
 
 	xassert(cg_conf_inited);
 
-	slurm_rwlock_rdlock(&cg_conf_lock);
-
-	/* Fill list with cgroup config key pairs */
 	cgroup_conf_l = list_create(destroy_config_key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CgroupMountpoint");
-	key_pair->value = xstrdup(cg_conf->cgroup_mountpoint);
-	list_append(cgroup_conf_l, key_pair);
+	slurm_rwlock_rdlock(&cg_conf_lock);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainCores");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_cores ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
+	add_key_pair(cgroup_conf_l, "CgroupMountpoint", "%s",
+		     cg_conf->cgroup_mountpoint);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainCores",
+			  cg_conf->constrain_cores);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainRAMSpace",
+			  cg_conf->constrain_ram_space);
+	add_key_pair(cgroup_conf_l, "AllowedRAMSpace", "%.1f%%",
+		     cg_conf->allowed_ram_space);
+	add_key_pair(cgroup_conf_l, "MaxRAMPercent", "%.1f%%",
+		     cg_conf->max_ram_percent);
+	add_key_pair(cgroup_conf_l, "MinRAMSpace", "%"PRIu64"MB",
+		     cg_conf->min_ram_space);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainSwapSpace",
+			  cg_conf->constrain_swap_space);
+	add_key_pair(cgroup_conf_l, "AllowedSwapSpace", "%.1f%%",
+		     cg_conf->allowed_swap_space);
+	add_key_pair(cgroup_conf_l, "MaxSwapPercent", "%.1f%%",
+		     cg_conf->max_swap_percent);
+	add_key_pair_bool(cgroup_conf_l, "ConstrainDevices",
+			  cg_conf->constrain_devices);
+	add_key_pair(cgroup_conf_l, "CgroupPlugin", "%s",
+		     cg_conf->cgroup_plugin);
+	add_key_pair_bool(cgroup_conf_l, "IgnoreSystemd",
+			  cg_conf->ignore_systemd);
+	add_key_pair_bool(cgroup_conf_l, "IgnoreSystemdOnFailure",
+			  cg_conf->ignore_systemd_on_failure);
+	add_key_pair_bool(cgroup_conf_l, "EnableControllers",
+			  cg_conf->enable_controllers);
+	add_key_pair(cgroup_conf_l, "EnableExtraControllers", "%s",
+		     cg_conf->enable_extra_controllers);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainRAMSpace");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_ram_space ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedRAMSpace");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->allowed_ram_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MaxRAMPercent");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->max_ram_percent);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MinRAMSpace");
-	key_pair->value = xstrdup_printf("%"PRIu64" MB",
-					 cg_conf->min_ram_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainSwapSpace");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_swap_space ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("AllowedSwapSpace");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->allowed_swap_space);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MaxSwapPercent");
-	key_pair->value = xstrdup_printf("%.1f%%", cg_conf->max_swap_percent);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ConstrainDevices");
-	key_pair->value = xstrdup_printf("%s", cg_conf->constrain_devices ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("MemorySwappiness");
 	if (cg_conf->memory_swappiness != NO_VAL64)
-		key_pair->value = xstrdup_printf("%"PRIu64,
-						 cg_conf->memory_swappiness);
-	list_append(cgroup_conf_l, key_pair);
+		add_key_pair(cgroup_conf_l, "MemorySwappiness", "%"PRIu64,
+			     cg_conf->memory_swappiness);
+	else
+		add_key_pair(cgroup_conf_l, "MemorySwappiness", "(null)");
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CgroupPlugin");
-	key_pair->value = xstrdup(cg_conf->cgroup_plugin);
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("IgnoreSystemd");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->ignore_systemd ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("IgnoreSystemdOnFailure");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->ignore_systemd_on_failure ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("EnableControllers");
-	key_pair->value = xstrdup_printf("%s",
-					 cg_conf->enable_controllers ?
-					 "yes" : "no");
-	list_append(cgroup_conf_l, key_pair);
-
-	list_sort(cgroup_conf_l, (ListCmpF) sort_key_pairs);
+	add_key_pair(cgroup_conf_l, "SystemdTimeout", "%"PRIu64" ms",
+		     cg_conf->systemd_timeout);
 
 	slurm_rwlock_unlock(&cg_conf_lock);
 
+	list_sort(cgroup_conf_l, (ListCmpF) sort_key_pairs);
+
 	return cgroup_conf_l;
 }
 
+/*
+ * This function is called from slurmd to send the cgroup state (at present
+ * only the scope path in cgroup/v2) to the recently forked slurmstepd, since
+ * slurmstepd might not be able to infer the correct scope path when we are
+ * running into a container.
+ */
+extern int cgroup_write_state(int fd)
+{
+	int len = 0;
+	char *step_path = NULL;
+
+	if (plugin_inited == PLUGIN_INITED) {
+		step_path = (*(ops.get_scope_path))();
+		if (step_path)
+			len = strlen(step_path) + 1;
+	}
+
+	safe_write(fd, &len, sizeof(int));
+	if (step_path)
+		safe_write(fd, step_path, len);
+
+	return SLURM_SUCCESS;
+rwfail:
+	return SLURM_ERROR;
+}
+
+/*
+ * This function is called from slurmstepd before the cgroup plugin is
+ * initialized. It records the cgroup plugin state passed from slurmd
+ * (at present only the scope path in cgroup/v2) in this slurmstepd so it
+ * can be later used by the plugin when it is initialized.
+ */
+extern int cgroup_read_state(int fd)
+{
+	int len;
+
+	safe_read(fd, &len, sizeof(int));
+
+	if (len)
+		safe_read(fd, scope_path, len);
+
+	return SLURM_SUCCESS;
+rwfail:
+	return SLURM_ERROR;
+}
+
 extern int cgroup_write_conf(int fd)
 {
 	int len;
@@ -710,9 +727,10 @@ extern bool cgroup_memcg_job_confinement(void)
 	/* read cgroup configuration */
 	slurm_rwlock_rdlock(&cg_conf_lock);
 
-	if ((slurm_cgroup_conf.constrain_ram_space ||
-	     slurm_cgroup_conf.constrain_swap_space) &&
-	    xstrstr(slurm_conf.task_plugin, "cgroup"))
+	if (xstrcmp(slurm_cgroup_conf.cgroup_plugin, "disabled") &&
+	    ((slurm_cgroup_conf.constrain_ram_space ||
+	      slurm_cgroup_conf.constrain_swap_space) &&
+	     xstrstr(slurm_conf.task_plugin, "cgroup")))
 		status = true;
 
 	slurm_rwlock_unlock(&cg_conf_lock);
@@ -733,12 +751,19 @@ extern int cgroup_g_init(void)
 
 	slurm_mutex_lock(&g_context_lock);
 
-	if (g_context)
+	if (plugin_inited)
 		goto done;
 
-	cgroup_conf_init();
+	if (cgroup_conf_init() != SLURM_SUCCESS)
+		log_flag(CGROUP, "cgroup conf was already initialized.");
+
 	type = slurm_cgroup_conf.cgroup_plugin;
 
+	if (!xstrcmp(type, "disabled")) {
+		plugin_inited = PLUGIN_NOOP;
+		goto done;
+	}
+
 	if (!xstrcmp(type, "autodetect")) {
 		if (!(type = autodetect_cgroup_version())) {
 			rc = SLURM_ERROR;
@@ -746,15 +771,31 @@ extern int cgroup_g_init(void)
 		}
 	}
 
+	if (running_in_slurmd())
+		if (!xstrcmp(type, "cgroup/v1"))
+			warning("cgroup/v1 plugin is deprecated, please upgrade to cgroup/v2 at your earliest convenience");
+
 	g_context = plugin_context_create(
 		plugin_type, type, (void **)&ops, syms, sizeof(syms));
 
 	if (!g_context) {
 		error("cannot create %s context for %s", plugin_type, type);
 		rc = SLURM_ERROR;
+		plugin_inited = PLUGIN_NOT_INITED;
 		goto done;
 	}
 
+	/*
+	 * We have recorded the scope_path here previously, configure it now in
+	 * the plugin.
+	 */
+	rc = (*(ops.setup_scope))(scope_path);
+	if (rc == SLURM_ERROR) {
+		error("cannot setup the scope for %s", plugin_type);
+		goto done;
+	}
+
+	plugin_inited = PLUGIN_INITED;
 done:
 	slurm_mutex_unlock(&g_context_lock);
 
@@ -763,102 +804,172 @@ done:
 
 extern int cgroup_g_fini(void)
 {
-	int rc;
-
-	if (!g_context)
-		return SLURM_SUCCESS;
+	int rc = SLURM_SUCCESS;
 
 	slurm_mutex_lock(&g_context_lock);
-	rc = plugin_context_destroy(g_context);
-	g_context = NULL;
-	slurm_mutex_unlock(&g_context_lock);
+	if (g_context) {
+		rc = plugin_context_destroy(g_context);
+		g_context = NULL;
+	}
 
 	cgroup_conf_destroy();
+	plugin_inited = PLUGIN_NOT_INITED;
 
+	slurm_mutex_unlock(&g_context_lock);
 	return rc;
 }
 
 extern int cgroup_g_initialize(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		fatal("%s: Trying to initialize cgroups but CgroupPlugin=disabled is set in cgroup.conf. Please, unset any configuration that is using cgroups.",
+		      __func__);
 
 	return (*(ops.initialize))(sub);
 }
 
 extern int cgroup_g_system_create(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_create))(sub);
 }
 
 extern int cgroup_g_system_addto(cgroup_ctl_type_t sub, pid_t *pids, int npids)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_addto))(sub, pids, npids);
 }
 
 extern int cgroup_g_system_destroy(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.system_destroy))(sub);
 }
 
 extern int cgroup_g_step_create(cgroup_ctl_type_t sub, stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.step_create))(sub, step);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.step_create))(sub, step);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern int cgroup_g_step_addto(cgroup_ctl_type_t sub, pid_t *pids, int npids)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.step_addto))(sub, pids, npids);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.step_addto))(sub, pids, npids);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern int cgroup_g_step_get_pids(pid_t **pids, int *npids)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.step_get_pids))(pids, npids);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		*npids = 0;
+		*pids = NULL;
+		return SLURM_SUCCESS;
+	}
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.step_get_pids))(pids, npids);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern int cgroup_g_step_suspend(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_suspend))();
 }
 
 extern int cgroup_g_step_resume(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.step_resume))();
 }
 
 extern int cgroup_g_step_destroy(cgroup_ctl_type_t sub)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.step_destroy))(sub);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.step_destroy))(sub);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern bool cgroup_g_has_pid(pid_t pid)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.has_pid))(pid);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return false;
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.has_pid))(pid);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern cgroup_limits_t *cgroup_g_constrain_get(cgroup_ctl_type_t sub,
 					       cgroup_level_t level)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return NULL;
 
 	return (*(ops.constrain_get))(sub, level);
 }
@@ -866,7 +977,10 @@ extern cgroup_limits_t *cgroup_g_constrain_get(cgroup_ctl_type_t sub,
 extern int cgroup_g_constrain_set(cgroup_ctl_type_t sub, cgroup_level_t level,
 				  cgroup_limits_t *limits)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.constrain_set))(sub, level, limits);
 }
@@ -874,21 +988,32 @@ extern int cgroup_g_constrain_set(cgroup_ctl_type_t sub, cgroup_level_t level,
 extern int cgroup_g_constrain_apply(cgroup_ctl_type_t sub, cgroup_level_t level,
                                     uint32_t task_id)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	return (*(ops.constrain_apply))(sub, level, task_id);
 }
 
-extern int cgroup_g_step_start_oom_mgr(void)
+extern int cgroup_g_step_start_oom_mgr(stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
 
-	return (*(ops.step_start_oom_mgr))();
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.step_start_oom_mgr))(step);
 }
 
 extern cgroup_oom_t *cgroup_g_step_stop_oom_mgr(stepd_step_rec_t *step)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		cgroup_oom_t *empty_oom = xmalloc(sizeof(*empty_oom));
+		return empty_oom;
+	}
 
 	return (*(ops.step_stop_oom_mgr))(step);
 }
@@ -896,28 +1021,79 @@ extern cgroup_oom_t *cgroup_g_step_stop_oom_mgr(stepd_step_rec_t *step)
 extern int cgroup_g_task_addto(cgroup_ctl_type_t sub, stepd_step_rec_t *step,
 			       pid_t pid, uint32_t task_id)
 {
-	xassert(g_context);
+	int rc;
 
-	return (*(ops.task_addto))(sub, step, pid, task_id);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	slurm_mutex_lock(&g_context_lock);
+	rc = (*(ops.task_addto))(sub, step, pid, task_id);
+	slurm_mutex_unlock(&g_context_lock);
+
+	return rc;
 }
 
 extern cgroup_acct_t *cgroup_g_task_get_acct_data(uint32_t taskid)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP) {
+		cgroup_acct_t *empty_acct = xmalloc(sizeof(*empty_acct));
+		return empty_acct;
+	}
 
 	return (*(ops.task_get_acct_data))(taskid);
 }
 
 extern long int cgroup_g_get_acct_units(void)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return (long int)USEC_IN_SEC;
 
 	return (*(ops.get_acct_units))();
 }
 
 extern bool cgroup_g_has_feature(cgroup_ctl_feature_t f)
 {
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return false;
 
 	return (*(ops.has_feature))(f);
 }
+
+extern int cgroup_g_signal(int signal)
+{
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.signal))(signal);
+}
+
+extern char *cgroup_g_get_task_empty_event_path(uint32_t taskid,
+						bool *on_modify)
+{
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.get_task_empty_event_path))(taskid, on_modify);
+}
+
+extern int cgroup_g_is_task_empty(uint32_t taskid)
+{
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
+
+	return (*(ops.is_task_empty))(taskid);
+}
