@@ -95,6 +95,9 @@ extern char *job_table;
 extern char *job_env_table;
 extern char *job_script_table;
 extern char *last_ran_table;
+extern char *qos_day_table;
+extern char *qos_hour_table;
+extern char *qos_month_table;
 extern char *qos_table;
 extern char *resv_table;
 extern char *res_table;
@@ -107,11 +110,13 @@ extern char *wckey_hour_table;
 extern char *wckey_month_table;
 extern char *wckey_table;
 
+extern list_t *g_user_coords_list;
+
 /* Since tables are cluster centric we have a global cluster list to
  * go off of.
  */
-extern List as_mysql_cluster_list;
-extern List as_mysql_total_cluster_list;
+extern list_t *as_mysql_cluster_list;
+extern list_t *as_mysql_total_cluster_list;
 extern pthread_rwlock_t as_mysql_cluster_list_lock;
 
 extern bool backup_dbd;
@@ -122,6 +127,24 @@ typedef enum {
 	QOS_LEVEL_MODIFY
 } qos_level_t;
 
+typedef struct {
+	char *assoc_char;
+	char *cluster_name;
+	time_t day_old;
+	bool default_account;
+	bool has_jobs;
+	bool jobs_running;
+	mysql_conn_t *mysql_conn;
+	char *name_char;
+	time_t now;
+	int *rc_ptr;
+	list_t *ret_list;
+	char *table;
+	uint16_t type;
+	list_t *use_cluster_list;
+	char *user_name;
+} remove_common_args_t;
+
 #define DB_DEBUG(flag, conn, fmt, ...) \
 	log_flag(flag, "%d(%s:%d) "fmt, conn, THIS_FILE, __LINE__, ##__VA_ARGS__);
 
@@ -138,6 +161,10 @@ extern int setup_assoc_limits(slurmdb_assoc_rec_t *assoc,
 				    char **cols, char **vals,
 				    char **extra, qos_level_t qos_level,
 				    bool for_add);
+extern int setup_assoc_limits_locked(slurmdb_assoc_rec_t *assoc,
+				     char **cols, char **vals,
+				     char **extra, qos_level_t qos_level,
+				     bool for_add);
 extern int modify_common(mysql_conn_t *mysql_conn,
 			 uint16_t type,
 			 time_t now,
@@ -146,17 +173,7 @@ extern int modify_common(mysql_conn_t *mysql_conn,
 			 char *cond_char,
 			 char *vals,
 			 char *cluster_name);
-extern int remove_common(mysql_conn_t *mysql_conn,
-			 uint16_t type,
-			 time_t now,
-			 char *user_name,
-			 char *table,
-			 char *name_char,
-			 char *assoc_char,
-			 char *cluster_name,
-			 List ret_list,
-			 bool *jobs_running,
-			 bool *default_account);
+extern int remove_common(remove_common_args_t *args);
 
 extern void mod_tres_str(char **out, char *mod, char *cur,
 			 char *cur_par, char *name, char **vals,
@@ -177,19 +194,18 @@ extern int get_cluster_dims(mysql_conn_t *mysql_conn, char *cluster_name,
 /*local api functions */
 extern int acct_storage_p_commit(mysql_conn_t *mysql_conn, bool commit);
 
-extern int acct_storage_p_add_assocs(mysql_conn_t *mysql_conn,
-					   uint32_t uid,
-					   List assoc_list);
+extern int acct_storage_p_add_assocs(mysql_conn_t *mysql_conn, uint32_t uid,
+				     list_t *assoc_list);
 
 extern int acct_storage_p_add_wckeys(mysql_conn_t *mysql_conn, uint32_t uid,
-				     List wckey_list);
+				     list_t *wckey_list);
 
-extern List acct_storage_p_get_assocs(
+extern list_t *acct_storage_p_get_assocs(
 	mysql_conn_t *mysql_conn, uid_t uid,
 	slurmdb_assoc_cond_t *assoc_cond);
 
-extern List acct_storage_p_get_wckeys(mysql_conn_t *mysql_conn, uid_t uid,
-				      slurmdb_wckey_cond_t *wckey_cond);
+extern list_t *acct_storage_p_get_wckeys(mysql_conn_t *mysql_conn, uid_t uid,
+					 slurmdb_wckey_cond_t *wckey_cond);
 
 extern int acct_storage_p_get_usage(mysql_conn_t *mysql_conn, uid_t uid,
 				    void *in, slurmdbd_msg_type_t type,
@@ -200,12 +216,12 @@ extern int clusteracct_storage_p_get_usage(
 	slurmdb_cluster_rec_t *cluster_rec,  slurmdbd_msg_type_t type,
 	time_t start, time_t end);
 
-extern List acct_storage_p_remove_coord(mysql_conn_t *mysql_conn, uint32_t uid,
-					List acct_list,
-					slurmdb_user_cond_t *user_cond);
+extern list_t *acct_storage_p_remove_coord(mysql_conn_t *mysql_conn,
+					   uint32_t uid, list_t *acct_list,
+					   slurmdb_user_cond_t *user_cond);
 
-extern List acct_storage_p_remove_wckeys(mysql_conn_t *mysql_conn,
-					 uint32_t uid,
-					 slurmdb_wckey_cond_t *wckey_cond);
+extern list_t *acct_storage_p_remove_wckeys(mysql_conn_t *mysql_conn,
+					    uint32_t uid,
+					    slurmdb_wckey_cond_t *wckey_cond);
 
 #endif
