@@ -45,7 +45,7 @@ static int _setup_assoc_cond_limits(
 	char **extra, bool user_query)
 {
 	int set = 0;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *object = NULL;
 
 	xstrfmtcat(*extra, "where t1.deleted=0");
@@ -106,14 +106,14 @@ static int _setup_assoc_cond_limits(
 
 extern int as_mysql_acct_no_assocs(mysql_conn_t *mysql_conn,
 				   slurmdb_assoc_cond_t *assoc_cond,
-				   List ret_list)
+				   list_t *ret_list)
 {
 	int rc = SLURM_SUCCESS;
 	char *query = NULL;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
-	List use_cluster_list = NULL;
-	ListIterator itr = NULL;
+	list_t *use_cluster_list = NULL;
+	list_itr_t *itr = NULL;
 	char *cluster_name = NULL;
 	bool locked = false;
 
@@ -124,7 +124,7 @@ extern int as_mysql_acct_no_assocs(mysql_conn_t *mysql_conn,
 	if (assoc_cond &&
 	    assoc_cond->acct_list && list_count(assoc_cond->acct_list)) {
 		int set = 0;
-		ListIterator itr = NULL;
+		list_itr_t *itr = NULL;
 		char *object = NULL;
 		xstrcat(query, " && (");
 		itr = list_iterator_create(assoc_cond->acct_list);
@@ -206,7 +206,7 @@ extern int as_mysql_acct_no_assocs(mysql_conn_t *mysql_conn,
 
 extern int as_mysql_acct_no_users(mysql_conn_t *mysql_conn,
 				  slurmdb_assoc_cond_t *assoc_cond,
-				  List ret_list)
+				  list_t *ret_list)
 {
 	int rc = SLURM_SUCCESS;
 	char *query = NULL, *tmp = NULL;
@@ -214,8 +214,8 @@ extern int as_mysql_acct_no_users(mysql_conn_t *mysql_conn,
 	int i = 0;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
-	List use_cluster_list = NULL;
-	ListIterator itr = NULL;
+	list_t *use_cluster_list = NULL;
+	list_itr_t *itr = NULL;
 	char *cluster_name;
 	bool locked = false;
 
@@ -313,14 +313,14 @@ extern int as_mysql_acct_no_users(mysql_conn_t *mysql_conn,
 extern int as_mysql_user_no_assocs_or_no_uid(
 	mysql_conn_t *mysql_conn,
 	slurmdb_assoc_cond_t *assoc_cond,
-	List ret_list)
+	list_t *ret_list)
 {
 	int rc = SLURM_SUCCESS;
 	char *query = NULL;
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
-	List use_cluster_list = NULL;
-	ListIterator itr = NULL;
+	list_t *use_cluster_list = NULL;
+	list_itr_t *itr = NULL;
 	char *cluster_name = NULL;
 	bool locked = false;
 
@@ -367,7 +367,7 @@ extern int as_mysql_user_no_assocs_or_no_uid(
 		slurmdb_assoc_rec_t *assoc = NULL;
 		uid_t pw_uid;
 
-		if (uid_from_string (row[0], &pw_uid) < 0) {
+		if (uid_from_string(row[0], &pw_uid) != SLURM_SUCCESS) {
 			assoc =	xmalloc(sizeof(slurmdb_assoc_rec_t));
 			list_append(ret_list, assoc);
 
