@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  pem_key.c - Build a PEM-formatted RSA public key from mod and exp
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC.
- *  Written by Tim Wickberg <tim@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -62,48 +61,17 @@ static void _handle_prepend(char **string)
 	}
 }
 
-static char *_to_base64_from_base64url(char *in)
-{
-	char *out;
-	int i;
-
-	/* extra padding in case the padding was stripped off */
-	out = xmalloc(strlen(in) + 3);
-
-	for (i = 0; i < strlen(in); i++) {
-		switch (in[i]) {
-		case '-':
-			out[i] = '+';
-			break;
-		case '_':
-			out[i] = '/';
-			break;
-		default:
-			out[i] = in[i];
-		}
-	}
-
-	/*
-	 * Fix padding in case it was trimmed.
-	 * String length must be a multiple of 4.
-	 */
-	for (int padding = 4 - (i % 4); padding < 4 && padding; padding--)
-		out[i++] = '=';
-
-	return out;
-}
-
 /*
  * Convert base64url encoded value to DER formatted hex.
  * Returns an xmalloc()'d string.
  */
-static char *_to_hex(char *base64url)
+static char *_to_hex(const char *base64url)
 {
 	char *base64, *hex;
 	unsigned char *bin;
 	int binlen;
 
-	base64 = _to_base64_from_base64url(base64url);
+	base64 = xbase64_from_base64url(base64url);
 
 	/*
 	 * The binary format is ~33% smaller,
@@ -194,7 +162,7 @@ static int _to_bin(char **bin, char *hex)
  *
  * mod and exp need to be input in base64url format
  */
-extern char *pem_from_mod_exp(char *mod, char *exp)
+extern char *pem_from_mod_exp(const char *mod, const char *exp)
 {
 	int modbytes, expbytes, binkeylen;
 	char *modhex, *exphex;
