@@ -133,8 +133,8 @@ static void _set_flag_bit_equal(const parser_t *const parser, void *dst,
 {
 	/* C allows compiler to choose a size for the enum
 	 *
-	 * If the comparsion is all or nothing, then clear all the masked bits
-	 * if it doesnt match which means parser order matters with these.
+	 * If the comparison is all or nothing, then clear all the masked bits
+	 * if it doesn't match which means parser order matters with these.
 	 */
 	if (parser->size == sizeof(uint64_t)) {
 		uint64_t *flags = dst;
@@ -353,22 +353,21 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 {
 	int rc = SLURM_SUCCESS;
 	char *path = NULL;
-	list_t **list_ptr = dst;
+	list_t **list = dst;
 	foreach_list_t list_args = {
 		.magic = MAGIC_FOREACH_LIST,
 		.dlist = NULL,
+		.list = *list,
 		.args = args,
 		.parser = parser,
 		.parent_path = parent_path,
 		.index = -1,
 	};
 
-	xassert(!*list_ptr || (list_count(*list_ptr) >= 0));
+	xassert(!*list || (list_count(*list) >= 0));
 	xassert(args->magic == MAGIC_ARGS);
 	check_parser(parser);
 
-	SWAP(*list_ptr, list_args.list);
-
 	log_flag(DATA, "%s: BEGIN: list parsing %s{%s(0x%"PRIxPTR")} to List 0x%"PRIxPTR" via parser %s(0x%"PRIxPTR")",
 		__func__, set_source_path(&path, args, parent_path),
 		data_get_type_string(src), (uintptr_t) src,
@@ -400,10 +399,10 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 			      data_get_type_string(src));
 	}
 
-	if (!rc)
-		SWAP(*list_ptr, list_args.list);
-	else
-		FREE_NULL_LIST(list_args.list);
+	if (!rc) {
+		*list = list_args.list;
+		list_args.list = NULL;
+	}
 
 	log_flag(DATA, "%s: END: list parsing %s{%s(0x%"PRIxPTR")} to List 0x%"PRIxPTR" via parser %s(0x%"PRIxPTR") rc[%d]:%s",
 		__func__, path, data_get_type_string(src),
@@ -411,6 +410,9 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 		(uintptr_t) parser, rc, slurm_strerror(rc)
 	);
 
+	if (rc)
+		*list = NULL;
+	FREE_NULL_LIST(list_args.list);
 	xfree(path);
 	return rc;
 }
@@ -436,7 +438,7 @@ static int _parse_pointer(const parser_t *const parser, void *dst, data_t *src,
 		 * Detect work around for OpenAPI clients being unable to handle
 		 * a null in place of a object/array by placing an empty
 		 * dict/array. Place the default allocated object pointer but
-+                * skip attempting to parse.
+		 * skip attempting to parse.
 		 */
 		return SLURM_SUCCESS;
 	}
@@ -1107,7 +1109,7 @@ static bool _match_flag_bit(const parser_t *const parser, void *src,
 	if (used_equal_bits & bit->mask)
 		return false;
 
-	/* C allows complier to choose a size for the enum */
+	/* C allows compiler to choose a size for the enum */
 	if (parser->size == sizeof(uint64_t)) {
 		uint64_t *flags = src;
 		return ((*flags & v) == v);
@@ -1328,7 +1330,7 @@ static int _dump_pointer(const parser_t *const field_parser,
 		    (field_parser && !field_parser->required)) {
 			xassert(data_get_type(dst) == DATA_TYPE_NULL);
 		} else if ((pt->model == PARSER_MODEL_ARRAY) ||
-		    (pt->obj_openapi == OPENAPI_FORMAT_OBJECT)) {
+			   (pt->obj_openapi == OPENAPI_FORMAT_OBJECT)) {
 			/*
 			 * OpenAPI clients can't handle a null instead of an
 			 * object. Work around by placing an empty dictionary
@@ -1368,8 +1370,8 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 			return SLURM_SUCCESS;
 
 		for (int i = 0; !rc && array[i]; i++) {
-			rc = dump(array[i], NO_VAL,
-				  NULL, find_parser_by_type(parser->array_type),
+			rc = dump(array[i], NO_VAL, NULL,
+				  find_parser_by_type(parser->array_type),
 				  data_list_append(dst), args);
 		}
 	} else if (parser->model == PARSER_MODEL_NT_ARRAY) {
