@@ -44,7 +44,6 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/mman.h>
-#include <sched.h>
 #include <fcntl.h>
 #include <sys/mount.h>
 #include <semaphore.h>
@@ -60,6 +59,7 @@
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
+#include "src/interfaces/switch.h"
 #include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 #include "read_jcconf.h"
@@ -105,7 +105,7 @@ static bool _is_plugin_disabled(char *basepath)
 	return ((!basepath) || (!xstrncasecmp(basepath, "none", 4)));
 }
 
-static int _restore_ns(List steps, const char *d_name)
+static int _restore_ns(list_t *steps, const char *d_name)
 {
 	char *endptr;
 	int fd;
@@ -141,11 +141,6 @@ static int _restore_ns(List steps, const char *d_name)
 	return SLURM_SUCCESS;
 }
 
-extern void container_p_reconfig(void)
-{
-	return;
-}
-
 /*
  * init() is called when the plugin is loaded, before any other functions
  *	are called.  Put global initialization here.
@@ -180,10 +175,6 @@ extern int fini(void)
 	int rc = SLURM_SUCCESS;
 	debug("%s unloaded", plugin_name);
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
 	if (step_ns_fd != -1) {
 		close(step_ns_fd);
 		step_ns_fd = -1;
@@ -199,13 +190,9 @@ extern int container_p_restore(char *dir_name, bool recover)
 {
 	DIR *dp;
 	struct dirent *ep;
-	List steps;
+	list_t *steps;
 	int rc = SLURM_SUCCESS;
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
 	if (plugin_disabled)
 		return SLURM_SUCCESS;
 
@@ -371,6 +358,43 @@ static int _clean_job_basepath(uint32_t job_id)
 	return SLURM_SUCCESS;
 }
 
+static char **_setup_script_env(uint32_t job_id,
+				stepd_step_rec_t *step,
+				char *src_bind,
+				char *ns_holder)
+{
+	char **env = env_array_create();
+
+	env_array_overwrite_fmt(&env, "SLURM_JOB_ID", "%u", job_id);
+	env_array_overwrite_fmt(&env, "SLURM_CONF", "%s", conf->conffile);
+	env_array_overwrite_fmt(&env, "SLURMD_NODENAME", "%s", conf->node_name);
+	if (src_bind)
+		env_array_overwrite_fmt(&env, "SLURM_JOB_MOUNTPOINT_SRC", "%s",
+					src_bind);
+	if (step) {
+		if (step->het_job_id && (step->het_job_id != NO_VAL))
+			env_array_overwrite_fmt(&env, "SLURM_HET_JOB_ID", "%u",
+						step->het_job_id);
+		env_array_overwrite_fmt(&env, "SLURM_JOB_GID", "%u",
+					step->gid);
+		env_array_overwrite_fmt(&env, "SLURM_JOB_UID", "%u",
+					step->uid);
+		env_array_overwrite_fmt(&env, "SLURM_JOB_USER", "%s",
+					step->user_name);
+		if (step->alias_list)
+			env_array_overwrite_fmt(&env, "SLURM_NODE_ALIASES",
+						"%s", step->alias_list);
+		if (step->cwd)
+			env_array_overwrite_fmt(&env, "SLURM_JOB_WORK_DIR",
+						"%s", step->cwd);
+	}
+
+	if (ns_holder)
+		env_array_overwrite_fmt(&env, "SLURM_NS", "%s", ns_holder);
+
+	return env;
+}
+
 static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 {
 	char *job_mount = NULL, *ns_holder = NULL, *src_bind = NULL;
@@ -381,10 +405,6 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 	sem_t *sem2 = NULL;
 	pid_t cpid;
 
-#ifdef HAVE_NATIVE_CRAY
-	return 0;
-#endif
-
 	_create_paths(job_id, &job_mount, &ns_holder, &src_bind);
 
 	if (mkdir(job_mount, 0700)) {
@@ -422,52 +442,25 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 	/* run any initialization script- if any*/
 	if (jc_conf->initscript) {
 		run_command_args_t run_command_args = {
-			.max_wait = 10000,
+			.max_wait = 10 * MSEC_IN_SEC,
 			.script_path = jc_conf->initscript,
 			.script_type = "initscript",
 			.status = &rc,
 		};
-		run_command_args.env = env_array_create();
-		if (step->het_job_id && (step->het_job_id != NO_VAL))
-			env_array_overwrite_fmt(&run_command_args.env,
-						"SLURM_HET_JOB_ID", "%u",
-						step->het_job_id);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_JOB_GID", "%u",
-					step->gid);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_JOB_ID", "%u", job_id);
+		run_command_args.env = _setup_script_env(job_id, step,
+							 src_bind, NULL);
 
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_JOB_MOUNTPOINT_SRC", "%s",
-					src_bind);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_JOB_UID", "%u",
-					step->uid);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_JOB_USER", "%s",
-					step->user_name);
-		if (step->cwd)
-			env_array_overwrite_fmt(&run_command_args.env,
-						"SLURM_JOB_WORK_DIR", "%s",
-						step->cwd);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURM_CONF", "%s",
-					slurm_conf.slurm_conf);
-		env_array_overwrite_fmt(&run_command_args.env,
-					"SLURMD_NODENAME", "%s",
-					conf->node_name);
+		log_flag(JOB_CONT, "Running InitScript");
 		result = run_command(&run_command_args);
+		log_flag(JOB_CONT, "InitScript rc: %d, stdout: %s", rc, result);
 		env_array_free(run_command_args.env);
-		if (rc) {
-			error("%s: init script: %s failed",
-			      __func__, jc_conf->initscript);
-			xfree(result);
-			goto exit2;
-		} else {
-			log_flag(JOB_CONT, "initscript stdout: %s", result);
-		}
 		xfree(result);
+
+		if (rc) {
+			error("%s: InitScript: %s failed with rc: %d",
+			      __func__, jc_conf->initscript, rc);
+			goto exit2;
+		}
 	}
 
 	rc = mkdir(src_bind, 0700);
@@ -575,6 +568,16 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 			goto child_exit;
 		}
 
+		/*
+		 * switch/nvidia_imex needs to create an ephemeral device
+		 * node under /dev in this new namespace.
+		 */
+		if ((rc = switch_g_fs_init(step))) {
+			error("%s: switch_g_fs_init failed", __func__);
+			rc = -1;
+			goto child_exit;
+		}
+
 		/*
 		 * This umount is to remove the basepath mount from being
 		 * visible inside the namespace. So if a user looks up the
@@ -641,6 +644,31 @@ static int _create_ns(uint32_t job_id, stepd_step_rec_t *step)
 		rc = 0;
 	}
 
+	/* run any post clone initialization script */
+	if (jc_conf->clonensscript) {
+		run_command_args_t run_command_args = {
+			.max_wait = jc_conf->clonensscript_wait * MSEC_IN_SEC,
+			.script_path = jc_conf->clonensscript,
+			.script_type = "clonensscript",
+			.status = &rc,
+		};
+		run_command_args.env = _setup_script_env(job_id, step,
+							 src_bind, ns_holder);
+
+		log_flag(JOB_CONT, "Running CloneNSScript");
+		result = run_command(&run_command_args);
+		log_flag(JOB_CONT, "CloneNSScript rc: %d, stdout: %s",
+			 rc, result);
+		xfree(result);
+		env_array_free(run_command_args.env);
+
+		if (rc) {
+			error("%s: CloneNSScript %s failed with rc=%d",
+			      __func__, jc_conf->clonensscript, rc);
+			goto exit2;
+		}
+	}
+
 exit1:
 	sem_destroy(sem1);
 	munmap(sem1, sizeof(*sem1));
@@ -672,11 +700,6 @@ end_it:
 	return rc;
 }
 
-extern int container_p_create(uint32_t job_id, uid_t uid)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int container_p_join_external(uint32_t job_id)
 {
 	char *job_mount = NULL, *ns_holder = NULL;
@@ -698,32 +721,36 @@ extern int container_p_join_external(uint32_t job_id)
 	return step_ns_fd;
 }
 
-extern int container_p_add_cont(uint32_t job_id, uint64_t cont_id)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int container_p_join(uint32_t job_id, uid_t uid)
+extern int container_p_join(slurm_step_id_t *step_id, uid_t uid,
+			    bool step_create)
 {
 	char *job_mount = NULL, *ns_holder = NULL;
 	int fd;
 	int rc = SLURM_SUCCESS;
 
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
-
 	if (plugin_disabled)
 		return SLURM_SUCCESS;
 
+	/*
+	 * Handle EntireStepInNS setting. If set, the join needs to happen
+	 * during the fork+exec chain that creates the slurmstepd process, and
+	 * all successive calls within slurmstepd need to be skipped. If not
+	 * set, do the opposite.
+	 */
+	if ((!jc_conf->entire_step_in_ns && running_in_slurmd() &&
+	     step_create) ||
+	    (jc_conf->entire_step_in_ns && running_in_slurmstepd() &&
+	     step_id->step_id != SLURM_EXTERN_CONT))
+		return SLURM_SUCCESS;
+
 	/*
 	 * Jobid 0 means we are not a real job, but a script running instead we
 	 * do not need to handle this request.
 	 */
-	if (job_id == 0)
+	if (step_id->job_id == 0)
 		return SLURM_SUCCESS;
 
-	_create_paths(job_id, &job_mount, &ns_holder, NULL);
+	_create_paths(step_id->job_id, &job_mount, &ns_holder, NULL);
 
 	/* This is called on the slurmd so we can't use ns_fd. */
 	fd = open(ns_holder, O_RDONLY);
@@ -743,7 +770,7 @@ extern int container_p_join(uint32_t job_id, uid_t uid)
 		xfree(ns_holder);
 		return SLURM_ERROR;
 	} else {
-		log_flag(JOB_CONT, "job %u entered namespace", job_id);
+		log_flag(JOB_CONT, "job %u entered namespace", step_id->job_id);
 	}
 
 	close(fd);
@@ -757,13 +784,34 @@ static int _delete_ns(uint32_t job_id)
 {
 	char *job_mount = NULL, *ns_holder = NULL;
 	int rc = 0, failures = 0;
-
-#ifdef HAVE_NATIVE_CRAY
-	return SLURM_SUCCESS;
-#endif
+	char *result = NULL;
 
 	_create_paths(job_id, &job_mount, &ns_holder, NULL);
 
+	/* run any post clone epilog script */
+	/* initialize environ variable to include jobid and namespace file */
+	if (jc_conf->clonensepilog) {
+		run_command_args_t run_command_args = {
+			.max_wait = jc_conf->clonensepilog_wait * MSEC_IN_SEC,
+			.script_path = jc_conf->clonensepilog,
+			.script_type = "clonensepilog",
+			.status = &rc,
+		};
+		run_command_args.env = _setup_script_env(job_id, NULL,
+							 NULL, ns_holder);
+		log_flag(JOB_CONT, "Running CloneNSEpilog");
+		result = run_command(&run_command_args);
+		env_array_free(run_command_args.env);
+		log_flag(JOB_CONT, "CloneNSEpilog rc: %d, stdout: %s",
+			 rc, result);
+		xfree(result);
+
+		if (rc) {
+			error("%s: CloneNSEpilog script %s failed with rc=%d",
+			      __func__, jc_conf->clonensepilog, rc);
+		}
+	}
+
 	errno = 0;
 
 	/*
@@ -813,11 +861,6 @@ static int _delete_ns(uint32_t job_id)
 	return SLURM_SUCCESS;
 }
 
-extern int container_p_delete(uint32_t job_id)
-{
-	return SLURM_SUCCESS;
-}
-
 extern int container_p_stepd_create(uint32_t job_id, stepd_step_rec_t *step)
 {
 	if (plugin_disabled)
