@@ -49,7 +49,12 @@
 typedef struct slurm_jc_conf {
 	bool auto_basepath;
 	char *basepath;
+	char *clonensscript;
+	char *clonensepilog;
+	uint32_t clonensscript_wait;
+	uint32_t clonensepilog_wait;
 	char *dirs;
+	bool entire_step_in_ns;
 	char *initscript;
 	bool shared;
 } slurm_jc_conf_t;
@@ -59,7 +64,7 @@ extern char *tmpfs_conf_file;
 /*
  * Init the job_container/tmpfs config if required.
  *
- * Return a pointer to the config structure if succesful or NULL on error.
+ * Return a pointer to the config structure if successful or NULL on error.
  */
 extern slurm_jc_conf_t *init_slurm_jc_conf(void);
 
