@@ -38,7 +38,12 @@
 
 #include "src/slurmctld/slurmctld.h"
 
-extern buf_t *jobcomp_common_load_state_file(char *state_file);
-extern data_t *jobcomp_common_job_record_to_data(job_record_t *job_ptr);
+extern void jobcomp_common_conf_init(void);
+extern void jobcomp_common_conf_fini(void);
+
+extern char *jobcomp_common_get_event_name(uint32_t event);
+extern data_t *jobcomp_common_job_record_to_data(job_record_t *job_ptr,
+						 uint32_t event);
+extern uint32_t jobcomp_common_parse_enabled_events(void);
 
 #endif
