@@ -85,8 +85,7 @@ extern void fini(void)
 	if (backfill_thread) {
 		verbose("Backfill scheduler plugin shutting down");
 		stop_backfill_agent();
-		pthread_join(backfill_thread, NULL);
-		backfill_thread = 0;
+		slurm_thread_join(backfill_thread);
 	}
 	slurm_mutex_unlock(&thread_flag_mutex);
 }
