@@ -79,6 +79,10 @@
 #define SLINGSHOT_RDZV_GET_EN_DEFAULT_FMT \
 	"/sys/module/cxi_%s/parameters/rdzv_get_en_default"
 
+/* RGID sharing: max_lnis_per_rgid value found here - insert device name */
+#define SLINGSHOT_RGIDS_AVAIL_FMT \
+	"/sys/class/cxi/%s/device/properties/rgids_avail"
+
 extern int free_vnis; /* Number of free VNIs */
 
 /* Set of valid auth types used for REST */
@@ -125,7 +129,7 @@ typedef struct slingshot_state {
 #define SLINGSHOT_TLE_DEF     1       /* Per-thread trigger list entries */
 #define SLINGSHOT_PTE_DEF     6       /* Per-thread portal table entries */
 #define SLINGSHOT_LE_DEF      16      /* Per-thread list entries */
-#define SLINGSHOT_AC_DEF      4       /* Per-thread addressing contexts */
+#define SLINGSHOT_AC_DEF      2       /* Per-thread addressing contexts */
 
 /* NIC resource limit structure */
 typedef struct slingshot_limits {
@@ -151,6 +155,7 @@ typedef struct slingshot_limits_set {
  * 'SwitchParameters' slurm.conf variable
  */
 typedef struct slingshot_config {
+	uint32_t destroy_retries; /* retry count for destroying services */
 	uint8_t single_node_vni;        /* Allocate VNIs for single-node apps */
 	uint8_t job_vni;                /* Allocate extra VNI per-job */
 	uint32_t tcs;                   /* Bitmap of default traffic classes */
@@ -164,6 +169,10 @@ typedef struct slingshot_config {
 	char *fm_url;                   /* fabric manager REST interface URL */
 	slingshot_rest_auth_t fm_auth;  /* fabric manager authentication type */
 	char *fm_authdir;               /* fabric manager auth file directory */
+	char *fm_mtls_ca; /* fabric manager certificate bundle path */
+	char *fm_mtls_cert; /* fabric manager client public certificate path */
+	char *fm_mtls_key; /* fabric manager client private key path */
+	char *fm_mtls_url; /* fabric manager REST interface URL for mtls */
 } slingshot_config_t;
 
 /* Values for slingshot_config_t.single_node_vni */
@@ -253,14 +262,14 @@ typedef struct slingshot_stepinfo {
  * resource limit reservations by subtracting system service reserved/used
  * resources
  *
+ * If SLINGSHOT_FLAGS_ENABLE_MTLS is set, Slurm daemons will use mTLS
+ * authentication with the fabric manager for the duration of the application
+ *
  * If SLINGSHOT_FLAGS_DISABLE_RDZV_GET is set, slurmd will disable rendezvous
  * gets in the Cassini NIC for the duration of the application
  */
 #define SLINGSHOT_FLAGS_ADJUST_LIMITS 0x1
-/*
- * #define SLINGSHOT_FLAGS_VNI_PIDS      0x2 DEPRECATED in 23.02, can be used in
- *					     25.02
- */
+#define SLINGSHOT_FLAGS_ENABLE_MTLS 0x2
 #define SLINGSHOT_FLAGS_DISABLE_RDZV_GET 0x4
 #define SLINGSHOT_FLAGS_DEFAULT SLINGSHOT_FLAGS_ADJUST_LIMITS
 
@@ -297,7 +306,9 @@ extern void slingshot_release_collectives_job_step(slingshot_stepinfo_t *job);
 extern void slingshot_release_collectives_job(uint32_t job_id);
 /* config.c */
 extern void slingshot_free_config(void);
+extern bool slingshot_stepd_init(const char *switch_params);
 extern bool slingshot_setup_config(const char *switch_params);
+extern int slingshot_update_vni_table(void);
 extern bool slingshot_setup_job_vni_pool(job_record_t *job_ptr);
 extern bool slingshot_setup_job_step_vni(
 	slingshot_stepinfo_t *job, int node_cnt,
