@@ -60,8 +60,10 @@ static void _aggregate_tres_usage_stats_internal(char **dest_tres_max,
 	dest_tres_list = list_create(slurmdb_destroy_tres_rec);
 	from_tres_list = list_create(slurmdb_destroy_tres_rec);
 
-	slurmdb_tres_list_from_string(&dest_tres_list, *dest_tres_max, flags);
-	slurmdb_tres_list_from_string(&from_tres_list, from_tres_max, flags);
+	slurmdb_tres_list_from_string(&dest_tres_list, *dest_tres_max, flags,
+				      NULL);
+	slurmdb_tres_list_from_string(&from_tres_list, from_tres_max, flags,
+				      NULL);
 
 	itr = list_iterator_create(from_tres_list);
 	while ((from_tres_rec = list_next(itr))) {
