@@ -41,7 +41,6 @@
 #include <sys/stat.h>
 
 #include "src/sacctmgr/sacctmgr.h"
-#include <sys/param.h>		/* MAXPATHLEN */
 #include "src/common/proc_args.h"
 #include "src/common/util-net.h"
 
@@ -335,7 +334,7 @@ static int _set_cond(int *start, int argc, char **argv,
 			if (!job_cond->userid_list)
 				job_cond->userid_list = list_create(xfree_ptr);
 			if (slurm_addto_id_char_list(job_cond->userid_list,
-			                             argv[i]+end, false))
+						     argv[i]+end, false) > 0)
 				set = 1;
 			else
 				exit_code = 1;
@@ -511,10 +510,18 @@ extern int sacctmgr_archive_load(int argc, char **argv)
 	rc = slurmdb_archive_load(db_conn, arch_rec);
 	if (rc == SLURM_SUCCESS) {
 		if (commit_check("Would you like to commit changes?")) {
-			slurmdb_connection_commit(db_conn, 1);
+			rc = slurmdb_connection_commit(db_conn, 1);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr,
+					"Error committing changes: %s\n",
+					slurm_strerror(rc));
 		} else {
 			printf(" Changes Discarded\n");
-			slurmdb_connection_commit(db_conn, 0);
+			rc = slurmdb_connection_commit(db_conn, 0);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr,
+					"Error rolling back changes: %s\n",
+					slurm_strerror(rc));
 		}
 	} else {
 		exit_code = 1;
