@@ -414,13 +414,19 @@ extern int sacctmgr_add_cluster(int argc, char **argv)
 		rc = slurmdb_clusters_add(db_conn, cluster_list);
 		notice_thread_fini();
 		if (rc == SLURM_SUCCESS) {
-			slurmdb_connection_commit(db_conn, 1);
+			rc = slurmdb_connection_commit(db_conn, 1);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error committing changes: %s\n",
+					slurm_strerror(rc));
 		} else {
-			exit_code=1;
+			exit_code = 1;
 			fprintf(stderr, " Problem adding clusters: %s\n",
 				slurm_strerror(rc));
 			/* this isn't really needed, but just to be safe */
-			slurmdb_connection_commit(db_conn, 0);
+			rc = slurmdb_connection_commit(db_conn, 0);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error rolling back changes: %s\n",
+					slurm_strerror(rc));
 		}
 	} else {
 		printf(" Changes Discarded\n");
@@ -748,56 +754,56 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 		}
 	}
 
-	if (cluster->fed.name && cluster->fed.name[0]) {
-		int rc;
-		/* Make sure federation exists. */
-		list_t *fed_list = list_create(xfree_ptr);
-		list_append(fed_list, xstrdup(cluster->fed.name));
-		rc = verify_federations_exist(fed_list);
-		FREE_NULL_LIST(fed_list);
-		if (rc)
-			goto end_it;
-
-		/* See if cluster is assigned to another federation already. */
-		if (list_count(cluster_cond.cluster_list)) {
-			if (_verify_fed_clusters(cluster_cond.cluster_list,
-						 cluster->fed.name,
-						 &existing_fed))
-					goto end_it;
-			else if (!list_count(cluster_cond.cluster_list)) {
-				/* irrelevant changes have been removed and
-				 * nothing to change now. */
-				printf("Nothing to change\n");
-				rc = SLURM_ERROR;
-				(void)rc; /* CLANG false positive */
+	if (rec_set & SA_SET_CLUST) {
+		if (cluster->fed.name && cluster->fed.name[0]) {
+			int rc;
+			/* Make sure federation exists. */
+			list_t *fed_list = list_create(xfree_ptr);
+			list_append(fed_list, xstrdup(cluster->fed.name));
+			rc = verify_federations_exist(fed_list);
+			FREE_NULL_LIST(fed_list);
+			if (rc)
 				goto end_it;
-			} else if (existing_fed) {
-				char *warning =
-					"\nAre you sure you want to continue?";
-				if (!commit_check(warning)) {
+
+			/*
+			 * See if cluster is assigned to another federation
+			 * already.
+			 */
+			if (list_count(cluster_cond.cluster_list)) {
+				if (_verify_fed_clusters(
+					    cluster_cond.cluster_list,
+					    cluster->fed.name,
+					    &existing_fed))
+					goto end_it;
+
+				/*
+				 * _verify_fed_clusters() can remove from
+				 * cluster_list.
+				 */
+				if (!list_count(cluster_cond.cluster_list)) {
+					/*
+					 * Irrelevant changes have been removed
+					 * and nothing to change now.
+					 */
+					printf("  Nothing to change\n");
+					rc = SLURM_ERROR;
+					goto assoc;
+				} else if (existing_fed &&
+					   !commit_check("\nAre you sure you want to continue?")) {
 					rc = SLURM_ERROR;
-					(void)rc; /* CLANG false positive */
 					goto end_it;
 				}
 			}
 		}
-	}
 
-	printf(" Setting\n");
-	if (rec_set & SA_SET_CLUST)
-		sacctmgr_print_cluster(cluster);
-	if (rec_set & SA_SET_ASSOC) {
-		printf("  Default Limits:\n");
-		sacctmgr_print_assoc_limits(assoc);
-	}
-
-	if (rec_set & SA_SET_CLUST) {
 		notice_thread_init();
 		ret_list = slurmdb_clusters_modify(
 			db_conn, &cluster_cond, cluster);
 
 		if (ret_list && list_count(ret_list)) {
 			char *object = NULL;
+			printf(" Setting\n");
+			sacctmgr_print_cluster(cluster);
 			list_itr_t *itr = list_iterator_create(ret_list);
 			printf(" Modified cluster...\n");
 			while((object = list_next(itr))) {
@@ -805,8 +811,8 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 			}
 			list_iterator_destroy(itr);
 			set = 1;
-		} else if (ret_list) {
-			printf(" Nothing modified\n");
+		} else if (ret_list || errno == SLURM_NO_CHANGE_IN_DATA) {
+			printf("  Nothing modified\n");
 			rc = SLURM_ERROR;
 		} else {
 			exit_code=1;
@@ -817,8 +823,23 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 
 		FREE_NULL_LIST(ret_list);
 		notice_thread_fini();
+		if (set) {
+			if (commit_check("Would you like to commit changes?")) {
+				rc = slurmdb_connection_commit(db_conn, 1);
+				if (rc != SLURM_SUCCESS)
+					fprintf(stderr, " Error committing changes: %s\n",
+						slurm_strerror(rc));
+			} else {
+				printf(" Changes Discarded\n");
+				rc = slurmdb_connection_commit(db_conn, 0);
+				if (rc != SLURM_SUCCESS)
+					fprintf(stderr, " Error rolling back changes: %s\n",
+						slurm_strerror(rc));
+			}
+			set = 0;
+		}
 	}
-
+assoc:
 	if (rec_set & SA_SET_ASSOC) {
 		list_append(assoc_cond->acct_list, "root");
 		notice_thread_init();
@@ -827,16 +848,21 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 
 		if (ret_list && list_count(ret_list)) {
 			char *object = NULL;
-			list_itr_t *itr = list_iterator_create(ret_list);
-			printf(" Modified cluster defaults for "
-			       "associations...\n");
+			list_itr_t *itr;
+
+			printf(" Setting\n");
+			printf("  Default Limits:\n");
+			sacctmgr_print_assoc_limits(assoc);
+
+			printf(" Modified cluster defaults for associations...\n");
+			itr = list_iterator_create(ret_list);
 			while((object = list_next(itr))) {
 				printf("  %s\n", object);
 			}
 			list_iterator_destroy(itr);
 			set = 1;
-		} else if (ret_list) {
-			printf(" Nothing modified\n");
+		} else if (ret_list || errno == SLURM_NO_CHANGE_IN_DATA) {
+			printf("  Nothing modified\n");
 			rc = SLURM_ERROR;
 		} else {
 			exit_code=1;
@@ -848,13 +874,18 @@ extern int sacctmgr_modify_cluster(int argc, char **argv)
 		notice_thread_fini();
 	}
 
-
 	if (set) {
-		if (commit_check("Would you like to commit changes?"))
-			slurmdb_connection_commit(db_conn, 1);
-		else {
+		if (commit_check("Would you like to commit changes?")) {
+			rc = slurmdb_connection_commit(db_conn, 1);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error committing changes: %s\n",
+					slurm_strerror(rc));
+		} else {
 			printf(" Changes Discarded\n");
-			slurmdb_connection_commit(db_conn, 0);
+			rc = slurmdb_connection_commit(db_conn, 0);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error rolling back changes: %s\n",
+					slurm_strerror(rc));
 		}
 	}
 end_it:
@@ -897,7 +928,9 @@ extern int sacctmgr_delete_cluster(int argc, char **argv)
 		return SLURM_ERROR;
 	}
 
-	if (!list_count(cluster_cond->cluster_list)
+	if (!list_count(cluster_cond->cluster_list) &&
+	    (!cluster_cond->rpc_version_list ||
+	     !list_count(cluster_cond->rpc_version_list))
 	   && !cluster_cond->classification
 	   && (!cluster_cond->federation_list ||
 	       !list_count(cluster_cond->federation_list))) {
@@ -937,10 +970,16 @@ extern int sacctmgr_delete_cluster(int argc, char **argv)
 		}
 		list_iterator_destroy(itr);
 		if (commit_check("Would you like to commit changes?")) {
-			slurmdb_connection_commit(db_conn, 1);
+			rc = slurmdb_connection_commit(db_conn, 1);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error committing changes: %s\n",
+					slurm_strerror(rc));
 		} else {
 			printf(" Changes Discarded\n");
-			slurmdb_connection_commit(db_conn, 0);
+			rc = slurmdb_connection_commit(db_conn, 0);
+			if (rc != SLURM_SUCCESS)
+				fprintf(stderr, " Error rolling back changes: %s\n",
+					slurm_strerror(rc));
 		}
 	} else if (ret_list) {
 		printf(" Nothing deleted\n");
@@ -966,13 +1005,14 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 	slurmdb_assoc_cond_t assoc_cond;
 	list_t *assoc_list = NULL;
 	list_t *acct_list = NULL;
+	list_t *qos_list = NULL;
 	list_t *user_list = NULL;
 	list_t *slurmdb_hierarchical_rec_list = NULL;
 	char *cluster_name = NULL;
 	char *file_name = NULL;
 	char *user_name = NULL;
 	char *line = NULL;
-	int i, command_len = 0;
+	int i, command_len = 0, rc = SLURM_SUCCESS;
 	FILE *fd = NULL;
 	char *class_str = NULL;
 
@@ -1014,10 +1054,9 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 	}
 
 	if (!cluster_name) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, " We need a cluster to dump.\n");
-		xfree(file_name);
-		return SLURM_ERROR;
+		goto end_it;
 	} else {
 		list_t *temp_list = NULL;
 		slurmdb_cluster_cond_t cluster_cond;
@@ -1030,24 +1069,20 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 		temp_list = slurmdb_clusters_get(db_conn, &cluster_cond);
 		FREE_NULL_LIST(cluster_cond.cluster_list);
 		if (!temp_list) {
-			exit_code = 1;
+			rc = SLURM_ERROR;
 			fprintf(stderr,
 				" Problem getting clusters from database.  "
 				"Contact your admin.\n");
-			xfree(cluster_name);
-			xfree(file_name);
-			return SLURM_ERROR;
+			goto end_it;
 		}
 
 		cluster_rec = list_peek(temp_list);
 		if (!cluster_rec) {
-			exit_code = 1;
+			rc = SLURM_ERROR;
 			fprintf(stderr, " Cluster %s doesn't exist.\n",
 				cluster_name);
-			xfree(cluster_name);
-			xfree(file_name);
 			FREE_NULL_LIST(temp_list);
-			return SLURM_ERROR;
+			goto end_it;
 		}
 		class_str = get_classification_str(cluster_rec->classification);
 		FREE_NULL_LIST(temp_list);
@@ -1082,47 +1117,34 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 	/* make sure this person running is an admin */
 	user_name = uid_to_string_cached(my_uid);
 	if (!(user = sacctmgr_find_user_from_list(user_list, user_name))) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, " Your uid (%u) is not in the "
 			"accounting system, can't dump cluster.\n", my_uid);
 		FREE_NULL_LIST(assoc_cond.cluster_list);
-		xfree(cluster_name);
-		xfree(file_name);
-		FREE_NULL_LIST(user_list);
-		return SLURM_ERROR;
-
+		goto end_it;
 	} else {
 		if ((my_uid != slurm_conf.slurm_user_id) && (my_uid != 0)
 		    && user->admin_level < SLURMDB_ADMIN_SUPER_USER) {
-			exit_code = 1;
+			rc = SLURM_ERROR;
 			fprintf(stderr, " Your user does not have sufficient "
 				"privileges to dump clusters.\n");
 			FREE_NULL_LIST(assoc_cond.cluster_list);
-			xfree(cluster_name);
-			xfree(file_name);
-			FREE_NULL_LIST(user_list);
-			return SLURM_ERROR;
+			goto end_it;
 		}
 	}
-	xfree(user_name);
 
 	/* assoc_cond is set up above */
 	assoc_list = slurmdb_associations_get(db_conn, &assoc_cond);
 	FREE_NULL_LIST(assoc_cond.cluster_list);
 	if (!assoc_list) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, " Problem with query.\n");
-		xfree(cluster_name);
-		xfree(file_name);
-		return SLURM_ERROR;
+		goto end_it;
 	} else if (!list_count(assoc_list)) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, " Cluster %s returned nothing.\n",
 			cluster_name);
-		FREE_NULL_LIST(assoc_list);
-		xfree(cluster_name);
-		xfree(file_name);
-		return SLURM_ERROR;
+		goto end_it;
 	}
 
 	slurmdb_hierarchical_rec_list = slurmdb_get_acct_hierarchical_rec_list(
@@ -1133,12 +1155,8 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 	if ((fd = fopen(file_name,"w")) == NULL) {
 		fprintf(stderr, "Can't open file %s, %s\n", file_name,
 			slurm_strerror(errno));
-		FREE_NULL_LIST(acct_list);
-		FREE_NULL_LIST(assoc_list);
-		xfree(cluster_name);
-		xfree(file_name);
-		FREE_NULL_LIST(slurmdb_hierarchical_rec_list);
-		return SLURM_ERROR;
+		rc = SLURM_ERROR;
+		goto end_it;
 	}
 
 	/* Add header */
@@ -1169,17 +1187,19 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 		    "# User - 'lipari':MaxTRESPerJob=node=2:MaxJobs=3:"
 		    "MaxTRESMinsPerJob=cpu=4:FairShare=1:"
 		    "MaxWallDurationPerJob=1\n") < 0) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, "Can't write to file");
-		FREE_NULL_LIST(acct_list);
-		FREE_NULL_LIST(assoc_list);
-		xfree(cluster_name);
-		xfree(file_name);
-		FREE_NULL_LIST(slurmdb_hierarchical_rec_list);
-		return SLURM_ERROR;
+		goto end_it;
 	}
 
-	line = xstrdup_printf("Cluster - '%s'", cluster_name);
+	qos_list = slurmdb_qos_get(db_conn, NULL);
+
+	if (qos_list) {
+		(void) list_for_each(qos_list, file_print_qos, &line);
+		FREE_NULL_LIST(qos_list);
+	}
+
+	xstrfmtcat(line, "Cluster - '%s'", cluster_name);
 
 	if (class_str)
 		xstrfmtcat(line, ":Classification='%s'", class_str);
@@ -1193,15 +1213,10 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 		print_file_add_limits_to_line(&line, assoc);
 
 	if (fprintf(fd, "%s\n", line) < 0) {
-		exit_code = 1;
+		rc = SLURM_ERROR;
 		fprintf(stderr, " Can't write to file");
-		FREE_NULL_LIST(acct_list);
-		FREE_NULL_LIST(assoc_list);
-		xfree(cluster_name);
-		xfree(file_name);
 		xfree(line);
-		FREE_NULL_LIST(slurmdb_hierarchical_rec_list);
-		return SLURM_ERROR;
+		goto end_it;
 	}
 	info("%s", line);
 	xfree(line);
@@ -1209,12 +1224,16 @@ extern int sacctmgr_dump_cluster (int argc, char **argv)
 	print_file_slurmdb_hierarchical_rec_list(
 		fd, slurmdb_hierarchical_rec_list, user_list, acct_list);
 
+end_it:
+	if (fd)
+		fclose(fd);
 	FREE_NULL_LIST(acct_list);
 	FREE_NULL_LIST(assoc_list);
+	FREE_NULL_LIST(user_list);
 	xfree(cluster_name);
 	xfree(file_name);
 	FREE_NULL_LIST(slurmdb_hierarchical_rec_list);
-	fclose(fd);
-
-	return SLURM_SUCCESS;
+	if (rc != SLURM_SUCCESS)
+		exit_code = 1;
+	return rc;
 }
