@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Copyright (C) 2010-2017 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Christopher J. Morrone <morrone2@llnl.gov>
  *  CODE-OCEC-09-009. All rights reserved.
@@ -74,7 +74,6 @@ static void *_get_script_buffer(const char *filename, int *size);
 static int   _job_wait(uint32_t job_id);
 static char *_script_wrap(char *command_string);
 static void  _set_exit_code(void);
-static void  _set_prio_process_env(void);
 static int   _set_rlimit_env(void);
 static void  _set_spank_env(void);
 static void  _set_submit_dir_env(void);
@@ -91,7 +90,7 @@ int main(int argc, char **argv)
 	int script_size = 0, het_job_argc, het_job_argc_off = 0, het_job_inx;
 	int i, rc = SLURM_SUCCESS, retries = 0, het_job_limit = 0;
 	bool het_job_fini = false;
-	List job_env_list = NULL, job_req_list = NULL;
+	list_t *job_env_list = NULL, *job_req_list = NULL;
 	sbatch_env_t *local_env = NULL;
 	bool quiet = false;
 
@@ -108,14 +107,14 @@ int main(int argc, char **argv)
 		fatal("failed to initialize cli_filter plugin");
 
 	_set_exit_code();
-	if (spank_init_allocator() < 0) {
+	if (spank_init_allocator()) {
 		error("Failed to initialize plugin stack");
 		exit(error_exit);
 	}
 
 	/* Be sure to call spank_fini when sbatch exits
 	 */
-	if (atexit((void (*) (void)) spank_fini) < 0)
+	if (atexit((void (*) (void)) spank_fini))
 		error("Failed to register atexit handler for plugins: %m");
 
 	script_name = process_options_first_pass(argc, argv);
@@ -133,11 +132,13 @@ int main(int argc, char **argv)
 
 	if (sbopt.wrap != NULL) {
 		script_body = _script_wrap(sbopt.wrap);
+	} else if (opt.job_flags & EXTERNAL_JOB) {
+		script_body = NULL;
 	} else {
 		script_body = _get_script_buffer(script_name, &script_size);
+		if (!script_body)
+			exit(error_exit);
 	}
-	if (script_body == NULL)
-		exit(error_exit);
 
 	het_job_argc = argc - opt.argc;
 	het_job_argv = argv;
@@ -176,19 +177,26 @@ int main(int argc, char **argv)
 			FREE_NULL_BUFFER(buf);
 		}
 
-		if (spank_init_post_opt() < 0) {
+		if (spank_init_post_opt()) {
 			error("Plugin stack post-option processing failed");
 			exit(error_exit);
 		}
 
-		if (opt.get_user_env_time < 0) {
+		if (!opt.get_user_env) {
 			/* Moab doesn't propagate the user's resource limits, so
 			 * slurmd determines the values at the same time that it
 			 * gets the user's default environment variables. */
 			(void) _set_rlimit_env();
 		}
 
-		_set_prio_process_env();
+		/*
+		 * if the environment is coming from a file, the
+		 * environment at execution startup, must be unset.
+		 */
+		if (sbopt.export_file != NULL)
+			env_unset_environment();
+
+		set_prio_process_env();
 		_set_spank_env();
 		_set_submit_dir_env();
 		_set_umask_env();
@@ -225,7 +233,7 @@ int main(int argc, char **argv)
 	}
 
 	if (job_env_list) {
-		ListIterator desc_iter, env_iter;
+		list_itr_t *desc_iter, *env_iter;
 		i = 0;
 		desc_iter = list_iterator_create(job_req_list);
 		env_iter  = list_iterator_create(job_env_list);
@@ -287,7 +295,8 @@ int main(int argc, char **argv)
 			break;
 		if (errno == ESLURM_ERROR_ON_DESC_TO_RECORD_COPY) {
 			msg = "Slurm job queue full, sleeping and retrying";
-		} else if (errno == ESLURM_NODES_BUSY) {
+		} else if ((errno == ESLURM_NODES_BUSY) ||
+			   (errno == ESLURM_PORTS_BUSY)) {
 			msg = "Job creation temporarily disabled, retrying";
 		} else if (errno == EAGAIN) {
 			msg = "Slurm temporarily unable to accept job, "
@@ -301,7 +310,8 @@ int main(int argc, char **argv)
 
 		if (retries)
 			debug("%s", msg);
-		else if (errno == ESLURM_NODES_BUSY)
+		else if ((errno == ESLURM_NODES_BUSY) ||
+			 (errno == ESLURM_PORTS_BUSY))
 			info("%s", msg); /* Not an error, powering up nodes */
 		else
 			error("%s", msg);
@@ -341,8 +351,7 @@ int main(int argc, char **argv)
 #ifdef MEMORY_LEAK_DEBUG
 	cli_filter_fini();
 	slurm_reset_all_options(&opt, false);
-	auth_g_fini();
-	slurm_conf_destroy();
+	slurm_fini();
 	log_fini();
 #endif /* MEMORY_LEAK_DEBUG */
 	xfree(script_body);
@@ -422,7 +431,19 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 
 	desc->wait_all_nodes = sbopt.wait_all_nodes;
 
+	if (sbopt.requeue != NO_VAL)
+		desc->requeue = sbopt.requeue;
+
 	desc->environment = NULL;
+
+	if (opt.job_flags & EXTERNAL_JOB)
+		return 0;
+
+	if (sbopt.export_file) {
+		desc->environment = env_array_from_file(sbopt.export_file);
+		if (desc->environment == NULL)
+			exit(1);
+	}
 	if (opt.export_env == NULL) {
 		env_array_merge(&desc->environment, (const char **) environ);
 	} else if (!xstrcasecmp(opt.export_env, "ALL")) {
@@ -435,12 +456,12 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 		desc->environment = env_array_create();
 		env_array_merge_slurm_spank(&desc->environment,
 					    (const char **) environ);
-		opt.get_user_env_time = 0;
+		opt.get_user_env = true;
 	} else {
 		env_merge_filter(&opt, desc);
-		opt.get_user_env_time = 0;
+		opt.get_user_env = true;
 	}
-	if (opt.get_user_env_time >= 0) {
+	if (opt.get_user_env) {
 		env_array_overwrite(&desc->environment,
 				    "SLURM_GET_USER_ENV", "1");
 	}
@@ -459,9 +480,6 @@ static int _fill_job_desc_from_opts(job_desc_msg_t *desc)
 	desc->std_in   = xstrdup(opt.ifname);
 	desc->std_out  = xstrdup(opt.ofname);
 
-	if (sbopt.requeue != NO_VAL)
-		desc->requeue = sbopt.requeue;
-
 	return 0;
 }
 
@@ -535,34 +553,6 @@ static int _set_umask_env(void)
 	return SLURM_SUCCESS;
 }
 
-/*
- * _set_prio_process_env
- *
- * Set the internal SLURM_PRIO_PROCESS environment variable to support
- * the propagation of the users nice value and the "PropagatePrioProcess"
- * config keyword.
- */
-static void  _set_prio_process_env(void)
-{
-	int retval;
-
-	errno = 0; /* needed to detect a real failure since prio can be -1 */
-
-	if ((retval = getpriority (PRIO_PROCESS, 0)) == -1)  {
-		if (errno) {
-			error ("getpriority(PRIO_PROCESS): %m");
-			return;
-		}
-	}
-
-	if (setenvf (NULL, "SLURM_PRIO_PROCESS", "%d", retval) < 0) {
-		error ("unable to set SLURM_PRIO_PROCESS in environment");
-		return;
-	}
-
-	debug ("propagating SLURM_PRIO_PROCESS=%d", retval);
-}
-
 /*
  * Checks if the buffer starts with a shebang (#!).
  */
