@@ -55,7 +55,6 @@
 #include "src/common/bitstring.h"
 #include "src/common/cron.h"
 #include "src/common/extra_constraints.h"
-#include "src/common/front_end.h"
 #include "src/common/identity.h"
 #include "src/common/job_record.h"
 #include "src/common/job_resources.h"
@@ -195,7 +194,7 @@ typedef struct diag_stats {
 	uint32_t jobs_canceled;
 	uint32_t jobs_failed;
 
-	uint32_t job_states_ts;
+	time_t job_states_ts;
 	uint32_t jobs_pending;
 	uint32_t jobs_running;
 
@@ -292,6 +291,7 @@ extern list_t *conf_includes_list;  /* list of conf_includes_map_t */
  *  booting_node_bitmap     Set if node in process of booting
  *  cg_node_bitmap          Set if node in completing state
  *  cloud_node_bitmap       Set if node in CLOUD state
+ *  external_node_bitmap    Set if node in EXTERNAL state
  *  future_node_bitmap      Set if node in FUTURE state
  *  idle_node_bitmap        Set if node has no jobs allocated to it
  *  power_down_node_bitmap  Set for nodes which are powered down
@@ -310,6 +310,7 @@ extern bitstr_t *bf_ignore_node_bitmap;	/* bitmap of nodes made available during
 extern bitstr_t *booting_node_bitmap;	/* bitmap of booting nodes */
 extern bitstr_t *cg_node_bitmap;	/* bitmap of completing nodes */
 extern bitstr_t *cloud_node_bitmap;	/* bitmap of cloud nodes */
+extern bitstr_t *external_node_bitmap;	/* bitmap of external nodes */
 extern bitstr_t *future_node_bitmap;	/* bitmap of FUTURE nodes */
 extern bitstr_t *idle_node_bitmap;	/* bitmap of idle nodes */
 extern bitstr_t *power_down_node_bitmap; /* Powered down nodes */
@@ -388,8 +389,10 @@ typedef struct slurmctld_resv {
 	time_t start_time;	/* start time of reservation		*/
 	time_t start_time_first;/* when the reservation first started	*/
 	time_t start_time_prev;	/* If start time was changed this is
-				 * the pervious start time.  Needed
+				 * the previous start time.  Needed
 				 * for accounting */
+	time_t time_force; /* The actual start time of the reservation if the
+			    * FORCE_START flag was used */
 	char *tres_fmt_str;     /* formatted string of tres to deal with */
 	char *tres_str;         /* simple string of tres to deal with */
 	char *users;		/* names of users permitted to use	*/
@@ -415,25 +418,6 @@ extern time_t last_resv_update;	/* time of last resv_list update */
 extern list_t *job_list;		/* list of job_record entries */
 extern list_t *purge_jobs_list;		/* list of job_record_t to free */
 
-/*****************************************************************************\
- *  Consumable Resources parameters and data structures
-\*****************************************************************************/
-
-/*
- * Define the type of update and of data retrieval that can happen
- * from the "select/cons_res" plugin. This information needed to
- * support processors as consumable resources.  This structure will be
- * useful when updating other types of consumable resources as well
-*/
-enum select_plugindata_info {
-	SELECT_CR_PLUGIN = 0,    /* data-> uint32 See SELECT_TYPE_* below */
-	SELECT_CONFIG_INFO = 6,  /* data-> list_t * get .conf info from select
-				  * plugin */
-};
-#define SELECT_TYPE_CONS_RES	1
-#define SELECT_TYPE_CONS_TRES	2
-
-
 /*****************************************************************************\
  *  Global assoc_cache variables
 \*****************************************************************************/
@@ -930,7 +914,7 @@ extern job_record_t *job_array_post_sched(job_record_t *job_ptr, bool list_add);
 
 /* Create an exact copy of an existing job record for a job array.
  * IN job_ptr - META job record for a job array, which is to become an
- *		individial task of the job array.
+ *		individual task of the job array.
  *		Set the job's array_task_id to the task to be split out.
  * IN list_add - add to the job_list or not.
  * RET - The new job record, which is the new META job record. */
@@ -1041,17 +1025,6 @@ extern void job_post_resize_acctg(job_record_t *job_ptr);
 extern int job_signal(job_record_t *job_ptr, uint16_t signal,
 		      uint16_t flags, uid_t uid, bool preempt);
 
-/*
- * job_signal_id - signal the specified job
- * IN job_id - id of the job to be signaled
- * IN signal - signal to send, SIGKILL == cancel the job
- * IN flags  - see KILL_JOB_* flags in slurm.h
- * IN uid - uid of requesting user
- * IN preempt - true if job being preempted
- * RET 0 on success, otherwise ESLURM error code
- */
-extern int job_signal_id(uint32_t job_id, uint16_t signal, uint16_t flags,
-			 uid_t uid, bool preempt);
 /*
  * het_job_signal - signal all components of a hetjob
  * IN het_job_leader - job record of job hetjob leader
@@ -1125,12 +1098,12 @@ extern int job_complete(uint32_t job_id, uid_t uid, bool requeue,
  * job_independent - determine if this job has a dependent job pending
  *	or if the job's scheduled begin time is in the future
  * IN job_ptr - pointer to job being tested
- * RET - true if job no longer must be defered for another job
+ * RET - true if job no longer must be deferred for another job
  */
 extern bool job_independent(job_record_t *job_ptr);
 
 /*
- * job_req_node_filter - job reqeust node filter.
+ * job_req_node_filter - job request node filter.
  *	clear from a bitmap the nodes which can not be used for a job
  *	test memory size, required features, processor count, etc.
  * NOTE: Does not support exclusive OR of features.
@@ -1235,20 +1208,12 @@ extern int kill_job_by_part_name(char *part_name);
 extern void kill_job_on_node(job_record_t *job_ptr, node_record_t *node_ptr);
 
 /*
- * kill_job_by_front_end_name - Given a front end node name, deallocate
- *	resource for its jobs and kill them.
- * IN node_name - name of a front end node
- * RET number of jobs associated with this front end node
- */
-extern int kill_job_by_front_end_name(char *node_name);
-
-/*
- * kill_running_job_by_node_name - Given a node name, deallocate RUNNING
+ * kill_running_job_by_node_ptr - Given a node_ptr, deallocate RUNNING
  *	or COMPLETING jobs from the node or kill them
  * IN node_name - name of a node
  * RET number of killed jobs
  */
-extern int kill_running_job_by_node_name(char *node_name);
+extern int kill_running_job_by_node_ptr(node_record_t *node_ptr);
 
 /* list_compare_config - compare two entry from the config list based upon
  *	weight, see common/list.h for documentation */
@@ -1608,7 +1573,7 @@ void purge_old_job(void);
 extern void free_old_jobs(void);
 
 /* Convert a comma delimited list of QOS names into a bitmap */
-extern void qos_list_build(char *qos, bitstr_t **qos_bits);
+extern void qos_list_build(char *qos, bool locked, bitstr_t **qos_bits);
 
 /* Request that the job scheduler execute soon (typically within seconds) */
 extern void queue_job_scheduler(void);
@@ -1907,11 +1872,11 @@ extern int update_node_record_acct_gather_data(
 /*
  * Create nodes from scontrol using slurm.conf nodeline syntax.
  *
- * IN nodeline - slurm.conf nodename description.
+ * IN msg - creation message including nodeline
  * OUT err_msg - pass error messages out.
  * RET SLURM_SUCCESS on success, SLURM_ERROR otherwise.
  */
-extern int create_nodes(char *nodeline, char **err_msg);
+extern int create_nodes(update_node_msg_t *msg, char **err_msg);
 
 /*
  * Create and add dynamic node to system from registration.
@@ -2010,20 +1975,6 @@ extern void validate_jobs_on_node(slurm_msg_t *slurm_msg);
  */
 extern int validate_node_specs(slurm_msg_t *slurm_msg, bool *newly_up);
 
-/*
- * validate_nodes_via_front_end - validate all nodes on a cluster as having
- *	a valid configuration as soon as the front-end registers. Individual
- *	nodes will not register with this configuration
- * IN reg_msg - node registration message
- * IN protocol_version - Version of Slurm on this node
- * OUT newly_up - set if node newly brought into service
- * RET 0 if no error, Slurm error code otherwise
- * NOTE: READ lock_slurmctld config before entry
- */
-extern int validate_nodes_via_front_end(
-		slurm_node_registration_status_msg_t *reg_msg,
-		uint16_t protocol_version, bool *newly_up);
-
 /*
  * validate_super_user - validate that the uid is authorized at the
  *      root, SlurmUser, or SLURMDB_ADMIN_SUPER_USER level
@@ -2060,7 +2011,7 @@ extern bool validate_operator_user_rec(slurmdb_user_rec_t *user);
  * when the slurmctld epilog finishes, whichever
  * comes last.
  */
-extern void cleanup_completing(job_record_t *job_ptr);
+extern void cleanup_completing(job_record_t *job_ptr, bool requeue);
 
 /*
  * Determine if slurmctld will respond to "configless" RPCs. If so,
@@ -2177,7 +2128,7 @@ extern bool waiting_for_node_boot(node_record_t *node_ptr);
 extern bool waiting_for_node_power_down(node_record_t *node_ptr);
 
 /*
- * Check if any part of job_ptr is overlaping node_map.
+ * Check if any part of job_ptr is overlapping node_map.
  * IN node_map - bitstr of nodes set.
  * IN job_ptr (hetjob or not) to check.
  *
@@ -2237,7 +2188,7 @@ extern char **job_common_env_vars(job_record_t *job_ptr, bool is_complete);
  * update_node_active_features - Update active features associated with nodes
  * IN node_names - list of nodes to update
  * IN active_features - New active features value
- * IN mode - FEATURE_MODE_IND : Print each node change indivually
+ * IN mode - FEATURE_MODE_IND : Print each node change individually
  *           FEATURE_MODE_COMB: Try to combine like changes (SEE NOTE BELOW)
  *           FEATURE_MODE_PEND: Print any pending change message
  * RET: SLURM_SUCCESS or error code
@@ -2252,7 +2203,7 @@ extern int update_node_active_features(char *node_names, char *active_features,
  *	nodes, build new config list records as needed
  * IN node_names - list of nodes to update
  * IN avail_features - New available features value
- * IN mode - FEATURE_MODE_IND : Print each node change indivually
+ * IN mode - FEATURE_MODE_IND : Print each node change individually
  *           FEATURE_MODE_COMB: Try to combine like changes (SEE NOTE BELOW)
  *           FEATURE_MODE_PEND: Print any pending change message
  * RET: SLURM_SUCCESS or error code
@@ -2308,6 +2259,8 @@ extern void set_node_reason(node_record_t *node_ptr,
 			    char *message,
 			    time_t time);
 
+extern int set_part_topology_idx(void *x, void *arg);
+
 extern void reconfigure_slurm(slurm_msg_t *msg);
 
 extern void notify_parent_of_success(void);
@@ -2332,7 +2285,7 @@ extern job_record_t *job_mgr_copy_resv_desc_to_job_record(
 extern int controller_init_scheduling(bool init_gang);
 
 /*
- * Finialize the various schedulers.
+ * Finalize the various schedulers.
  */
 extern void controller_fini_scheduling(void);
 
