@@ -124,7 +124,14 @@ static job_state_t *_find_job_state(uint32_t jobid)
 
 static void _clear_expired_job_states(void)
 {
-	time_t now = time(NULL);
+	time_t now;
+
+	if (!cred_job_list) {
+		warning("No cred_job_list, unable to clear expired job states");
+		return;
+	}
+
+	now = time(NULL);
 	list_delete_all(cred_job_list, _list_find_expired_job_state, &now);
 }
 
@@ -140,7 +147,14 @@ static int _list_find_expired_cred_state(void *x, void *key)
 
 static void _clear_expired_credential_states(void)
 {
-	time_t now = time(NULL);
+	time_t now;
+
+	if (!cred_state_list) {
+		warning("No cred_state_list, unable to clear expired credential states");
+		return;
+	}
+
+	now = time(NULL);
 	list_delete_all(cred_state_list, _list_find_expired_cred_state, &now);
 }
 
@@ -185,7 +199,6 @@ static void _cred_state_pack(void *x, uint16_t protocol_version, buf_t *buffer)
 {
 	cred_state_t *s = x;
 
-	/* WARNING: this is not safe if pack_step_id() ever changes format */
 	pack_step_id(&s->step_id, buffer, protocol_version);
 	pack_time(s->ctime, buffer);
 	pack_time(s->expiration, buffer);
@@ -196,8 +209,7 @@ static int _cred_state_unpack(void **out, uint16_t protocol_version,
 {
 	cred_state_t *s = xmalloc(sizeof(*s));
 
-	if (unpack_step_id_members(&s->step_id, buffer,
-				   SLURM_PROTOCOL_VERSION) != SLURM_SUCCESS)
+	if (unpack_step_id_members(&s->step_id, buffer, protocol_version))
 		goto unpack_error;
 	safe_unpack_time(&s->ctime, buffer);
 	safe_unpack_time(&s->expiration, buffer);
@@ -213,9 +225,10 @@ unpack_error:
 
 static void _cred_context_pack(buf_t *buffer)
 {
-	/* FIXME: find a way to version this file at some point */
 	uint16_t version = SLURM_PROTOCOL_VERSION;
 
+	pack16(version, buffer);
+
 	slurm_mutex_lock(&cred_cache_mutex);
 	slurm_pack_list(cred_job_list, _job_state_pack, buffer, version);
 	slurm_pack_list(cred_state_list, _cred_state_pack, buffer, version);
@@ -224,25 +237,44 @@ static void _cred_context_pack(buf_t *buffer)
 
 static void _cred_context_unpack(buf_t *buffer)
 {
-	/* FIXME: find a way to version this file at some point */
-	uint16_t version = SLURM_PROTOCOL_VERSION;
+	uint16_t version = 0;
 
 	slurm_mutex_lock(&cred_cache_mutex);
 
+	/*
+	 * Pre-24.11 files had no version header.
+	 * The first two bytes of those files will usually be 0x0000 - the top
+	 * half of the four byte length of the cred_job_list, which should
+	 * usually be under 65k records. Even if there are enough records set
+	 * to spill into these bytes, the check against the min protocol
+	 * version means we're safe unless the state file had 654 million
+	 * records.
+	 */
+
+	safe_unpack16(&version, buffer);
+
+	if (version < SLURM_24_11_PROTOCOL_VERSION) {
+		/* rewind by 2 bytes */
+		buffer->processed -= 2;
+		version = SLURM_24_05_PROTOCOL_VERSION;
+	}
+
 	FREE_NULL_LIST(cred_job_list);
 	if (slurm_unpack_list(&cred_job_list, _job_state_unpack,
-			      xfree_ptr, buffer, version)) {
-		warning("%s: failed to restore job state from file", __func__);
-	}
+			      xfree_ptr, buffer, version))
+		goto unpack_error;
 	_clear_expired_job_states();
 
 	FREE_NULL_LIST(cred_state_list);
 	if (slurm_unpack_list(&cred_state_list, _cred_state_unpack,
-			      xfree_ptr, buffer, version)) {
-		warning("%s: failed to restore job state from file", __func__);
-	}
+			      xfree_ptr, buffer, version))
+		goto unpack_error;
 	_clear_expired_credential_states();
+	slurm_mutex_unlock(&cred_cache_mutex);
+	return;
 
+unpack_error:
+	warning("%s: failed to restore job state from file", __func__);
 	slurm_mutex_unlock(&cred_cache_mutex);
 }
 
@@ -376,7 +408,7 @@ extern int cred_revoke(uint32_t jobid, time_t time, time_t start_time)
 			debug("job %u requeued, but started no tasks", jobid);
 			j->expiration = (time_t) MAX_TIME;
 		} else {
-			slurm_seterrno(EEXIST);
+			errno = EEXIST;
 			goto error;
 		}
 	}
@@ -417,12 +449,12 @@ extern int cred_begin_expiration(uint32_t jobid)
 	_clear_expired_job_states();
 
 	if (!(j = _find_job_state(jobid))) {
-		slurm_seterrno(ESRCH);
+		errno = ESRCH;
 		goto error;
 	}
 
 	if (j->expiration < (time_t) MAX_TIME) {
-		slurm_seterrno(EEXIST);
+		errno = EEXIST;
 		goto error;
 	}
 
@@ -482,11 +514,10 @@ static int _list_find_cred_state(void *x, void *key)
 	cred_state_t *s = x;
 	slurm_cred_t *cred = key;
 
-	if (!memcmp(&s->step_id, &cred->arg->step_id, sizeof(s->step_id)) &&
-	    (s->ctime == cred->ctime))
-		return 1;
+	if (s->ctime != cred->ctime)
+		return 0;
 
-	return 0;
+	return verify_step_id(&s->step_id, &cred->arg->step_id);
 }
 
 static bool _credential_replayed(slurm_cred_t *cred)
@@ -520,12 +551,12 @@ extern bool cred_cache_valid(slurm_cred_t *cred)
 	cred_handle_reissue(cred, true);
 
 	if (_credential_revoked(cred)) {
-		slurm_seterrno(ESLURMD_CREDENTIAL_REVOKED);
+		errno = ESLURMD_CREDENTIAL_REVOKED;
 		goto error;
 	}
 
 	if (_credential_replayed(cred)) {
-		slurm_seterrno(ESLURMD_CREDENTIAL_REPLAYED);
+		errno = ESLURMD_CREDENTIAL_REPLAYED;
 		goto error;
 	}
 
