@@ -50,17 +50,19 @@
 
 #include "slurm/slurm_errno.h"
 
-#include "src/common/log.h"
 #include "src/common/list.h"
+#include "src/common/log.h"
 #include "src/common/macros.h"
 #include "src/common/parse_config.h"
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/accounting_storage.h"
+#include "src/common/slurmdb_defs.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
-#include "src/common/slurmdb_defs.h"
+
+#include "src/interfaces/accounting_storage.h"
+
 #include "src/slurmdbd/read_config.h"
 
 /* Global variables */
@@ -380,8 +382,6 @@ extern int read_slurmdbd_conf(void)
 				slurm_conf.log_fmt = LOG_FMT_SHORT;
 			else if (xstrcasestr(temp_str, "thread_id"))
 				slurm_conf.log_fmt = LOG_FMT_THREAD_ID;
-			if (xstrcasestr(temp_str, "format_stderr"))
-				slurm_conf.log_fmt |= LOG_FMT_FORMAT_STDERR;
 			xfree(temp_str);
 		}
 
@@ -561,7 +561,8 @@ extern int read_slurmdbd_conf(void)
 		if (slurm_conf.slurm_user_name) {
 			uid_t uid;
 
-			if (uid_from_string(slurm_conf.slurm_user_name, &uid) < 0)
+			if (uid_from_string(slurm_conf.slurm_user_name, &uid) !=
+			    SLURM_SUCCESS)
 				fatal("failed to look up SlurmUser uid");
 
 			if (conf_path_uid != uid)
@@ -646,7 +647,7 @@ extern int read_slurmdbd_conf(void)
 		slurm_conf.plugindir = xstrdup(default_plugin_path);
 	if (slurm_conf.slurm_user_name) {
 		if (uid_from_string(slurm_conf.slurm_user_name,
-		                    &slurm_conf.slurm_user_id) < 0)
+				    &slurm_conf.slurm_user_id) != SLURM_SUCCESS)
 			fatal("Invalid user for SlurmUser %s, ignored",
 			      slurm_conf.slurm_user_name);
 	} else {
@@ -736,7 +737,7 @@ extern int read_slurmdbd_conf(void)
 /* Log the current configuration using verbose() */
 extern void log_config(void)
 {
-	List dbd_config_list;
+	list_t *dbd_config_list;
 	config_key_pair_t *key_pair;
 	list_itr_t *itr;
 
@@ -757,11 +758,11 @@ extern void log_config(void)
  * Dump the configuration in name,value pairs for output to
  * "sacctmgr show config", caller must call list_destroy()
  */
-extern List dump_config(void)
+extern list_t *dump_config(void)
 {
 	char time_str[32];
 	char *tmp_ptr = NULL;
-	List my_list = list_create(destroy_config_key_pair);
+	list_t *my_list = list_create(destroy_config_key_pair);
 
 	add_key_pair_bool(my_list, "AllowNoDefAcct",
 		(slurmdbd_conf->flags & DBD_CONF_FLAG_ALLOW_NO_DEF_ACCT));
