@@ -166,7 +166,7 @@ void print_fields(slurmdb_step_rec_t *step)
 			break;
 		case PRINT_CONSUMED_ENERGY_RAW:
 			field->print_routine(field,
-					     step->stats.consumed_energy,
+					     &step->stats.consumed_energy,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVEDISKREAD:
@@ -288,7 +288,7 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXDISKWRITE:
@@ -327,7 +327,7 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXPAGES:
@@ -366,7 +366,7 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXRSS:
@@ -405,7 +405,7 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXVSIZE:
@@ -444,7 +444,7 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MINCPU:
@@ -484,7 +484,12 @@ void print_fields(slurmdb_step_rec_t *step)
 				tmp_uint64 = NO_VAL64;
 
 			field->print_routine(field,
-					     tmp_uint64,
+					     &tmp_uint64,
+					     (curr_inx == field_count));
+			break;
+		case PRINT_TRESA:
+			tmp_char = step->tres_alloc_str;
+			field->print_routine(field, tmp_char,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_TRESUIA:
@@ -558,7 +563,7 @@ void print_fields(slurmdb_step_rec_t *step)
 			break;
 		case PRINT_NTASKS:
 			field->print_routine(field,
-					     step->ntasks,
+					     &step->ntasks,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_PIDS:
