@@ -222,10 +222,8 @@ static void _layout_node_record(GtkTreeView *treeview,
 	char *upper = NULL, *lower = NULL;
 	GtkTreeIter iter;
 	uint16_t alloc_cpus = 0;
-	uint64_t alloc_memory = 0;
 	node_info_t *node_ptr = sview_node_info_ptr->node_ptr;
 	int idle_cpus = node_ptr->cpus_efctv;
-	char *node_alloc_tres = NULL;
 	GtkTreeStore *treestore =
 		GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
 	if (!treestore)
@@ -296,10 +294,7 @@ static void _layout_node_record(GtkTreeView *treeview,
 						 SORTID_FREE_MEM),
 				   tmp_cnt);
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_SUBCNT,
-				     NODE_STATE_ALLOCATED,
-				     &alloc_cpus);
+	alloc_cpus = node_ptr->alloc_cpus;
 	idle_cpus -= alloc_cpus;
 	convert_num_unit((float)alloc_cpus, tmp_cnt,
 			 sizeof(tmp_cnt), UNIT_NONE, NO_VAL,
@@ -321,15 +316,11 @@ static void _layout_node_record(GtkTreeView *treeview,
 						 SORTID_TRES_CONFIG),
 				   node_ptr->tres_fmt_str);
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_TRES_ALLOC_FMT_STR,
-				     NODE_STATE_ALLOCATED, &node_alloc_tres);
-
 	add_display_treestore_line(update, treestore, &iter,
 				   find_col_name(display_data_node,
 						 SORTID_TRES_ALLOC),
-				   node_alloc_tres ? node_alloc_tres : "");
-	xfree(node_alloc_tres);
+				   (node_ptr->alloc_tres_fmt_str ?
+				    node_ptr->alloc_tres_fmt_str : ""));
 
 	upper = node_state_string(node_ptr->node_state);
 	lower = str_tolower(upper);
@@ -390,11 +381,8 @@ static void _layout_node_record(GtkTreeView *treeview,
 						 SORTID_REAL_MEMORY),
 				   tmp_cnt);
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_MEM_ALLOC,
-				     NODE_STATE_ALLOCATED,
-				     &alloc_memory);
-	snprintf(tmp_cnt, sizeof(tmp_cnt), "%"PRIu64"M", alloc_memory);
+	snprintf(tmp_cnt, sizeof(tmp_cnt), "%"PRIu64"M",
+		 node_ptr->alloc_memory);
 	add_display_treestore_line(update, treestore, &iter,
 				   find_col_name(display_data_node,
 						 SORTID_USED_MEMORY),
@@ -500,7 +488,6 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 				GtkTreeStore *treestore)
 {
 	uint16_t alloc_cpus = 0, idle_cpus;
-	uint64_t alloc_memory;
 	node_info_t *node_ptr = sview_node_info_ptr->node_ptr;
 	char tmp_disk[20], tmp_cpus[20], tmp_idle_cpus[20];
 	char tmp_mem[20], tmp_used_memory[20];
@@ -508,7 +495,6 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 	char tmp_current_watts[50], tmp_ave_watts[50];
 	char tmp_version[50];
 	char *tmp_state_lower, *tmp_state_upper, *tmp_state_complete;
-	char *node_alloc_tres = NULL;
 
 	if (node_ptr->energy->current_watts == NO_VAL) {
 		snprintf(tmp_current_watts, sizeof(tmp_current_watts),
@@ -536,21 +522,14 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 			 sizeof(tmp_cpus), UNIT_NONE, NO_VAL,
 			 working_sview_config.convert_flags);
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_SUBCNT,
-				     NODE_STATE_ALLOCATED,
-				     &alloc_cpus);
-
+	alloc_cpus = node_ptr->alloc_cpus;
 	idle_cpus = node_ptr->cpus_efctv - alloc_cpus;
 	convert_num_unit((float)alloc_cpus, tmp_used_cpus,
 			 sizeof(tmp_used_cpus), UNIT_NONE, NO_VAL,
 			 working_sview_config.convert_flags);
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_MEM_ALLOC,
-				     NODE_STATE_ALLOCATED,
-				     &alloc_memory);
-	snprintf(tmp_used_memory, sizeof(tmp_used_memory), "%"PRIu64"M", alloc_memory);
+	snprintf(tmp_used_memory, sizeof(tmp_used_memory), "%"PRIu64"M",
+		 node_ptr->alloc_memory);
 
 	convert_num_unit((float)alloc_cpus, tmp_used_cpus,
 			 sizeof(tmp_used_cpus), UNIT_NONE, NO_VAL,
@@ -588,10 +567,6 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 		xfree(user_name);
 	}
 
-	select_g_select_nodeinfo_get(node_ptr->select_nodeinfo,
-				     SELECT_NODEDATA_TRES_ALLOC_FMT_STR,
-				     NODE_STATE_ALLOCATED, &node_alloc_tres);
-
 	/* Combining these records provides a slight performance improvement */
 	gtk_tree_store_set(treestore, &sview_node_info_ptr->iter_ptr,
 			   SORTID_ACTIVE_FEATURES, node_ptr->features_act,
@@ -632,8 +607,8 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 			   SORTID_STATE_COMPLETE, tmp_state_complete,
 			   SORTID_STATE_NUM, node_ptr->node_state,
 			   SORTID_THREADS,   node_ptr->threads,
-			   SORTID_TRES_ALLOC, node_alloc_tres ?
-			   node_alloc_tres : "",
+			   SORTID_TRES_ALLOC, node_ptr->alloc_tres_fmt_str ?
+			   node_ptr->alloc_tres_fmt_str : "",
 			   SORTID_TRES_CONFIG, node_ptr->tres_fmt_str,
 			   SORTID_USED_CPUS, tmp_used_cpus,
 			   SORTID_USED_MEMORY, tmp_used_memory,
@@ -644,7 +619,6 @@ static void _update_node_record(sview_node_info_t *sview_node_info_ptr,
 
 	xfree(tmp_state_complete);
 	xfree(tmp_state_lower);
-	xfree(node_alloc_tres);
 	return;
 }
 
@@ -1018,7 +992,7 @@ extern int update_active_features_node(GtkDialog *dialog, const char *nodelist,
 			global_row_count, nodelist);
 	if (!dialog) {
 		snprintf(tmp_char, sizeof(tmp_char),
-			 "Update Acitve Features for Node(s) %s?",
+			 "Update Active Features for Node(s) %s?",
 			 nodelist);
 
 		dialog = GTK_DIALOG(
@@ -1369,7 +1343,8 @@ extern int update_state_node(GtkDialog *dialog,
 				goto end_it;
 			}
 			if (uid_from_string(getlogin(),
-					    &node_msg->reason_uid) < 0)
+					    &node_msg->reason_uid) !=
+			    SLURM_SUCCESS)
 				node_msg->reason_uid = getuid();
 
 		}
