@@ -246,9 +246,6 @@ static void _layout_conf_ctl(GtkTreeStore *treestore,
 	char *select_title = "Select Plugin Configuration";
 	char *tmp_title = NULL;
 
-	if (cluster_flags & CLUSTER_FLAG_CRAY)
-		select_title = "\nCray configuration\n";
-
 	if (!slurm_ctl_conf_ptr)
 		return;
 
@@ -267,8 +264,6 @@ static void _layout_conf_ctl(GtkTreeStore *treestore,
 	_gtk_print_key_pairs(slurm_ctl_conf_ptr->cgroup_conf,
 			     "Cgroup Support", 0, treestore, &iter);
 
-	_gtk_print_key_pairs(slurm_ctl_conf_ptr->ext_sensors_conf,
-			     "External Sensors", 0, treestore, &iter);
 	_gtk_print_key_pairs(slurm_ctl_conf_ptr->mpi_conf,
 			     "MPI Plugins Configuration:", 0, treestore, &iter);
 
@@ -291,8 +286,9 @@ static void _layout_conf_dbd(GtkTreeStore *treestore)
 	time_t now = time(NULL);
 	char tmp_str[256], *user_name = NULL;
 	list_t *dbd_config_list = NULL;
+	void *db_conn = NULL;
 
-	/* first load accounting parms from slurm.conf */
+	/* first load accounting params from slurm.conf */
 	uint16_t track_wckey = slurm_get_track_wckey();
 
 	slurm_make_time_str(&now, tmp_str, sizeof(tmp_str));
@@ -343,10 +339,14 @@ static void _layout_conf_dbd(GtkTreeStore *treestore)
 	add_display_treestore_line(update, treestore, &iter,
 				   "TrackWCKey", tmp_str);
 
-	/* now load accounting parms from slurmdbd.conf */
+	/* now load accounting params from slurmdbd.conf */
 
-	/* second load slurmdbd.conf parms */
-	if (!(dbd_config_list = slurmdb_config_get(NULL)))
+	/* second load slurmdbd.conf params */
+	if (!(db_conn = slurmdb_connection_get(NULL)))
+		return;
+	dbd_config_list = slurmdb_config_get(db_conn);
+	slurmdb_connection_close(&db_conn);
+	if (!dbd_config_list)
 		return;
 
 	add_display_treestore_line_with_font(
@@ -630,8 +630,8 @@ extern void create_search_popup(GtkAction *action, gpointer user_data)
 			{G_TYPE_NONE, NODE_STATE_MIXED, "Mixed", true, -1},
 			{G_TYPE_NONE, NODE_STATE_NO_RESPOND,
 			 "No Respond", true, -1},
-			{G_TYPE_NONE, NODE_STATE_NET | NODE_STATE_IDLE,
-			 "PerfCTRs", true, -1},
+			{G_TYPE_NONE, NODE_STATE_IDLE | NODE_STATE_BLOCKED,
+			 "Blocked", true, -1},
 			{G_TYPE_NONE, NODE_STATE_IDLE | NODE_STATE_PLANNED,
 			 "Planned", true, -1},
 			{G_TYPE_NONE, NODE_STATE_POWERED_DOWN,
