@@ -51,7 +51,7 @@
 #include "src/common/list.h"
 #include "src/common/macros.h"
 #include "src/common/parse_time.h"
-#include "src/common/select.h"
+#include "src/interfaces/select.h"
 #include "src/common/slurmdb_defs.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
@@ -85,7 +85,6 @@ enum { JOB_PAGE,
        RESV_PAGE,
        BB_PAGE,
        NODE_PAGE,
-       FRONT_END_PAGE,
        SUBMIT_PAGE,
        ADMIN_PAGE,
        INFO_PAGE,
@@ -152,7 +151,7 @@ typedef struct  {
 } switch_record_bitmaps_t;
 
 typedef struct {
-	List col_list;
+	list_t *col_list;
 	bool def_col_list;
 	display_data_t *display_data;
 	char *page_name;
@@ -239,8 +238,8 @@ struct popup_info {
 	GtkWidget *event_box;
 	int force_refresh;
 	int full_grid;
-	List grid_button_list;
-	List multi_button_list;
+	list_t *grid_button_list;
+	list_t *multi_button_list;
 	GtkTable *grid_table;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -258,7 +257,7 @@ struct popup_info {
 
 typedef struct {
 	GtkWidget *button;
-	List button_list; /*list this grid_button exists in does not
+	list_t *button_list; /*list this grid_button exists in does not
 			   * need to be freed, should only be a
 			   * pointer to an existing list */
 	char *color;
@@ -291,7 +290,7 @@ typedef struct {
 
 typedef struct {
 	display_data_t *display_data;
-	List *button_list;
+	list_t **button_list;
 } signal_params_t;
 
 extern sview_config_t default_sview_config;
@@ -302,10 +301,10 @@ extern bool toggled;
 extern bool force_refresh;
 extern bool apply_hidden_change;
 extern bool apply_partition_check;
-extern List popup_list;
-extern List grid_button_list;
-extern List multi_button_list;
-extern List signal_params_list;
+extern list_t *popup_list;
+extern list_t *grid_button_list;
+extern list_t *multi_button_list;
+extern list_t *signal_params_list;
 extern bool global_entry_changed;
 extern bool global_send_update_msg;
 extern bool global_edit_error;
@@ -323,10 +322,8 @@ extern gint last_event_y;
 extern GdkCursor* in_process_cursor;
 extern char *sview_colors[];
 extern int sview_colors_cnt;
-extern int cluster_dims;
 extern uint32_t cluster_flags;
-extern List cluster_list;
-extern front_end_info_msg_t *g_front_end_info_ptr;
+extern list_t *cluster_list;
 extern job_info_msg_t *g_job_info_ptr;
 extern node_info_msg_t *g_node_info_ptr;
 extern partition_info_msg_t *g_part_info_ptr;
@@ -336,6 +333,7 @@ extern slurm_ctl_conf_info_msg_t *g_ctl_info_ptr;
 extern job_step_info_response_msg_t *g_step_info_ptr;
 extern topo_info_response_msg_t *g_topo_info_msg_ptr;
 extern switch_record_bitmaps_t *g_switch_nodes_maps;
+extern int g_switch_nodes_maps_count;
 extern popup_positioner_t main_popup_positioner[];
 extern popup_pos_t popup_pos;
 extern char *federation_name;
@@ -358,7 +356,6 @@ extern void close_tab(GtkWidget *widget, GdkEventButton *event,
 extern void create_config_popup(GtkAction *action, gpointer user_data);
 extern void create_create_popup(GtkAction *action, gpointer user_data);
 extern void create_dbconfig_popup(GtkAction *action, gpointer user_data);
-extern void create_daemon_popup(GtkAction *action, gpointer user_data);
 extern void create_search_popup(GtkAction *action, gpointer user_data);
 extern void change_refresh_popup(GtkAction *action, gpointer user_data);
 extern void change_grid_popup(GtkAction *action, gpointer user_data);
@@ -371,26 +368,27 @@ extern void destroy_grid_button(void *arg);
 extern grid_button_t *create_grid_button_from_another(
 	grid_button_t *grid_button, char *name, int color_inx);
 /* do not free the char * from this function it is static */
-extern void change_grid_color(List button_list, int start, int end,
+extern void change_grid_color(list_t *button_list, int start, int end,
 			      int color_inx, bool change_unused,
 			      enum node_states state_override);
-extern void change_grid_color_array(List button_list, int array_len,
+extern void change_grid_color_array(list_t *button_list, int array_len,
 				    int *color_inx, bool *color_set_flag,
 				    bool only_change_unused,
 				    enum node_states state_override);
-extern void highlight_grid(GtkTreeView *tree_view,
-			   int node_inx_id, int color_inx_id, List button_list);
-extern void highlight_grid_range(int start, int end, List button_list);
-extern void set_grid_used(List button_list, int start, int end,
+extern void highlight_grid(GtkTreeView *tree_view, int node_inx_id,
+			   int color_inx_id, list_t *button_list);
+extern void highlight_grid_range(int start, int end, list_t *button_list);
+extern void set_grid_used(list_t *button_list, int start, int end,
 			  bool used, bool reset_highlight);
-extern void get_button_list_from_main(List *button_list, int start, int end,
+extern void get_button_list_from_main(list_t **button_list, int start, int end,
 				      int color_inx);
-extern List copy_main_button_list(int initial_color);
-extern void put_buttons_in_table(GtkTable *table, List button_list);
+extern list_t *copy_main_button_list(int initial_color);
+extern void put_buttons_in_table(GtkTable *table, list_t *button_list);
 extern int get_system_stats(GtkTable *table);
-extern int setup_grid_table(GtkTable *table, List button_list, List node_list);
+extern int setup_grid_table(GtkTable *table, list_t *button_list,
+			    list_t *node_list);
 extern void sview_init_grid(bool reset_highlight);
-extern void sview_clear_unused_grid(List button_list, int color_inx);
+extern void sview_clear_unused_grid(list_t *button_list, int color_inx);
 extern void setup_popup_grid_list(popup_info_t *popup_win);
 extern void post_setup_popup_grid_list(popup_info_t *popup_win);
 
@@ -424,23 +422,6 @@ extern void admin_edit_accnt(GtkCellRendererText *cell,
 			     gpointer data);
 extern void specific_info_accnt(popup_info_t *popup_win);
 
-// front_end_info.c
-extern void admin_edit_front_end(GtkCellRendererText *cell,
-				 const char *path_string,
-				 const char *new_text, gpointer data);
-extern void cluster_change_front_end(void);
-extern GtkListStore *create_model_front_end(int type);
-extern void get_info_front_end(GtkTable *table, display_data_t *display_data);
-extern int  get_new_info_front_end(front_end_info_msg_t **info_ptr, int force);
-extern void popup_all_front_end(GtkTreeModel *model, GtkTreeIter *iter, int id);
-extern void refresh_front_end(GtkAction *action, gpointer user_data);
-extern void select_admin_front_end(GtkTreeModel *model, GtkTreeIter *iter,
-				  display_data_t *display_data,
-				  GtkTreeView *treeview);
-extern void set_menus_front_end(void *arg, void *arg2, GtkTreePath *path,
-				int type);
-extern void specific_info_front_end(popup_info_t *popup_win);
-
 // job_info.c
 extern void refresh_job(GtkAction *action, gpointer user_data);
 extern GtkListStore *create_model_job(int type);
@@ -462,7 +443,7 @@ extern void cluster_change_job(void);
 // node_info.c
 extern void refresh_node(GtkAction *action, gpointer user_data);
 /* don't destroy the list from this function */
-extern List create_node_info_list(node_info_msg_t *node_info_ptr,
+extern list_t *create_node_info_list(node_info_msg_t *node_info_ptr,
 				  bool by_partition);
 extern int update_active_features_node(GtkDialog *dialog, const char *nodelist,
 				      const char *old_features);
@@ -536,7 +517,8 @@ extern void make_options_menu(GtkTreeView *tree_view, GtkTreePath *path,
 extern GtkScrolledWindow *create_scrolled_window(void);
 extern GtkWidget *create_entry(void);
 extern void create_page(GtkNotebook *notebook, display_data_t *display_data);
-extern GtkTreeView *create_treeview(display_data_t *local, List *button_list);
+extern GtkTreeView *create_treeview(display_data_t *local,
+				    list_t **button_list);
 extern GtkTreeView *create_treeview_2cols_attach_to_table(GtkTable *table);
 extern void create_treestore(GtkTreeView *tree_view,
 			     display_data_t *display_data, int count,
@@ -624,6 +606,9 @@ extern void select_admin_common(GtkTreeModel *model, GtkTreeIter *iter,
 						     GtkTreePath *path,
 						     GtkTreeIter *iter,
 						     gpointer userdata));
+extern void set_column_width_fixed(GtkTreeView *tree_view,
+				   int sortid,
+				   int width);
 
 // defaults.c
 extern int load_defaults(void);
